/*
 *  Copyright (C) 2006:Michael Poppitz 2011: Mario Mauerer, Lukas Schrittwieser
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 *
 */
package org.sump.analyzer;
import java.net.*;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Vector;
import java.util.concurrent.LinkedBlockingQueue;


/**
 * Device provides access to the physical logic analyzer device.
 *  
 * @version 0.7
 * @author Michael "Mr. Sump" Poppitz
 *
 * @version 1.0
 * @author Lukas Schrittwieser
 * 
 * @version 2.0
 * @author Mario Mauerer / Lukas Schrittwieser
 */
public class Device extends Object {
	
	// Hardware configuratino parameters
	
	/** broadcast port used by the LA to advertise itself to the network */
	private final static int ADV_BROADCAST_PORT = 1024;
	
	/** how many samples are transmitted in one udp data packet */
	private final static int SAMPLES_PER_UDP_P	= 320;
	
	/** use internal clock */
	public final static int CLOCK_INTERNAL = 0;
	/** use external clock rising edge */
	public final static int CLOCK_EXTERNAL_RISING = 1;
	/** use external clock falling edge */
	public final static int CLOCK_EXTERNAL_FALLING = 2;
	
	/** Width (in bits) of the Hardware-Counter for the Divider:*/
	public final static int HW_Divider_Width = 16;
	
	/** reset analyzer */
	private final static int RESET = 0x00;
	/** arm trigger / run device */
	private final static int RUN = 0x01;
	/** ask for device id */
	private final static int ID = 0x02;
	/** enable data compression on hw */
	private final static int EN_COMPRESSION = 0x03;
	/** disable data compression on hw */
	private final static int DIS_COMPRESSION = 0x04;
	
	/** set trigger mask */
	private final static int SETTRIGMASK = 0xc0;
	/** set trigger value */
	private final static int SETTRIGVAL = 0xc1;
	/** set trigger configuration */
	private final static int SETTRIGCFG = 0xc2;
	/** set trigger delay:*/
	private final static int SETTRIGDELAY = 0xc3;
	/** select trigger unit:*/
	private final static int SELTRIGUNIT = 0xc4;
	/** set clock divider */
	private final static int SETDIVIDER = 0x80;
	/** set flags */
	private final static int SETFLAGS = 0x82;
	/** set the number of samples to take before the trigger */
	private final static int SET_PRE_TRIGGER_LEN =  0x84;
	/** set the number of samples to take after the trigger */
	private final static int SET_POST_TRIGGER_LEN = 0x83;
	/** set data port */
	private final static int SETDATAPORT = 0x85;
	/** tell the la we need a datagram resent */
	private final static int RESEND = 0x86;
	
	
	private final static int FLAG_DEMUX = 0x00000001;		// demultiplex
	private final static int FLAG_FILTER = 0x00000002;		// noise filter
	private final static int FLAG_EXTERNAL = 0x00000040;	
	private final static int FLAG_INVERTED = 0x00000080;	

	// TODO: This should be extracted from the information provided by the device	
	private final static int CLOCK = 200000000;	// device clock in Hz
	private final static int TRIGGER_STAGES = 8; // number of trigger stages
	
		
	/**
	 * Creates a device object.
	 *
	 */
	public Device() {
		triggerMask = new int[TRIGGER_STAGES];
		triggerValue = new int[TRIGGER_STAGES];
		triggerConfig = new int[TRIGGER_STAGES];
		triggerDelay = new int[TRIGGER_STAGES];
		for (int i = 0; i < TRIGGER_STAGES; i++) {
			triggerMask[i] = 0;
			triggerValue[i] = 0;
			triggerConfig[i] = 0;
			triggerDelay[i] = 0;
		}
		
		triggerEnabled = false;
		filterEnabled = false;
		demux = false;
		setClockSource(CLOCK_INTERNAL);
		divider = 0;
		ratio = 0.5;
		size = 512;
		enabledGroups = new boolean[4];
		setEnabledChannels(-1); // enable all channels
		
		useCompression = false;
		percentageDone = -1;
		stop();
		
		//port = null;
		cmdSock = null;
		// create a decompressor instance for later use
		decompressor = new LZRWDecoder();
	}

	/**
	 * Sets the number of samples to obtain when started.
	 * 
	 * @param size number of samples, must be between 4 and 256*1024
	 */
	public void setSize(int size) {
		this.size = size;
	}
	
	/**
	 * Sets the ratio for samples to read before and after started.
	 * @param ratio	value between 0 and 1; 0 means all before start, 1 all after
	 */
	public void setRatio(double ratio) {
		this.ratio = ratio;
	}

	/**
	 * Set the sampling rate.
	 * All rates must be a divisor of 400.000.000.
	 * Other rates will be adjusted to a matching divisor.
	 * 
	 * @param rate		sampling rate in Hz
	 */
	public void setRate(int rate) throws IOException 
	{
		if (rate > CLOCK) {
			demux = true;
			divider = (2 * CLOCK / rate) - 1;
		} else {
			demux = false;
			divider = (CLOCK / rate) - 1;
			
			if(divider > 1<<HW_Divider_Width)
			{
				throw new IOException("Frequency not applicable");
			}	
				
			
		}
	}
	
	/** 
	 * Configures a trigger Stage
	 * @param stage trigger stage selected
	 * @param config config-integer
	 * @param mask trigger mask
	 * @param value trigger values
	 * @param delay for the trigger
	 */
	public void setTriggerStage(int stage, int config, int mask, int value, int delay)
	{
		triggerMask[stage] = mask;
		triggerValue[stage] = value;
		triggerConfig[stage] = config;
		triggerDelay[stage] = delay;
	}

	/**
	 * Sets wheter or not to enable the trigger.
	 * @param enable <code>true</code> enables the trigger, <code>false</code> disables it.
	 */
	public void setTriggerEnabled(boolean enable) {
		triggerEnabled = enable;
	}

	/**
	 * Sets wheter or not to enable the noise filter.
	 * @param enable <code>true</code> enables the noise filter, <code>false</code> disables it.
	 */
	public void setFilterEnabled(boolean enable) {
		filterEnabled = enable;
	}
	
	public void setCompressionEnabled (boolean enable) {
		useCompression = enable;
	}
	
	/**
	 * Set enabled channels.
	 * @param mask bit map defining enabled channels
	 */
	public void setEnabledChannels(int mask) {
		enabledChannels = mask;
		// determine enabled groups
		for (int i = 0; i < 4; i++)
			enabledGroups[i] = ((enabledChannels  >> (8 * i)) & 0xff) > 0;
	}

	/**
	 * Sets the clock source to use.
	 * @param source can be any CLOCK_ property of this class
	 */
	public void setClockSource(int source) {
		clockSource = source;
	}

	/**
	 * Get the maximum sampling rate available.
	 * @return maximum sampling rate
	 */
	public int getMaximumRate() {
		return (2 * CLOCK);
	}

	/**
	 * Returns the current trigger mask.
	 * @param stage trigger stage to read mask from
	 * @return current trigger mask
	 */
	public int getTriggerMask(int stage) {
		return (triggerMask[stage]);
	}

	/**
	 * Returns the current trigger value.
	 * @param stage trigger stage to read value from
	 * @return current trigger value
	 */
	public int getTriggerValue(int stage) {
		return (triggerValue[stage]);
	}

	/**
	 * Returns the current clock source.
	 * @return the clock source currently used as defined by the CLOCK_ properties
	 */
	public int getClockSource() {
		return (clockSource);
	}
	
	/**
	 * Returns the currently enabled channels.
	 * @return bitmask with enabled channels represented as 1
	 */
	public int getEnabledChannels() {
		return (enabledChannels);
	}

	/**
	 * Returns wether or not the trigger is enabled.
	 * @return <code>true</code> when trigger is enabled, <code>false</code> otherwise
	 */
	public boolean isTriggerEnabled() {
		return (triggerEnabled);
	}

	/**
	 * Returns wether or not the noise filter is enabled.
	 * @return <code>true</code> when noise filter is enabled, <code>false</code> otherwise
	 */
	public boolean isFilterEnabled() {
		return (filterEnabled);
	}

	/**
	 * Returns wether or not the noise filter can be used in the current configuration.
	 * @return <code>true</code> when noise filter is available, <code>false</code> otherwise
	 */
	public boolean isFilterAvailable() {
		return (clockSource == CLOCK_INTERNAL);
	}

	/**
	 * Returns wether the device supports data compression to speed up transfer times.
	 * @return <code>true</code> when compression is supported, <code>false</code> otherwise 
	 */
	 public boolean isCompressionAvailable() {
		return true;
	 }
	
	/**
	 * Returns the number of available trigger stages.
	 * @return number of available trigger stages
	 */
	public int getTriggerStageCount() {
		return (TRIGGER_STAGES);
	}

	/**
	 * Returns the number of available channels in current configuration.
	 * @return number of available channels
	 */
	public int getAvailableChannelCount() {
		if (demux && clockSource == CLOCK_INTERNAL)
			return (16);
		else
			return (32);
	}

	/**
	 * Returns wether or not the device is currently running.
	 * It is running, when another thread is inside the run() method reading data from the serial port.
	 * @return <code>true</code> when running, <code>false</code> otherwise
	 */
	public boolean isRunning() {
		return (running);
	}

	/**
	 * Returns the percentage of the expected data that has already been read.
	 * The return value is only valid when <code>isRunning()</code> returns <code>true</code>. 
	 * @return percentage already read (0-100)
	 */
	public int getPercentage() {
		return (percentageDone);
	}

	/**
	 * Gets a string array containing the names all available serial ports.
	 * @return array containing serial port names
	 */
	static public String[] getPorts() {

		Vector<InetSocketAddress> devices = new Vector<InetSocketAddress>();
		int devicePort;
		InetAddress deviceAddr;	
		boolean running=true;
		// we try some time
		long stopTick = System.currentTimeMillis() + 1500;
		DatagramSocket advBcS;
		try {
			advBcS = new DatagramSocket(ADV_BROADCAST_PORT);
			advBcS.setSoTimeout(20);	// set the timeout in ms which we will wait for a packet
		} catch (Exception e) {
			System.err.println("Can't create a socket for advertisement datagrams: "+e);
			e.printStackTrace();
			return new String[0];			
		}
		while (running)
		{
			try {
				// create a packet to receive advertisement broadcasts (udp datagrams) with 100 bytes max.
				DatagramPacket advBcP = new DatagramPacket(new byte[100], 100);
				
				advBcS.receive(advBcP);	// try to get an advertisement broadcast from the LA
				// parse it 
				byte [] d = advBcP.getData();
				int o = advBcP.getOffset();
				// check the magic code in the first six bytes
				if ((d[o+0]!='S') || (d[o+1]!='u') || (d[o+2]!='m') || (d[o+3]!='p') || (d[o+4]!='L') || (d[o+5]!='A'))	{
					System.err.println("Magic value of advertisement broadcast incorrect");
				}
				devicePort = (d[o+10]<<8) + d[o+11];
				deviceAddr = InetAddress.getByAddress(java.util.Arrays.copyOfRange(d,o+12,o+16));
				System.out.println("Got Advertisement from "+deviceAddr+" at port "+devicePort);
				InetSocketAddress sockAddr = new InetSocketAddress(deviceAddr, devicePort); 
				if (!devices.contains(sockAddr))
					devices.add(sockAddr);
	
			} 
			catch (SocketTimeoutException e)
			{
				// we received no packet
				// we try again until we run out of time
				if (System.currentTimeMillis() > stopTick)
					running = false;
			}
			catch (Exception e) {
				System.err.println("Can't receive advertisement broadcast: "+e);
				e.printStackTrace();
				return new String[0];
			}
		}
		// translate the addresses and port numbers to strings
		String [] res = new String[devices.size()];
		Enumeration<InetSocketAddress> it = devices.elements();
		for (int i=0; it.hasMoreElements(); i++)
			res[i] = ((InetSocketAddress)it.nextElement()).toString();
		
		return res;
	}

	/**
	 * Connects to the device at the given IP address and port number.
	 * The method will try to open a TCP stream.
	 * <p>
	 * If the device is already attached to a port this port will be
	 * detached automatically. It is therefore not necessary to manually
	 * call <code>detach()</code> before reattaching.
	 *
	 * @param portName		IP address and TCP port number of the device specified as /a.b.c.d:e where a.b.c.d is the ip and e is the port
	 * @return				<code>true</code> when the port has been assigned successfully;
	 * 						<code>false</code> otherwise.
	 */
	public boolean attach(String portName) {
		
		//boolean found = false;
		
		InetAddress deviceAddr;
		int			devicePort;
		
		try {
			detach();

			// the port name is of the form /a.b.c.d:y
			if ((portName == null) || (portName.length() == 0))	{
				System.err.println("illegal portName");
				return false;
			}
			
			// first: remove the leading /
			if (portName.charAt(0) == '/')
				portName = portName.substring(1);
			
			// split at the colon
			String [] parts = portName.split(":");
			if (parts.length != 2) {
				System.err.println("port: "+portName+" splits into "+parts.length+" parts.");
				return false;
			}
			//System.out.println("splitted into "+parts[0]+" and "+parts[1]);			
			
			// extract ip and port from passed string
			deviceAddr = InetAddress.getByName(parts[0]);
			devicePort = Integer.valueOf(parts[1]);
			
			System.out.println("Trying to connect to "+deviceAddr+" at port "+devicePort);

			// connect to the LA command interface
			cmdSock = new Socket(deviceAddr, devicePort);
			// tell java not to group small segments into larger ones as we have very little memory 
			// for this on the device
			//cmdSock.setTcpNoDelay(true);	// Note by Lukas: fixed in firmware, not needed any more
			inputStream = cmdSock.getInputStream();
			outputStream = cmdSock.getOutputStream();
			
			// create a udp socket for reception of sample data
			dataSock = new DatagramSocket();
			// tell the LA the port number of our data reception socket
			int dataPort = dataSock.getLocalPort();
			// connect the port to only accept packet from the LA and the data port
			// Note: the same port number is used as source and destination port
			dataSock.connect(deviceAddr,dataPort);	
			System.out.println("data port is "+dataPort);
			// tell the LA the port number for UDP datagrams
			sendCommand(SETDATAPORT, dataPort);
		} catch(Exception E) {
			System.err.println("Can't connect to device");
			E.printStackTrace(System.out);
			return (false);
		}		
		return (true);
	}
	
	/**
	 * Detaches the currently attached port, if one exists.
	 * This will close the serial port.
	 *
	 */
	public void detach() {
		if (cmdSock != null) {
			try {
				// try to make sure device is reset (see run() for loop explanation)
				if (outputStream != null) {
					for (int i = 0; i < 5; i++)
						sendCommand(RESET);
					outputStream.close();
				}
				if (inputStream != null)
					inputStream.close();
				cmdSock.close();				
				cmdSock = null;
				inputStream = null;
				outputStream = null;
			} catch (IOException e) { /* don't care */ }
		}		
	}
	
	/**
	 * Sends a long command to the given stream.
	 * 
	 * @param opcode	one byte operation code
	 * @param data		four byte data portion
	 * @throws IOException if writing to stream fails
	 */
	private void sendCommand(int opcode, int data) throws IOException {
		byte[] raw = new byte[5];
		int mask = 0xff;
		int shift = 0;
		
		raw[0] = (byte)opcode;
		for (int i = 1; i < 5; i++) {
			raw[i] = (byte)((data & mask) >> shift);
			mask = mask << 8;
			shift += 8;
		}
	
		outputStream.write(raw);
	}
	
	/**
	 * Sends a short command to the given stream.
	 * 
	 * This method is intended to be used for short commands, but can also be called
	 * with long command opcodes if the data portion is to be set to 0.
	 * 
	 * @param opcode	one byte operation code
	 * @throws IOException if writing to stream fails
	 */
	private void sendCommand(int opcode) throws IOException {
		byte raw = (byte)opcode;
		outputStream.write(raw);
	}

	/**
	 * Reads a integer (32bits) from stream and compiles them into a single integer.
	 * 
	 * @param channels number of channels to read (must be multiple of 8)
	 * @return	integer containing four bytes read
	 * @throws IOException if stream reading fails
	 */
	private int readInteger() throws IOException, InterruptedException {
		outputStream.flush();
		int value = 0;
		for (int i = 0; i < 4; i++) {
			int v = inputStream.read();
			if (v < 0 || Thread.interrupted())
				throw new InterruptedException("Data readout interrupted.");
			value |= v << (8 * i);
		}

		return (value);
	}

	/** 
	 * Parses an received datagram and copies the sample data into the passed sample buffer
	 * @param packet		The received udp datagram packet
	 * @param buffer		The buffer to store the samples
	 * @return 				The buffer index of the first sample received with this datagram
	 * @throws IOException 	When the datagram is missformed
	 */
	private int parseDatagram(DatagramPacket packet, int [] buffer) throws IOException 
	{		
		byte [] rxBuf = packet.getData();
		// the first 4 bytes are the buffer offset of the first sample
		// Note: java doesn't support unsigned datatypes, so we need some tricks here
		int ind = 0;
		for (int i=3; i>=0; i--) {	
			ind *= 256;
			if (rxBuf[i]<0)
				ind += (256+rxBuf[i]);
			else
				ind += rxBuf[i];
		}
		//System.out.println("found packet starting at "+ind);
		// the index has to be a multiple of the number of samples per packet
		if (ind%SAMPLES_PER_UDP_P != 0)	{
			System.err.println("index is not a multiple of "+SAMPLES_PER_UDP_P+" (samples per packet)");
			throw new IOException("received broken data packet: index position failure");	
		}			
		// the next two bytes are the number of samples in this packet
		int len = rxBuf[5];	
		if (len<0)
			len += 256;
		len <<= 8;
		if (rxBuf[4] < 0)
			len += 256+rxBuf[4];
		else
			len += rxBuf[4];
		
		// check the length of the packet
		// NOTE: there are always 4 bytes per sample right now, this might be changed
		if ((packet.getLength() != (len*4+6)) && (!useCompression)) {
			System.err.println("received broken data packet: length mismatch (ind="+ind+", len="+len+", packet.getLength="+packet.getLength()+")");
			throw new IOException ("received broken data packet: length mismatch (ind="+ind+", len="+len+", packet.getLength="+packet.getLength()+")");
		}
		
		// initiate the decompressor if the frame is compressed
		if (useCompression)
			decompressor.loadCompressedData(rxBuf, 6);	// compressed data starts in byte 6
		
		// copy it into the buffer
		// NOTE: there are always 4 bytes per sample right now, this might be changed
		for (int i=0; i<len; i++) {
			int s=0;	
			for (int j=0; j<4; j++) {
				int b = 0;
				if (useCompression)
				{
					b = decompressor.decodeNextByte();
					if (b < 0)
						throw new IOException("encountered end of data too early (data corrupt)");
				}
				else	
					b = rxBuf[6+(i*4)+j];
				if (b<0)
					b += 256;
				s += b<<(8*j);	
			}
			// in demux mode we in fact receive two samples packed into one, so we demux them here
			if (demux) {
				// Note: the higher 16 bits hold the older (ie taken earlier sample)
				buffer[2*(ind+i)] = ((s>>16) & 0x0000FFFF);
				buffer[2*(ind+i)+1] = (s & 0x0000FFFF);
			} else {
				buffer[ind+i] = s;
			}
		}
	
		if (useCompression)
		{
			// check that there are no extra bytes in the compressed datagram
			int b = decompressor.decodeNextByte();
			while (b != -1)
			{
				System.err.println("found extra byte in datagram x"+String.format("%08x",ind)+": "+b);
				b = decompressor.decodeNextByte();
			}				
		}
	
		return ind;
	}
	
	/**
	 * Sends the configuration to the device, starts it, reads the captured data
	 * and returns a CapturedData object containing the data read as well as device configuration information.
	 * @return captured data
	 * @throws IOException when writing to or reading from device fails
	 * @throws InterruptedException if a read time out occurs after trigger match or stop() was called before trigger match
	 */
	public CapturedData run() throws IOException, InterruptedException {
		
		running = true;
		percentageDone = 0;

		if ((outputStream == null) || (inputStream == null))
			throw new IOException("no device attached");

		// send reset 5 times because in worst case first 4 are interpreted as data of long command
		for (int i = 0; i < 6; i++)
			sendCommand(RESET);
				
		// check if device is ready
		sendCommand(ID);
		int id = 0;
		try {
			id = readInteger();
		} catch (Exception e) { /* don't care */ }
		System.out.println("Device ID: 0x" + Integer.toHexString(id));
		if (id != 0x32414c53) { // SLA2
			System.out.println("id doesn't match: "+Integer.toString(id,16));
			throw new IOException("Device not found.");
		}	
		
		// configure device
			
		//Set the Trigger:
		for(int i = 0; i<TRIGGER_STAGES; i++)
		{
			sendCommand(SELTRIGUNIT,i); //select the stage to configure
			sendCommand(SETTRIGMASK,triggerMask[i]); //set the mask
			sendCommand(SETTRIGVAL,triggerValue[i]); //set the value
			sendCommand(SETTRIGCFG,triggerConfig[i]); //set the config-Flags
			sendCommand(SETTRIGDELAY,triggerDelay[i]); //set the Delay
		}
				
		sendCommand(SETDIVIDER, divider);
		if (useCompression)
			sendCommand(EN_COMPRESSION);
		else
			sendCommand(DIS_COMPRESSION);

		int flags = 0;
		if (clockSource == CLOCK_EXTERNAL_RISING || clockSource == CLOCK_EXTERNAL_FALLING) {
			flags |= FLAG_EXTERNAL;
			if (clockSource == CLOCK_EXTERNAL_FALLING)
				flags |= FLAG_INVERTED;
		}
		if (demux) 
			flags |= FLAG_DEMUX;
		if (filterEnabled && isFilterAvailable())
			flags |= FLAG_FILTER;
				
		System.out.println("Flags: " + Integer.toString(flags, 2));
		sendCommand(SETFLAGS, flags);
		
		// check if data needs to be multiplexed
		int channels;
		
		//System.out.println("ratio: "+ratio);
		//System.out.println("size: "+size);
		// ratio is the amount of samples to take after trigger compared to the total amount of samples
		int postTriggerLen = (int)(size * ratio);
		int preTriggerLen = size - postTriggerLen;
		int samples = preTriggerLen + postTriggerLen;
		
		if (demux) {
			channels = 16;
			// demultiplexing effectively doubles the number of samples so we request only half the number
			preTriggerLen /= 2; 
			postTriggerLen /= 2;
			sendCommand(SET_PRE_TRIGGER_LEN, preTriggerLen);
			sendCommand(SET_POST_TRIGGER_LEN, postTriggerLen);
			// update number of samples. It might change as we can only have even numbers now
			samples = 2*(preTriggerLen + postTriggerLen);
			preTriggerLen *= 2;		// this effectively rounds pre and postTriggerLen down to even numbers
			postTriggerLen *= 2;
			
		} else {
			channels = 32;
			sendCommand(SET_PRE_TRIGGER_LEN, preTriggerLen);
			sendCommand(SET_POST_TRIGGER_LEN, postTriggerLen);
		}
		
		int[] buffer = new int[samples];
		int numPackets;
		if (demux)
			numPackets = (int)(java.lang.Math.ceil(((double)(samples/2))/SAMPLES_PER_UDP_P));	
		else
			numPackets = (int)(java.lang.Math.ceil(((double)samples)/SAMPLES_PER_UDP_P));
		System.out.println("we expect "+numPackets+" packets from the LA");
		// flag to store which packet were received
		boolean [] rxFlag = new boolean[numPackets];
		for (int i=0; i<numPackets; i++)
			rxFlag[i] = false;
		
		// create a receiver thread
		UdpReceiver rxTh[] = new UdpReceiver[2];
		for (int i=0; i<rxTh.length; i++)
		{
			rxTh[i] = new UdpReceiver(dataSock);
			rxTh[i].start();
		}
		Thread.sleep(1);
		
		// tell the hardware to start
		sendCommand(RUN);
		outputStream.flush();
		
		int rxPacketCnt=0;
		boolean done=false;
		while(!done)
		{
			// wait for a datagram to arrive
			DatagramPacket packet = UdpReceiver.getDatagram();
			// a zero length packet is inserted into the fifo by the receiver thread if a timeout occures
			if (packet.getLength() == 0)
			{	
				System.out.println("A timeout occured");
				break;
			}
			// parse the datagram and store samples into sampling buffer			
			int ind;
			try
			{
				ind = parseDatagram(packet, buffer);
			} 
			catch (IOException e)
			{
				System.err.println("parsing error: "+e);
				e.printStackTrace();
				throw new IOException ("Parsing error");
			}
			// mark this packet as received
			rxFlag[ind/SAMPLES_PER_UDP_P] = true;
			// count the received packet and update progress information
			rxPacketCnt++;
			percentageDone = 100*rxPacketCnt / numPackets; 
			//System.out.println(""+rxPacketCnt+"/"+numPackets+"  "+percentageDone+"%");
			// check if we have got all packets
			done = true;	// assume we are done
			for (int i=0; (i<numPackets) && done; i++)
				done = done && rxFlag[i];	// if one rxFlag is not set done goes back to false
		}
		int retryCnt = 0;
		while (!done)
		{	
			// a timeout occured during reception, this might happen if a datagram was lost
			// check which datagram we need
			int pNum;
			for (pNum=0; pNum<numPackets; pNum++)
				if (!rxFlag[pNum])
					break;
				
			// calculate the sample number of the first sample in this datagram
			int ind = SAMPLES_PER_UDP_P * pNum;
			//System.out.println("requesting retransmission of packet "+pNum+" starting with sample "+ind);	
			dataSock.setSoTimeout(500);
			int len = 1500;
			DatagramPacket dataPacket = new DatagramPacket(new byte[len], len);
			// receive a packet
			
			sendCommand(RESEND, ind);
			//Thread.sleep(2);
			outputStream.flush();
			try	{ 
				dataSock.receive(dataPacket);
				// tell the hardware we need this datagram again
				retryCnt = 0;
				
				// parse it
				if (parseDatagram(dataPacket, buffer) != ind)
					throw new IOException("Received broken data packet: resent packet has wrong buffer index");	
			
			}
			catch (SocketTimeoutException toe) {
				retryCnt++;
				if (retryCnt == 5)
					throw new IOException("Timeout during data retransmission: LA might be unavailable");
			}
			catch (IOException e) {
				System.err.println("parsing error: "+e);
				e.printStackTrace();
				retryCnt++;
				if (retryCnt == 5)
					throw new IOException("Multiple IOErrors during one datagram, this indicates a major malfunction");
			}
			
			// mark it as received
			rxFlag[ind/SAMPLES_PER_UDP_P] = true;
			
			// check if we have got all packets
			done = true;	// assume we are done
			for (int i=0; (i<numPackets) && done; i++)
				done = done && rxFlag[i];	// if one rxFlag is not set done goes back to false
			// update progress information
			rxPacketCnt++;
			percentageDone = 100*rxPacketCnt / numPackets; 
		}
		
		//validate(numPackets, buffer);
		
		// tell the threads we don't need them anymore
		for (int i=0; i<rxTh.length; i++)
		{
			rxTh[i].done();
		}
		
		// collect additional information about CapturedData
		int pos = CapturedData.NOT_AVAILABLE;
		if (triggerEnabled)
			pos = preTriggerLen;
		
		int rate = CapturedData.NOT_AVAILABLE;
		if (clockSource == CLOCK_INTERNAL)
		{
			rate = demux ? (2*CLOCK)/(divider + 1) : CLOCK/(divider + 1);
		}

		return (new CapturedData(buffer, pos, rate, channels, enabledChannels));
	}
	
	private void validate(int numPackets, int [] buf) throws IOException
	{
		int [] backBuf = new int[buf.length];
		sendCommand(DIS_COMPRESSION);
		useCompression = false;
		
		dataSock.setSoTimeout(500);
		
		// retransmit all datagrams and compare them with the data from the previous transfer
		for (int pNum=0; pNum < numPackets; pNum++)
		{	
			// calculate the sample number of the first sample in this datagram
			int ind = SAMPLES_PER_UDP_P * pNum;
			int len = 1500;
			DatagramPacket dataPacket = new DatagramPacket(new byte[len], len);
			// receive a packet
			sendCommand(RESEND, ind);
			//Thread.sleep(2);
			outputStream.flush();
			try	{ 
				dataSock.receive(dataPacket);
				
				
				// parse it
				if (parseDatagram(dataPacket, backBuf) != ind)
					throw new IOException("Received broken data packet: resent packet has wrong buffer index");	
			
			}
			catch (SocketTimeoutException toe) {
				throw new IOException("Timeout during data retransmission: LA might be unavailable");
			}
			catch (IOException e) {
				System.err.println("parsing error: "+e);
				e.printStackTrace();
				throw new IOException("Multiple IOErrors during one datagram, this indicates a major malfunction");
			}
		}
		// compare buffer and back buffer
		for (int i=0; i<buf.length; i++)
		{
			if (buf[i] != backBuf[i])
				throw new IOException("verify failed at index "+i+" buf: x"+String.format("%08x",buf[i])+"    backBuf: x"+String.format("%08x",backBuf[i]));
		}
		System.out.println("verify passed");
	}
	
	/**
	 * Informs the thread in run() that it is supposed to stop reading data and return.
	 *
	 */
	public void stop() {
		running = false;
	}
	
	LZRWDecoder decompressor;	// decompresses compressed sample data
	
	private Socket cmdSock;
	private DatagramSocket dataSock;
	private InputStream inputStream;
	private OutputStream outputStream;
		
	private boolean running;
	private int percentageDone;
	
	private int clockSource;
	private boolean demux;
	private boolean filterEnabled;
	//trigger: 
	private boolean triggerEnabled; //says whether the trigger runs or not
	private int triggerMask[]; //mask for the stages
	private int triggerValue[]; //trigger-values for the stages
	private int triggerConfig[]; //Config-Flags for the stages
	private int triggerDelay[]; //Trigger-Delay-Value for the stages

	private boolean useCompression;
	private int enabledChannels;
	private boolean enabledGroups[];
	
	private int divider;
	private int size;
	private double ratio;
}


/**
 * Embedded class to receive udp data datagrams. 
 * The reception of udp datagrams needs it's own thread as datagrams
 * are not buffered on host system. This means we have to read as fast as possible,
 * otherwise some of them might get lost. 
 */
class UdpReceiver extends Thread
{
	/**
	 * Fifo to transfer the received packets to the processes thread
	 */
	static LinkedBlockingQueue<DatagramPacket> receivedDatagrams = null;
	
	/** 
	 * Used to stop the receiver when all data was received
	 */
	boolean running; 
	
	/**
	 * The socket which is used to receive datagrams
	 */
	DatagramSocket socket;
	
	/**
	 * Constructor
	 * @param sock	The socket to receive udp datagrams
	 */
	public UdpReceiver(DatagramSocket sock)
	{
		// create a fifo if it does not exist yet
		if (receivedDatagrams == null)
			receivedDatagrams = new LinkedBlockingQueue<DatagramPacket>();
		socket = sock;
		setPriority(Thread.MAX_PRIORITY);
	}
	
	/**
	 * Blocks until a datagram was received and returns it
	 * Returns null if a timeout was encountered
	 */
	public static DatagramPacket getDatagram() throws InterruptedException
	{
		return receivedDatagrams.take();
	}
	
	/** 
	 * Tells the thread to stop upon the next reception timeout
	 */
	public void done()
	{
		running = false;
	}
			
	public void run()
	{
		receivedDatagrams.clear();
		running = true;
		assert(socket != null);
		DatagramPacket dataPacket;
		int len = 1500;	// max length of a datagram
		while (running)
		{		
			try 
			{
				dataPacket = new DatagramPacket(new byte[len], len);
				// receive a packet
				socket.receive(dataPacket);
				// after the first packet we use a timeout for the others
				socket.setSoTimeout(100);
				// add this datagram to fifo
				receivedDatagrams.put(dataPacket);
			}
			catch (SocketTimeoutException toe)
			{
				// we got a timeout -> inform receiver by inputing a empty packet into the fifo (if it awaits more data)
				if (running)
				{
					dataPacket = new DatagramPacket(new byte[1],0);
					try { 
						receivedDatagrams.put(dataPacket); } 
					catch(Exception e) { 
						/* this is a fault in a fault handler -> what should we do? */
						System.err.println("Fatal error in receiver thread: can't report timeout");
						return;
					}
				}						
				running = false;
			}
			catch (Exception e)
			{
				// we got some other exception
				System.err.println("receiver exception: "+e);
				e.printStackTrace();
				running = false;
			}
		}				
	}
}
