/*
 *  Copyright (C) 2012 Lukas F Schrittwieser
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 *
 */

package org.sump.analyzer.tools; 
 
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener; 
import java.awt.GridLayout;
import java.awt.BorderLayout;
import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.table.DefaultTableModel;
import javax.swing.JPanel;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


class CsvExportableTableModel extends DefaultTableModel 
{
	/**
	 * This constructor is exactly the same as for DefaultTableModel
	 */
	public CsvExportableTableModel (Object[] columnNames, int rowCount) 
	{
		super(columnNames, rowCount);
	}
	
	
	/**
	 * Presents a dialog to the user which allows to select the columns to be saved.
	 * @param owner The frame to be set as owner for the dialog shown to the user.
	 * @param f 	The file in which the table model data will be saved
	 */
	public boolean exportToCsv (Frame owner, File f)
	{
		// first step: show a dialog so the user can select the columns to be saved
		ColumnSelectDialog dialog = new ColumnSelectDialog(owner, this);
		dialog.setVisible(true);
		
		// check for user aborts
		if (dialog.getClickedButton() != dialog.BUTTON_OK)
			return false;
		
		// get the results
		boolean [] columnSel = dialog.getResults();
		
		try
		{
			BufferedWriter bw = new BufferedWriter(new FileWriter(f));
			// loop all rows in the model
			for (int r=0; r<getRowCount(); r++)
			{
				boolean lineHasData = false;
				
				// check if at least on of the columns in this line has valid data to be stored
				for (int c=0; c<getColumnCount(); c++)
				{
					if ((getValueAt(r,c)!=null) && (columnSel[c]))
						lineHasData = true;
				}
				
				if (!lineHasData)
					continue;	// no data to print -> go to next line
				
				boolean first = true;
				for (int c=0; c<getColumnCount(); c++)
				{
					if (!columnSel[c])
						continue;
					// if this is not first column we output a comma
					if (first)
						first = false;
					else
						bw.write(',');
					Object o = getValueAt(r,c);
					if (o == null)
						continue;
					bw.write(o.toString());
				}
				bw.write('\n');				
			}
			bw.close();		
		} catch (IOException e)
		{
			System.out.println("can't write csv file: " + e);	
		}
		return true;
	}
	
	
	// inner class implementing the dialog shown to the user
	class ColumnSelectDialog extends JDialog implements ActionListener
	{
		
		/** Indicates that the user has not clicked a dialog button (ie closed with x symbol of window) */
		final static int BUTTON_NONE	= 0;
		
		/** Indicates that the OK button was clicked */
		final static int BUTTON_OK 		= 1;
		
		/** Indicates that the user clicked the cancel button */
		final static int BUTTON_CANCEL	= 2;
		
		/**
		 * Creates the dialog using the columns contained in the model passed as paramter
		 * @param owner 	The frame to be set as the dialogs owner
		 * @param model 	A table model containing the column names which will be shown to the user
		 */
		public ColumnSelectDialog(Frame owner, DefaultTableModel model)
		{
			super(owner, "Setup CVS Export", true);
			
			clickedButton = BUTTON_NONE;
			
			setLayout(new BorderLayout());
			
			// create a panel for all the check boxes
			JPanel panBox = new JPanel();
			panBox.setLayout(new GridLayout(0,2,5,5));
			panBox.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder("Select Columns"),
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			
			setLayout(new GridLayout(0,1,5,5));
			// create all the check boxes
			columnBoxes = new JCheckBox[model.getColumnCount()];
			for (int i=0; i<model.getColumnCount(); i++)
			{
				columnBoxes[i] = new JCheckBox(model.getColumnName(i), true); // boxes are checked by default
				panBox.add(columnBoxes[i]);
			}
			add(panBox, BorderLayout.CENTER);
				
			// create a panel for the buttons
			JPanel panBut = new JPanel();
			panBut.setLayout(new GridLayout(1,0,5,0));
			// create buttons
			JButton okButton = new JButton("Ok");
			okButton.addActionListener(this);
			panBut.add(okButton);
			JButton cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(this);
			panBut.add(cancelButton);
			// add panel to dialog
			add(panBut, BorderLayout.SOUTH);			
			
			pack();
		}
		
		public void setVisible (boolean v)
		{
			// if the dialog is shown to the user we clear the stored status information about clicked button
			if (v)
				clickedButton = BUTTON_NONE;
			super.setVisible(v);
		}
		
		/**
		 * This is the action listener for user interaction with the tools dialog.
		 * It performs the necessary actions for the buttons.
		 */		
		public void actionPerformed(ActionEvent e)
		{
			if (e.getActionCommand().equals("Ok"))
			{
				// hide the dialog
				setVisible(false);
				clickedButton = BUTTON_OK;
			}
			if (e.getActionCommand().equals("Cancel"))
			{
				// hide the dialog
				setVisible(false);
				clickedButton = BUTTON_CANCEL;
			}			
		}
		
		/**
		 * Returns a boolean array representing the users selection. 
		 */
		public boolean [] getResults()
		{
			boolean [] res = new boolean[columnBoxes.length];
			for (int i=0; i<res.length; i++)
				res[i] = columnBoxes[i].isSelected();	
			return res;
		}
		
		/**
		 * Returns which button was clicked to close the dialog. This is one of the BUTTON_xx constants
		 */
		public int getClickedButton()
		{
			return clickedButton;
		}
		
		JCheckBox [] 	columnBoxes;
		int				clickedButton;
	}
	
}
