/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Functions for low level io like gpio and spi.
--*
--***************************************************************************************************************/

#include "io.h"
#include <util/delay.h>


/**************************************************************************************
*   G L O B A L S                                                                    */

uint8_t lcdCursorPos = 0;


/**************************************************************************************
*   P R O T O T Y P E S                                                              */

void txLCD(uint8_t data, uint8_t rs);



/**************************************************************************************
*   I M P L E M E N T A T I O N                                                      */

/* read on of the four hardware switches 
   i: index of the switch (0..3) */
uint8_t getHwSwitch(int i)
{
	uint8_t ret = BSWIN;
	ret = ret >> (4+i);
	ret &= 0x01;
	return ret;	
}

/* init the spi hardware and set the spi mode 
	cpol	the CPOL flag, see spi core documentation
	cpha	the CPHA flag, see spi core docu
	div		clock divider ratio, allowed: 2,4,8,16,32,64,128, others default to 128
	*/
void configSpi (uint8_t cpol, uint8_t cpha, uint8_t div)
{
	uint8_t spcr = (1<<SPE) | (1<<MSTR);
	uint8_t sper = 0;
	if (cpol)
		spcr |= (1<<CPOL);
	if (cpha)
		spcr |= (1<<CPHA);
	switch (div)
	{
		case 2:	  break;
		case 4:   spcr |= (1<<SPR0); break;
		case 8:   sper |= (1<<ESPR0); break;
		case 16:  spcr |= (1<<SPR1); break;
		case 32:  spcr |= (1<<SPR1) | (1<<SPR0); break;
		case 64:  sper |= (1<<ESPR0); spcr |= (1<<SPR0); break;
		case 128: 
		default:  sper |= (1<<ESPR0); spcr |= (1<<SPR1); break;
	}
	SPCR = spcr;
	SPER = sper;
}


/* Transfers one byte over spi and returns the received byte */
uint8_t spiTransferByte (uint8_t d)
{
	SPDR = d;
	while (!(SPSR&(1<<SPIF)))
		asm("nop");
	SPSR |= (1<<SPIF);	/* clear the bit */
	return (SPDR);
}


void initLCD()
{
	txLCD(0x38,0);	/* set 8 bit interface */
	_delay_ms(5);
	txLCD(0x38,0);	
	_delay_ms(5);
	txLCD(0x38,0);	
	_delay_us(100);
	txLCD(0x38,0);	
	_delay_us(100);
	txLCD(0x06,0);	/* entry mode set: cursor moves forward, display does not shift */
	_delay_us(100);
	txLCD(0x0C,0);	/* display on, cursor off */
	_delay_us(100);
	clrLCD();
}

void putLCD(unsigned char ch)
{
	txLCD(ch,1);
	_delay_us(100);
	lcdCursorPos++;
	// note: the first half of the display is from 0x00 to 0x07, the second half is 0x40 to 0x47
	if (lcdCursorPos == 8)
	{
		// jump cursor to correct position
		txLCD(0xc0,0);
		_delay_us(100);
	}
}

void putsLCD(const char* p)
{
	while(*p)
	{
		putLCD(*p);
		p++;
	}
}

/* print one 8 bit unsigned int on the LC display */
void printUIntLCD (uint8_t i)
{
	uint8_t printZero = 0;
	uint8_t d;

	d = i / 100;
	if (d!=0)
	{
		putLCD(d+'0');
		printZero = 1;
	}
	i = i - (100*d);

	d = i / 10;
	if ((d!=0) || (printZero))
	{
		putLCD(d+'0');
	}

	i = i - (10*d);
	putLCD(i+'0');
}

/* delete everything from LCD and set cursor to home position */
void clrLCD()
{
	txLCD(0x01,0);
	lcdCursorPos = 0;	// cursor is on the first char position now.
	_delay_ms(2);	
}

/* write data or control reg of LCD 
   rs: 0 = ctrl reg
       1 = data reg */
void txLCD(uint8_t data, uint8_t rs)
{
	uint8_t b = 0;
	rs &= 0x01;
	b |= (rs<<1);	// set or clear RS, enable is zero
	// setup RS bit and data
	SPI_SEL_LCD();
	spiTransferByte(b);
	spiTransferByte(data);	// use transfer here because it blocks until the byte is sent, other wise the deselect would be too early
	SPI_DESELECT();
	_delay_us(10);
	// now set enable to 1
	b |= 0x01;
	SPI_SEL_LCD();
	spiTransferByte(b);
	spiTransferByte(data);	// use transfer here because it blocks until the byte is sent, other wise the deselect would be too early
	SPI_DESELECT();
	_delay_us(10);
	// reset enable to 0
	b &= ~0x01;
	SPI_SEL_LCD();
	spiTransferByte(b);
	spiTransferByte(data);	// use transfer here because it blocks until the byte is sent, other wise the deselect would be too early
	SPI_DESELECT();
}


