/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* This is the high level application. It processes segments passed by the TCP stack and implements the 
--* communication protocol with the client (GUI application).
--*
--***************************************************************************************************************/

#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "app.h"
#include "eth.h"
#include "uip.h"
#include "uip_arp.h"
#include "uart.h"
#include "timer.h"
#include "dhcpc.h"
#include "io.h"
#include "dram.h"

#define BROADCAST_PORT		1024
#define	TCP_PORT			1024
#define DEFAULT_DATA_PORT	1025

// Note: this was reduced to allow the therotical case were data compression actually increases the amount of data.
// using a lower number of samples allows us to stay within the size limit of a datagram even in the worst case
//#define SAMPLES_PER_UDP_P	360
#define SAMPLES_PER_UDP_P	320

#define OPCODE_RESET		0x00
#define OPCODE_ARM			0x01
#define	OPCODE_ID			0x02
#define OPCODE_EN_COMPR		0x03
#define OPCODE_DIS_COMPR 	0x04
#define OPCODE_SET_READ_DEL	0x81
#define OPCODE_SIZE_FWD		0x83
#define OPCODE_SIZE_BWD		0x84
#define OPCODE_DATA_PORT	0x85
#define OPCODE_RESEND		0x86

#define ID_CODE_READY		"SLA2"
#define ID_CODE_ERR0    	"ERR0"

#define ST_IDLE				0
#define ST_RCV_DATA 		1

#ifndef HTONL
	#if UIP_BYTE_ORDER == UIP_BIG_ENDIAN
		#define HTONL(x) 	(x)
	#else
		#define HTONL(x) 	((((uint32_t)(HTONS((x)&0xffff)))<<16) | HTONS(((x)>>16)&0xffff)) 
	#endif 
#endif

#define DRAM_BASE				((uint16_t)0xa000)
#define DRAM_CFG_STAT			(*((volatile uint32_t*)(DRAM_BASE+0)))


/**************************************************************************************************
*   G L O B A L S                                            
*/

/* globals for communication with uIP */
struct uip_udp_conn* 	broadcastConn = 0;
struct uip_udp_conn*	udpDataConn = 0;
struct timer 			udpTimer;
uint8_t					configured 		= 0;
struct uip_conn*		connectedClient = 0;

/* globals for communication with the sampling core and client */	
//int32_t				preTrigLen = 0;		// how many samples (not bytes!) before the trigger should be sent to the client
//int32_t				postTrigLen = 0;	// how many samples (not bytes!) after the trigger should be sent to the client

uint16_t	dataDestPort;		// dest. UDP port number for sample data
uint8_t		compressionEn = 0;
uint8_t		hwDataReady = 0;	// 1: the hardware has finished sampling and has data to send 
uint8_t		waitForTrigger  = 0;
uint32_t	samplesSent = 0;	// how many samples have been sent to the client
uint32_t	samplesToSend = 0;	// total number of samples to send to the client		

/* structure of the UDP broadcast advertisement package 
   All uint16_t and uint32_t are in network byte order */
struct advertisementPkg
{
	char		name[6];			// fixed magic string: "SumpLA"
	char		idCode[4];			// id code of the logic analyzer
	uint16_t	port;				// port number of the tcp server (where the LA awaits connections)
	uint32_t	deviceIpAddr;		// ip v4 address of the LA
	uint32_t	sampleMemLength;	// how many samples the LA can store at maximum (when using all channels)
	uint16_t	numChannels;		// how many sampling channels does the LA have?
	uint16_t	maxSamplingFrq;		// maximum sampling frequency of the LA in MHz
};


// This is a global variabled used to tell uIP the length of an outgoing UDP datagram
// for some reason there is no extern declartion in the uIP headers so we do it here
extern uint16_t uip_slen;



/**************************************************************************************************
*   P R O T O T Y P E S                                             
*/
void execute (struct sump_state* s);
void process(struct sump_state* s);
void handle_connection(struct sump_state *s);
uint8_t send (struct sump_state* s, const void* data, uint8_t len);
void transferSamples();



/**************************************************************************************************
*   I M P L E M E N T A T I O N 
*/

/* this gets called by the stack in any tcp related event occured */
void appcall(void)
{
	/* before the configuration all tcp frames are ignored. (the app is configured after
	   we got an ip address via dhcp) */
	if (!configured)
		return;

	struct sump_state *s = (struct sump_state *)&(uip_conn->appstate);
 
   	if (uip_closed() || uip_aborted() || uip_timedout()) 
	{
		// if the connection of the active client was closed we are free for a new client
		if (connectedClient == uip_conn)
			connectedClient=NULL;
		puts_P(PSTR("closed"));
	}
	else if (uip_connected()) 
	{
		// a new client wishes to connect, prepare the data structures
		s->timer = 0;
		s->txBufWr = 0;
		s->txBufRd = 0;
		s->txSegLen = 0;
		s->dataRcvCnt = 0;
		s->state = ST_IDLE;
		puts_P(PSTR("connected"));
		// The first client automatically gets access to the hardware
		if (connectedClient==NULL)
			connectedClient = uip_conn;		
     	handle_connection(s);
   	}
	else if(s != 0)
	{
		// if a connection is idle for too long we close it. This timeout
		// prevents that a crashed client which has not closed the connection
		// blocks the firmware
    	if(uip_poll() && (!waitForTrigger))
		{
			uartPutc('.');
       		s->timer++;
       		if(s->timer >= 60) 
			{
				puts_P(PSTR("60s idle: abort"));
         		uip_close();
				connectedClient = NULL;
       		}
     	}
		else
		{
			// reset timer variable
       		s->timer = 0;
     	}
		// this is a call for special connection, do the data reception and transmission
     	handle_connection(s);
   	}
   	else
   	{
		puts_P(PSTR("s=0: abort"));
   		uip_abort();
   	}

}

// TCP connection handler (called by the tcp/ip stack)
void handle_connection(struct sump_state *s)
{
	uint8_t i;

	// check if we should retransmit the last packet
	if (uip_rexmit())
	{
		puts_P(PSTR("retrans"));
		if (s->txSegLen == 0)
		{
			// the stack requests a retransmission but we have no segment to retransmit, this is an error
			puts_P(PSTR("app, err: no retrans data, closing conn\n"));
			uip_close();
		}
		uip_send(s->txSeg, s->txSegLen);
	}
	if (uip_acked())
	{
		//puts_P(PSTR("got ack"));
		s->txSegLen = 0;
	}
	// check if we got new data
	if (uip_newdata())
	{
		// process the received data if
		process(s);
	}
	// do we have new data to send and is there no segment in transmission?
	if ((s->txBufRd != s->txBufWr) && (s->txSegLen==0))
	{
		// we copy the data from the ring-buffer into the segment buffer
		for (i=0; (i<TCP_TX_LEN) && (s->txBufRd != s->txBufWr); i++)
		{
			s->txSeg[i] = s->txBuf[(s->txBufRd)++]; // copy the byte
			// check read pointer overrun
			if ((s->txBufRd) >= BUFFER_LENGTH)
				s->txBufRd = 0;
		}
		s->txSegLen = i;
		uip_send(s->txSeg, s->txSegLen);
	}
}


// decode the incoming byte stream into commands
void process(struct sump_state* s)
{
	uint8_t i;
	uint8_t d; 	// data byte

	for (i=0; i<uip_datalen();i++)
	{
        d = ((uint8_t*)uip_appdata)[i];		

		switch (s->state)
		{
			case ST_IDLE:	// first byte is the opcode
				s->opcode = d;
				if ((s->opcode) & 0x80)
				{
					// this is a command with data -> we need four more bytes
					s->state = ST_RCV_DATA;
					s->dataRcvCnt = 0;
				}
				else
				{
					// this is a command with only an opcode -> execute it
					execute(s);
				}
				break;
			
			case ST_RCV_DATA:
				s->data[s->dataRcvCnt++] = d;
				if (s->dataRcvCnt == 4)
				{
					// we got the data -> execute the command
					execute(s);
					s->state = ST_IDLE;
				}
				break;

			default:
				puts_P(PSTR("app: unknown state"));
				// we are in an unknown state
				break;
		}
		
	}
}


/* returns a pointer to the UDP connection used to transmit sample data or NULL if no open connection exists */
struct uip_udp_conn* appGetUDPDataConn()
{
	return udpDataConn;
}


// execute the command specified in opcode (probabely using data as well)
void execute (struct sump_state* s)
{
	//uint8_t i;
	/* make the code more readable by renaming opcode and data */
	uint8_t opcode = s->opcode;
	uint32_t data32 = ((uint32_t)s->data[3])<<24 | \
					 ((uint32_t)s->data[2])<<16 | \
					 ((uint32_t)s->data[1])<<8 | ((uint32_t)s->data[0]);

	if (opcode == OPCODE_RESET)
	{
		//puts_P(PSTR("cmd: reset"));
		waitForTrigger = 0;	
		hwDataReady = 0;
		compressionEn = 0;
	}
	if (opcode == OPCODE_ID)
	{
		//puts_P(PSTR("cmd: id"));
		// if this is the connection from the active client we respond with a proper
		// id code. Otherwise send back a notification that the hardware is in use
		if (connectedClient == uip_conn)
		{
			if (send(s, ID_CODE_READY, 4) != 4)
			{
				puts_P(PSTR("err: txBuffer overflow"));
				uip_close();
			}
		}
		else
		{
			if (send(s, ID_CODE_ERR0, 4) != 4)
			{
				puts_P(PSTR("err: txBuffer overflow"));
				uip_close();
			}
		}
		return;
	}
	// other commands are only valid if they are sent from the connected client
	if (connectedClient != uip_conn)
		return;
	// if we are sampling right now other commands are not valid
	if (waitForTrigger)
		return;
	if (opcode == OPCODE_SIZE_BWD)
	{
		// client tells us how many bytes of pre-trigger data it wants
		dramSetPreTrigLen(data32);
	}
	if (opcode == OPCODE_SIZE_FWD)
	{
		// client tells us how many bytes of post-trigger data it wants
		dramSetPostTrigLen(data32);
		data32--;		// we tell the hardware where to stop, so it takes fwd+1 samples
	}
	if (opcode == OPCODE_SET_READ_DEL)
	{
		/*
		// this is a legacy command, we could keep it for compatibilty
		postTrigLen = (((uint32_t)s->data[3])<<8) | ((uint32_t)s->data[2]);
		postTrigLen++;
		postTrigLen <<= 2;	// the value is in multiples of 4 samples
		preTrigLen  = (((uint32_t)s->data[1])<<8) | ((uint32_t)s->data[0]);
		preTrigLen++;
		preTrigLen <<= 2;
		// we now have the total number of samples in preTrigLen
		preTrigLen -= postTrigLen;
		// the hardware doesn't support this command any more -> set fwd and bwd
		LA_DATA_REG = postTrigLen;
		asm("nop");
		asm("nop");
		asm("nop");
		LA_OPCODE_REG = OPCODE_SIZE_FWD;
		// give the hardware some time to process the command
		for (i=0;i<15;i++)
			asm("nop");
		LA_DATA_REG = preTrigLen;
		asm("nop");
		asm("nop");
		asm("nop");
		LA_OPCODE_REG = OPCODE_SIZE_BWD;
		*/
		puts_P(PSTR("The legacy cmd OPCODE_SET_READ_DEL (0x81) is not supported anymore."));
		uip_close();
		return;
	}
	if (opcode == OPCODE_DATA_PORT)
	{
		// client tells where to send the sample data (in little endian)
		dataDestPort = ((uint16_t)s->data[1])<<8 | s->data[0];
		//printf_P(PSTR("data port: %u\n"),dataDestPort);
		return;
	}
	if (opcode == OPCODE_RESEND)
	{
		// the client wants a udp data packet resent because it missed it
		// this only makes sense if we have data in the sampling memory
		if (!hwDataReady)
			return;

		//printf_P(PSTR("cmd: resend, start: %08x\n"), data32);
		//uartFlush();
		
		// setup DMA source address for requested sample
		dramSetSource(data32);
		
		// Note: the correct sample number has to apper in the packet. Therefore
		// we bias samplesSent and samplesToSend with requested index
		samplesSent = data32;	
		// calc the full length and subtract the offset
		samplesToSend = dramGetSamplingLen() - data32;
		// we never send more than 1 packet here -> limit the length
		if (samplesToSend > SAMPLES_PER_UDP_P)
			samplesToSend = SAMPLES_PER_UDP_P;
		samplesToSend += data32;	// add the offset, we now have the correct stop value
		
		// register a udp connection for sampling data transfer with the stack
		if (udpDataConn == NULL)
		{
			udpDataConn = uip_udp_new(&(uip_conn->ripaddr), htons(dataDestPort));
			if (udpDataConn != NULL)
				uip_udp_bind(udpDataConn, htons(dataDestPort));
		}

		return;
	}
	if (opcode == OPCODE_EN_COMPR)
	{
		compressionEn = 1;
		return;
	}
	if (opcode == OPCODE_DIS_COMPR)
	{
		compressionEn = 0;
		return;
	}
	if (opcode == OPCODE_ARM)
	{
		hwDataReady = 0;	// we start sampling -> the memory content is invalid
		samplesSent = 0;	// until now we have not send any samples
		samplesToSend = dramGetSamplingLen();	// total number of samples to send
		// register a udp connection for sample data transfer with the stack
		if (udpDataConn != NULL)
		{
			// we have an old connection, close it
			uip_udp_remove(udpDataConn);	
		}
		udpDataConn = uip_udp_new(&(uip_conn->ripaddr), htons(dataDestPort));
		if (udpDataConn != NULL)
			uip_udp_bind(udpDataConn, htons(dataDestPort));
		waitForTrigger = 1;	// stop the timeout counter for this connection while we wait for the trigger
	}
	// pass the command to the hardware
	START_CRIT_SEC();
	if (opcode & 0x80)
		LA_DATA_REG = data32;
	LA_OPCODE_REG = opcode;
	END_CRIT_SEC();
	
	//printf_P(PSTR("hw ex: x%02x\n"),opcode);
	//uartFlush();
}


/* Called periodically by main to perform sample data transfer */
void appPeriodic ()
{
	/*
	static int8_t bufInd = -1;	// the ethernet buffer which we are transfering data to right now
	static uint16_t len = 0;	// number of samples in the frame
	int8_t finishBufInd = -1;	// buffer to be finished (create headers)
	*/
	// create tasks structs for all ethernet tx only buffers (and one extra) They have to be
	// static because they are stored in lists managed by dram.c 
	// -1 to leave one tx only buffer for other purposes (uIP specific stuff)
	static task_t tasks[NUM_ETH_BUFFERS-NUM_ETH_RX_BUFFERS-1];
	// list of currently unused task structs (we use 0xffff as an illegal pointer indicating that 
	// the list is not yet initialized (NULL means it's empty)
	static volatile task_t* unusedTasks = (void*)0xffff;	
	
	// searching the arp table takes a lot of time -> keep a local copy of the LLC (ethernet) header
	static uint8_t llcHeaderTemplate[sizeof(struct uip_eth_hdr)];
	static uint8_t llcHeaderTemplateValid = 0;
	
	int8_t bufInd = -1;
	uint8_t j;
	volatile task_t* t;
	
	// initialize task list if necessary
	if (unusedTasks == (void*)(0xffff))
	{
		uint8_t i;
		unusedTasks = NULL;
		for (i=0; i < (NUM_ETH_BUFFERS-NUM_ETH_RX_BUFFERS-1); i++)
		{
			tasks[i].next = unusedTasks;
			unusedTasks = &(tasks[i]);
		}
		//puts_P(PSTR("task list init done"));
	}
	
	if (!configured)
		return;
	
	// check whether a sampling core interrupt occured
	if (dramGetIntFlag())
	{
		// the sample memory (dram) is now valid
		puts_P(PSTR("sampling done"));
		//uartFlush();
		hwDataReady = 1;
		waitForTrigger = 0;
		llcHeaderTemplateValid = 0; 	// this could be a new host -> renew ethernet header template
	}
	
	if (udpDataConn == NULL)
		return;
	
	// make sure we have data to send
	if ((!hwDataReady) || (waitForTrigger))
		return;
	
	// create data transfer tasks for all free tasks
	while (unusedTasks != NULL)
	{
		uint16_t len;
		if ((samplesToSend-samplesSent)>=SAMPLES_PER_UDP_P)
			len = SAMPLES_PER_UDP_P;
		else
			len = samplesToSend-samplesSent;
		
		if (len == 0)	// all data has been copied
			break;
		
		// get a free ethernet buffer
		if ((bufInd=ethAllocBuffer()) < 0)
		{
			// no ethernet buffer available
			puts_P(PSTR("app: no free eth buf for task"));
			uartFlush();
			break;
		}
		
		// remove task struct from list
		t = unusedTasks;
		unusedTasks = t->next;		
		
		// reserve space in the buffers for LLC (ethernet), IP and UDP headers
		uint8_t* buf = (void*)(ethBuf[bufInd] + UIP_LLH_LEN + UIP_IPUDPH_LEN);
				
		// copy the sample number of the first sample into the first word of the datagram
		*((uint32_t*)buf) = samplesSent;
		// copy the number of samples into the datagram
		buf[4] = (uint8_t)(len & 0xff);
		buf[5] = (uint8_t)((len>>8) & 0xff);
		
		t->bufInd = bufInd;
		t->bufSize = UIP_BUFSIZE - UIP_LLH_LEN - UIP_IPUDPH_LEN - 6; // -6 because we have a six byte header within the UDP datagram 
		//t->bufOffset = UIP_LLH_LEN + UIP_IPUDPH_LEN + 6;	// reserve space for LLC, IP, UDP and our header
		t->dest = (void*)(&(buf[6]));
		t->useCompr = compressionEn;
		t->numSamples = len;
		//printf_P(PSTR("reg task for buf %i\n"),bufInd);
		//uartFlush();
		j = dramRegTask(t);
		if (j != 0)
		{
			printf_P(PSTR("can't reg task: %u\n"),j);
			uartFlush();
			ethFreeBuffer(bufInd);
			// put task back into list of free tasks
			t->next = unusedTasks;
			unusedTasks = t;
		}
		// remember that we have initiated the transmission of len samples
		samplesSent += len;
	}
	
	while ((t=dramGetFinishedTask()) != NULL)
	{
		// data transfer done, finish the datagram	
		switch (t->state)
		{
		case ST_TX_DONE:
			//printf_P(PSTR("buffer done %i\n"),t->bufInd);
			//uartFlush();
			// this frame is done, create ip and udp headers
			uip_udp_conn = udpDataConn;	// tell uip which connection we use
			uip_buf = (void*)ethBuf[t->bufInd];
			uip_slen = t->len + 6;	// +6 for the six byte header in the udp datagram (see above)
			uip_process(UIP_UDP_SEND_CONN);	// call stack to create IP and UDP headers
			if (uip_len > 0)
			{
				if (!llcHeaderTemplateValid)
				{
					uip_arp_out();	// create LLC (eth) header
					// save the eth header for the following datagrams
					memcpy(llcHeaderTemplate, uip_buf, sizeof(struct uip_eth_hdr));				
					llcHeaderTemplateValid = 1;
				}
				else
				{
					// copy the header template into the buffer.
					memcpy(uip_buf, llcHeaderTemplate, sizeof(struct uip_eth_hdr));
					// we have to inc the uip length variable (this is usually done by uip_arp_out()
					uip_len += sizeof(struct uip_eth_hdr);
				}
				ethSend(t->bufInd, uip_len);	// schedule buffer for transmission
			}
			else
			{
				ethFreeBuffer(t->bufInd);	// no data to send -> free buffer (this should not happen)
			}
			break;
		
		case ST_DRAM_ERR:
			puts_P(PSTR("appPeriodic: ST_DRAM_ERR reached"));
			uartFlush();
			ethFreeBuffer(t->bufInd);	// no data to send -> free buffer (this should not happen)
			break;
		
		case ST_CMPR_ERR:
			puts_P(PSTR("appPeriodic: ST_CMPR_ERR reached"));
			uartFlush();
			ethFreeBuffer(t->bufInd);	// no data to send -> free buffer (this should not happen)
			break;
			
		default:
			printf_P(PSTR("appPeriodic: t->state unknown %u\n"),t->state);
			ethFreeBuffer(t->bufInd);
		}
		// return task structure to list of unused elements
		//START_CRIT_SEC();	// not sure whether this is necessary or not, but better be save than sorry
		t->next = unusedTasks;
		unusedTasks = t;
		//END_CRIT_SEC();
	}
}


/* called by the stack when a new udp datagram arrived or polled 
   for all registered udp connections */
void udpAppcall (void)
{
	if (!configured)
	{
		// the dhcp client is active, we simply forward all udp data to it
		handle_dhcp();
		return;
	}	

	if (uip_poll())
	{
		if ((uip_udp_conn == broadcastConn) && (timer_expired(&udpTimer)))
		{
			timer_reset(&udpTimer);
			if  (connectedClient==NULL)
			{
				// send an advertisement package 				
				struct advertisementPkg* p = (struct advertisementPkg*)uip_appdata;
				memcpy(p->name, "SumpLA", 6);
				memcpy(p->idCode, ID_CODE_READY, 4);
				p->port = HTONS(TCP_PORT);
				memcpy(&(p->deviceIpAddr), uip_hostaddr, 4);
				p->sampleMemLength = HTONL(SAMPLE_MEM_LENGTH);
				p->numChannels = HTONS(NUM_SAMPLING_CHANNELS);
				p->maxSamplingFrq = HTONS(MAX_SAMPLING_FRQ);	
				uip_udp_send(sizeof(struct advertisementPkg));
			}
		} 
	}
	if (uip_newdata())
	{
		puts_P(PSTR("got an udp datagram"));
	}
}


/* initialize global variables and register connections with the stack */
void appInit (void)
{
	// register the TCP port we want to listen on
	uip_listen(HTONS(TCP_PORT));
	connectedClient = NULL;
	udpDataConn = NULL;
	hwDataReady = 0;
	waitForTrigger = 0;
	dataDestPort = DEFAULT_DATA_PORT;
	// register a udp connection to send our annotation broadcasts
	uip_ipaddr_t addr;
 	uip_ipaddr(&addr,255,255,255,255);
 	broadcastConn = uip_udp_new(&addr, HTONS(BROADCAST_PORT));
 	timer_set(&udpTimer, (uint32_t)CLOCK_SECOND);

	if(broadcastConn != 0) 
	{
		// ok the stack gave us a connection, set the destination port
   		uip_udp_bind(broadcastConn, HTONS(BROADCAST_PORT));
	}
	else
	{
		// this should never happen, it means the stack has not enough connections
		puts_P(PSTR("err: bc conn=0"));
	}
	
	// initialize DRAM hardware
	dramInit();
	
	// ok, the application was configured
	configured = 1;	
}


/* reset the application, this might be necessary if the ethernet link is lost
   This function restores an unconfigured state similar to a hardware reset */
void appReset()
{
	/* reset all globals to their default reset value */
	cli();
	broadcastConn 	= 0;
	udpDataConn 	= 0;
	configured 		= 0;
	connectedClient = 0;
	dataDestPort 	= 0;		
	compressionEn   = 0;
	hwDataReady 	= 0;	
	waitForTrigger  = 0;
	samplesSent 	= 0;	
	samplesToSend 	= 0;			
	sei();
}


/* copy a message into the TCP transmit buffer, transmission starts when the stack is ready to do so */
/* if the data is longer than the free buffer space only as many bytes as fit into the buffer will
   be copied. 
   returns: the number of bytes copied into the transmit buffer */
uint8_t send (struct sump_state* s, const void* data, uint8_t len)
{
	uint8_t i;
	int8_t nFree;
	// calculate the number of free bytes in the buffer
	nFree = s->txBufRd - s->txBufWr - 1;
	if (nFree < 0)
		nFree += BUFFER_LENGTH;

	if (len > nFree)
		len = nFree;

	for (i=0; i<len; i++)
	{
		s->txBuf[(s->txBufWr)++] = ((uint8_t*)data)[i];
		// check index overflow
		if (s->txBufWr >= BUFFER_LENGTH)
			s->txBufWr = 0;
	}
	return i;
}


/* end of file */

