/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2012   Lukas Schrittwieser (LS)
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2012/12/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Implements hardware access function for the DRAM controller and the data compressor hardware.
--*
--***************************************************************************************************************/

#include <avr/interrupt.h>
#include <stdint.h>
#include <util/delay.h>
#include "dram.h"
#include "io.h"
#include "uart.h"
#include "eth.h"


/**************************************************************************************************
*   H A R D W A R E   P R O P E R T I E S                                          
*/

// DRAM DMA interface
// NOTE: due to the wb-wb-bridge inthe hardware all accesses to these registers have to be atomic
#define DRAM_BASE				((uint16_t)0xa000)
#define DRAM_CFG_STAT			(*((volatile uint32_t*)(DRAM_BASE+0)))
#define DRAM_SRC_ADDR			(*((volatile uint32_t*)(DRAM_BASE+4)))
#define DRAM_DEST_ADDR			(*((volatile uint32_t*)(DRAM_BASE+8)))
#define DRAM_TX_LEN				(*((volatile uint32_t*)(DRAM_BASE+12)))
#define DRAM_SAMPLING_PTR		(*((volatile uint32_t*)(DRAM_BASE+16)))
#define DRAM_BUSY				0x01
#define DRAM_IE					0x02
#define DRAM_DONE				0x04
#define DRAM_ERR				0x08
#define DRAM_DONT_INC_DEST		0x10

// Data Compressor Hardware
// define macros for register access
#define CMPR_BASE				((uint16_t)0xc000)
#define CMPR_DATA_IN			(*((volatile uint32_t*)(CMPR_BASE+0)))
#define CMPR_CFG_STAT			(*((volatile uint32_t*)(CMPR_BASE+4)))
#define CMPR_IN_FIFO_THR		(*((volatile uint32_t*)(CMPR_BASE+8)))
#define CMPR_OUT_FIFO_THR		(*((volatile uint32_t*)(CMPR_BASE+12)))
#define CMPR_DMA_DEST			(*((volatile uint32_t*)(CMPR_BASE+16)))
#define CMPR_DMA_LEN			(*((volatile uint32_t*)(CMPR_BASE+20)))
#define CMPR_FIFO_LENS			(*((volatile uint32_t*)(CMPR_BASE+24)))
#define CMPR_CMD				(*((volatile uint32_t*)(CMPR_BASE+28)))
// define bits of command register
#define CMPR_RESET				((uint32_t)1)
#define CMPR_FLUSH				((uint32_t)2)
// define bits of config & status register
#define CMPR_DMA_BUSY			((uint32_t)(1<<3))
#define CMPR_DMA_INC_DEST		(((uint32_t)1)<<8)
#define CMPR_IE_IN_EMPTY		(((uint32_t)1)<<16)
#define CMPR_IE_IN_FULL			(((uint32_t)1)<<17)
#define CMPR_IE_OUT_EMPTY		(((uint32_t)1)<<18)
#define CMPR_IE_OUT_FULL		(((uint32_t)1)<<19)
#define CMPR_IE_ERR				(((uint32_t)1)<<20)
#define CMPR_IE_DONE			(((uint32_t)1)<<21)
#define CMPR_IN_FIFO_EMPTY		(((uint32_t)(1))<<24)
#define CMPR_IN_FIFO_FULL		(((uint32_t)(1))<<25)
#define CMPR_OUT_FIFO_EMPTY		(((uint32_t)(1))<<26)
#define CMPR_OUT_FIFO_FULL		(((uint32_t)(1))<<27)
#define CMPR_DMA_ERR			(((uint32_t)(1))<<28)
#define CMPR_CORE_DONE			(((uint32_t)(1))<<29)



/**************************************************************************************************
*   M A C R O S 
*/

// macro to access an 32 bit config register (memory mapped)
#define REG32(x)		*((volatile uint32_t *)(x))



/**************************************************************************************************
*   G L O B A L S
*/

volatile int32_t	preTrigLen = 0;		// how many samples (not bytes!) before the trigger should be sent to the client
volatile int32_t	postTrigLen = 0;	// how many samples (not bytes!) after the trigger should be sent to the client
//volatile uint8_t	compEnabled = 0;	// whether compression is enabled or not
uint16_t			origDmaLen = 0;		// max DMA length for compressor (to calc number of bytes created by compressor)
volatile uint8_t	intFlag = 0;
//volatile uint8_t	state = ST_DRAM_IDLE;	// state of the data transfer hardware

//volatile task_t* 	freeTaskNodes = NULL;	// list of unused task nodes
volatile task_t* 	scheduledTasks = NULL;	// list of transfer tasks to be done
volatile task_t*	finishedTasks = NULL;	// list of executed tasks
volatile task_t*	finishedTasksTail = NULL;	// tail (last object) of list of executed tasks
volatile task_t*	activeTask = NULL;		// task in execution by hw right now

//task_t	taskNode[NUM_TASK_NODES];	// the objects used in the lists above



/**************************************************************************************************
*   P R O T O T Y P E S
*/

void startTask (volatile task_t* t);
void taskDone();



/**************************************************************************************************
*   I M P L E M E N T A T I O N 
*/

void dramInit()
{
	START_CRIT_SEC();
	CMPR_CMD = CMPR_RESET; // reset the compressor core
	CMPR_OUT_FIFO_THR = (uint32_t)(0xffff0001L); // set a full threshold of 0xffff (unused) and an empty threshold of 1 (empty when there are no bytes in fifo)
	CMPR_IN_FIFO_THR = (uint32_t)(0xffff0001L); // set a full threshold of 0xffff (unused) and an empty threshold of 1 (empty when there are no bytes in fifo)
	finishedTasks = NULL;
	activeTask = NULL;
	scheduledTasks = NULL;
	intFlag = 0;
	END_CRIT_SEC();	
}

void dramSetPreTrigLen(uint32_t pre)
{
	if (pre > SAMPLE_MEM_LENGTH)
		puts_P(PSTR("dram.c: setPreTrig: value too big.\n"));
	else
	{
		START_CRIT_SEC();
		preTrigLen = pre;
		END_CRIT_SEC();
	}
}

void dramSetPostTrigLen(uint32_t post)
{
	if (post > SAMPLE_MEM_LENGTH)
		puts_P(PSTR("dram.c: setPostTrig: value too big.\n"));
	else
	{
		START_CRIT_SEC();
		postTrigLen = post;
		END_CRIT_SEC();
	}
}

/* register a new data transfer task
	newTask: The task to be carried out. It has to reside in static memory (eg
	         global variable as it will be used during the copying process.
	returns: 0 on sucess, all other values are error codes. */
uint8_t	dramRegTask (volatile task_t* newTask)
{
	// check the task
	if (newTask == NULL)
		return 1;
	if (newTask->bufInd < 0)
		return 2;
	if (newTask->bufInd >= NUM_ETH_BUFFERS)
		return 3;
	if (newTask->dest == NULL)
		return 4;
	// task is ok
	START_CRIT_SEC();	// note: tasks get started in ISRs as well
	// make sure there is no race condition by blocking interrupts early enough
	if (activeTask != NULL)
	{
		// insert the new task at the end of the list
		newTask->next = NULL;
		if (scheduledTasks == NULL)
		{	
			// list is empty
			scheduledTasks = newTask;	
		}
		else
		{
			// traverse list to find its end	
			volatile task_t* n = scheduledTasks;
			while (n->next != NULL)
				n = n->next;
			// add new task to end of list
			n->next = newTask;
		}
	}
	else
	{
		// transfer hardware is idle, start task right now
		startTask (newTask);
	}	
	END_CRIT_SEC();
	return 0;
}


/* set source address for a dma transfer. An offset of 0 refers to the first byte sampled by the
   hardware. Note that offset is in samples */
void dramSetSource(uint32_t offset)
{
	uint32_t a;
	START_CRIT_SEC();
	// use <<2 to convert from samples (used throught the firmware) into bytes as the hardware expects byte addresses
	a = DRAM_SAMPLING_PTR - (preTrigLen<<2) - (postTrigLen<<2);		// address of first sample
	DRAM_SRC_ADDR = (uint32_t)((a + (offset<<2)) & SAMPLE_MEM_ADDR_MASK);
	END_CRIT_SEC();
}

void startTask (volatile task_t* t)
{
	if (t == NULL)
		return;
	
	START_CRIT_SEC();
	
	//uint16_t dest = (uint16_t)(ethBuf[t->bufInd]) + t->bufOffset;
	//uint16_t maxLen = t->bufSize - t->bufOffset;
	t->state = ST_DRAM_DMA;
	
	if (t->useCompr)
	{
		// config compressor core
		CMPR_CMD = CMPR_RESET; // reset the compressor core
		CMPR_CFG_STAT = CMPR_DMA_INC_DEST; // tell core to increment the destination address with each transfer
		CMPR_DMA_DEST = (uint32_t)((uint16_t)(t->dest)); // tell the compressors dma unit where to put the data
		CMPR_DMA_LEN = ((uint32_t)(t->bufSize)); // max payload size of a datagram in bytes
		//origDmaLen = maxLen;	// save original length to calc number of bytes created by compressor
		
		// clear DRAM DMA error and done flags
		// tell the DRAM DMA unit to write to a fixed memory location insted of incrementing with every transfer
		DRAM_CFG_STAT = (uint32_t)(DRAM_DONE | DRAM_ERR | DRAM_DONT_INC_DEST | DRAM_IE);
		// tell the dram dma unit to write the data to the compressor
		DRAM_DEST_ADDR = (uint32_t)((uint16_t)(&(CMPR_DATA_IN)));
		DRAM_TX_LEN = (uint32_t)(t->numSamples);
	}
	else
	{
		// tell the DRAM DMA unit to write to an incrementing memory location
		DRAM_CFG_STAT  = (uint32_t)(DRAM_DONE | DRAM_ERR | DRAM_IE);
		
		// tell the dram dma unit to write the data to the compressor
		DRAM_DEST_ADDR = (uint32_t)((uint16_t)(t->dest));
		DRAM_TX_LEN = (uint32_t)(t->numSamples);
	}
	activeTask = t;
	END_CRIT_SEC();
}

/*
uint8_t dramGetState()
{
	uint8_t s = state;
	// we report the DONE state only once (calling function will take care of data transfer immediately)
	if (s == ST_TX_DONE)
		state = ST_DRAM_IDLE;
	return s;	// reading an uint8_t is atomic -> no critical section needed
}*/

// get the total number of samples requested (pre + post trigger length)
uint32_t dramGetSamplingLen ()
{
	uint32_t r;
	START_CRIT_SEC();
	r = postTrigLen + preTrigLen;
	END_CRIT_SEC();
	return r;
}

// read and clear the interrupt flag (set when sampling core interrupt occures)
uint8_t dramGetIntFlag ()
{
	if (intFlag)
	{
		intFlag = 0;
		return 1;
	}
	return 0;
}

/* returns one finished task struct or NULL if no finished tasks are available */
volatile task_t* dramGetFinishedTask()
{
	volatile task_t* ret;
	// remove the task from the list (if there is one)
	START_CRIT_SEC();
	ret = finishedTasks;	// read has to be within crit sec because it is not atomic on AVR (16 bit pointers!)
	if (ret != NULL)
		finishedTasks = finishedTasks->next;	
	END_CRIT_SEC();
	return ret;
}


/* sampling core interrupt handler */
ISR (INT3_vect)
{
	intFlag = 1;	// set interrupt flag
	// set the dma source address to the beginning of the data section
	/*uint32_t	startAdr;
	startAdr =  DRAM_SAMPLING_PTR - (preTrigLen<<2) - (postTrigLen<<2);
	DRAM_SRC_ADDR = (startAdr&SAMPLE_MEM_ADDR_MASK) ;*/
	dramSetSource(0);
}

void taskDone ()
{
	START_CRIT_SEC();
	// insert the active task into the list of finished tasks
	activeTask->next = NULL;
	if (finishedTasks == NULL)
	{
		finishedTasks = activeTask;	
		finishedTasksTail = activeTask;	// this is the only task in the list so it is the tail
	}
	else
	{
		// traverse list to find its end	
		/*volatile task_t* n = finishedTasks;
		while (n->next != NULL)
			n = n->next;
		// add task to the end of the list
		n->next = activeTask;*/
		// add task to the end of the list
		finishedTasksTail->next = activeTask;
		finishedTasksTail = activeTask;
	}
	
	activeTask = NULL;
	// start the next task if we have one		
	if (scheduledTasks != NULL)
	{
		startTask(scheduledTasks);
		scheduledTasks = scheduledTasks->next;
	}
	END_CRIT_SEC();
}

/* DRAM DMA unit interrupt handler */
ISR (INT7_vect)
{
	uint32_t flags = DRAM_CFG_STAT;	// read DMA unit status
		
	// disable this interrupt and clear err and done flag
	DRAM_CFG_STAT	= (uint32_t)(DRAM_DONE | DRAM_ERR);

	if (flags & DRAM_ERR)
	{
		activeTask->state = ST_DRAM_ERR;
		taskDone();
		return;
	}
	
	if (flags & DRAM_DONE)
	{
		if (activeTask->useCompr) 
		{ 	
			activeTask->state = ST_CMPR_FLUSH;
			// flush the compressor core
			CMPR_CMD = CMPR_FLUSH;
			CMPR_CFG_STAT |= (uint32_t)(CMPR_IE_DONE | CMPR_IE_ERR);	// enable core done interrupt
		}
		else
		{
			// This is an uncompressed data transfer so we are done
			activeTask->state = ST_TX_DONE;
			activeTask->len = (activeTask->numSamples)<<2;	// each sample is 4 bytes
			taskDone();
		}
	}
}

/* LZRW compressor unit interrupt handler */
ISR (INT6_vect)
{
	uint32_t i, cfg = CMPR_CFG_STAT;
	
	CMPR_CFG_STAT = (uint32_t)0; // disable this interrupt
	
	if (activeTask->state == ST_CMPR_FLUSH)
	{
		if (cfg & CMPR_DMA_ERR)
		{
			// the DMA unit detected a bus error
			activeTask->state = ST_CMPR_ERR;
			taskDone();
			return;
		}
		
		if  (cfg & CMPR_CORE_DONE)
		{	
			// flush is done, wait until fifo is empty (this can't take long, so we simply block) 	
			do
			{
				i = CMPR_CFG_STAT;
				asm("nop");
			}
			while (!(i&CMPR_OUT_FIFO_EMPTY));
			// this task is done
			activeTask->state = ST_TX_DONE;
			// calc buffer length by comparing the dma length register with its previous value
			i = CMPR_DMA_LEN;
			activeTask->len = activeTask->bufSize - ((uint16_t)i);
			taskDone();
		}
		
	}
	
	
}

/* end of file */
