/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* BitHound firmware for ATmega103 core (this is NOT a whole uC!) on Spartan6 FPGA running at 40MHz.
--*
--***************************************************************************************************************/

#include <avr/interrupt.h>
#include "eth.h"
#include "uart.h"
#include "clock-arch.h"
#include "uip_arp.h"
#include "uip.h"
#include "timer.h"
#include "dhcpc.h"
#include "io.h"



/**************************************************************************************************
*   G L O B A L S
*/

// system tick counter
volatile uint32_t ticCnt=0;

// uIP data buffer
u8_t *uip_buf;

// ethernet address of the NIC
struct uip_eth_addr uipEthAddr;



/**************************************************************************************************
*   D E F I N E S
*/

#define BUF ((struct uip_eth_hdr *)&uip_buf[0])



/**************************************************************************************************
*   P R O T O T Y P E S
*/

void processRx(uint8_t bufferInd);



/**************************************************************************************************
*   I M P L E M E N T A T I O N 
*/

int main (void)
{
	uint8_t 	i;
	int8_t  	ind;
	uint32_t 	j;
	uint8_t 	led = 0x1;
	uint8_t		lastChannelLedUpdate = 0;
	uint8_t     lastEthLinkState = 1;
	//uint8_t		schLoopCnt = 0;
	struct timer periodic_timer, arp_timer;
	
	stdout = &mystdout;	// redirect stdout to uart

	initUart();
	puts_P(PSTR("\n\nhw init start"));
		
	configSpi(0,0,16);	/* cpol = 0, cpha = 0, div clock by 16 */
	initLCD();
	ethInitPhy();
	ethInit();
	sei();

initEthLabel:
	led = 0x01;	
	LEDS = led;
	clrLCD();
	putsLCD("BitHound");

	/* wait until the phy established an ethernet connection */
	while(!ethGetLinkState());
		asm("nop");
	
	putsLCD("  eth up");
	puts_P(PSTR("Ethernet connected"));
		
	// set up the ip stack
	timer_set(&periodic_timer, CLOCK_SECOND / 2);
	timer_set(&arp_timer, CLOCK_SECOND * 10);
	uip_init();
	
	// set ethernet address
 	struct uip_eth_addr e;
	e.addr[0] = OWN_MAC_ADDRESS_0;
	e.addr[1] = OWN_MAC_ADDRESS_1;
	e.addr[2] = OWN_MAC_ADDRESS_2;
	e.addr[3] = OWN_MAC_ADDRESS_3;
	e.addr[4] = OWN_MAC_ADDRESS_4;
	e.addr[5] = OWN_MAC_ADDRESS_5;
	uip_setethaddr 	( e ) ;

	// check the hardware DHCP disable switch
	if (!getHwSwitch(0))
	{
		// initialize the dhcp client and try to obtain an address
		dhcpc_init(&e, 6);
	}
	else
	{
		// dhcp disabled -> configure stack and application with static IP address
		dhcpc_configured(NULL);	
	}
	
	puts_P(PSTR("BitHound running"));
		
	while(1)
    {
		// UDP transmission of sample data: high priority task
		appPeriodic ();
				
		// check if we got new data
		ind = ethGetRxPacket();
		if (ind >= 0)
		{
			// buffer contains data, process it
			START_CRIT_SEC();
			uip_len = ethBufLen[ind];		
			END_CRIT_SEC();
			processRx(ind);
		}
		
		// UDP transmission of sample data: high priority task
		appPeriodic ();
				
		// check for Ethernet link state changes	   
		i = ethGetLinkState();
		if (i != lastEthLinkState)
		{
			lastEthLinkState = i;
			if (i==0)
			{
				puts_P(PSTR("Ethernet connection lost"));
				/* we lost the ethernet connection, so we reset the MAC, stack and everthing above */
				appReset();	/* reset high level application to power up state */
				goto initEthLabel;
			}
			else
				puts_P(PSTR("Ethernet connected"));
		}
		
		// UDP transmission of sample data: high priority task
		appPeriodic ();
		
		// check the periodic timers
    	if(timer_expired(&periodic_timer)) 
		{
			// this happens twice every second
      		timer_reset(&periodic_timer);
			// the stack might generate a packet -> get a free transmit buffer
			do { ind = ethGetTxBuffer(); } while (ind<0);
			uip_buf = (uint8_t*)ethBuf[ind];
			// walk through all TCP connections
      		for(i = 0; i < UIP_CONNS; i++) 
			{
				uip_periodic(i);
				/* If the above function invocation resulted in data that
	   			   should be sent out on the network, the global variable
	   			   uip_len is set to a value > 0. */
				if(uip_len > 0) 
				{
	  				uip_arp_out();
	  				ethSend(ind, uip_len);	// buffer number 2 is the transmit buffer
					// get a new buffer for the next connection
					do { ind = ethGetTxBuffer(); asm("nop"); } while (ind<0);
					uip_buf = (uint8_t*)ethBuf[ind];
				}
			}
			
			// now check all UDP sockets (called connections)
			for(i = 0; i < UIP_UDP_CONNS; i++) 
			{
				uip_udp_periodic(i);
				/* If the above function invocation resulted in data that
				   should be sent out on the network, the global variable
				   uip_len is set to a value > 0. */
				if(uip_len > 0)
				{
					uip_arp_out();
					ethSend(ind, uip_len);	
					// get a new buffer for the next connection
					do { ind = ethGetTxBuffer(); asm("nop"); } while (ind<0);
					uip_buf = (uint8_t*)ethBuf[ind];
				}
			}
      	}
		
      	// UDP transmission of sample data: high priority task
		appPeriodic ();
      	
		// check the arp timer and call it if the timer expires
      	if(timer_expired(&arp_timer)) 
		{
			timer_reset(&arp_timer);
			uip_arp_timer();
		}
		
		// UDP transmission of sample data: high priority task
		appPeriodic ();
		
		// update the channel leds everytime the clock ticks (i.e. with 100hz)
		j = clock_time();
		if ((j&0xff) != lastChannelLedUpdate)	// note we only compare the least significant byte to save calc power
		{
			lastChannelLedUpdate = (uint8_t)(j&0xff);
			START_CRIT_SEC();	// access to hw registers should be atomic
			j = LA_CHANNELS;	// get the current state of the channels
			END_CRIT_SEC();
			SPI_SEL_CH_LEDS();	// select the channel leds
			spiTransferByte((j>>24)&0xff);	// send the channel state Msb to Lsb
			spiTransferByte((j>>16)&0xff);
			spiTransferByte((j>>8)&0xff);
			spiTransferByte((j)&0xff);
			SPI_DESELECT();
		}
    }	// end of while(1) main loop
}


/* passes data contained in buffer number bufferInd to the ip stack */
void processRx(uint8_t bufferInd)
{
	//uartPrintStr_P(PSTR("rx "));
	uip_buf = (uint8_t*) ethBuf[bufferInd];
	if (BUF->type == htons(UIP_ETHTYPE_IP))	// did we receive an ip packet?
	{
		uip_arp_ipin();	
		uip_input();
		if(uip_len > 0)
			uip_arp_out();
	} 
	else if(BUF->type == htons(UIP_ETHTYPE_ARP)) // did we receive an arp packet?
	{
		uip_arp_arpin();
	}
	
	/* If the above function invocation resulted in data that
	   should be sent out on the network, the global variable
	   uip_len is set to a value > 0. */
	if(uip_len > 0)
		ethSend(bufferInd, uip_len);	// send buffer, it will be marked as free when the transmission is finished
	
	// the packet was processed, tell that the eth driver that we don't need it anymore
	ethRxConsume(bufferInd);
}


void uip_log(char *m)
{
	puts(m);
	putchar('\n');
}


/* the dhcp client received a configuration -> inform the stack */
void dhcpc_configured(const struct dhcpc_state *s)
{
	uip_ipaddr_t ipaddr;
	// if s is null the dhcp client has given up -> default ip address
	if (s == NULL)
	{
		// set ip address
		uip_ipaddr(ipaddr, 10,0,0,2);
		uip_sethostaddr(ipaddr);
		// set default gateway
		uip_ipaddr(ipaddr, 10,0,0,1);
		uip_setdraddr(ipaddr);
		// set net mask
		uip_ipaddr(ipaddr, 255,0,0,0);
		uip_setnetmask(ipaddr);

		puts_P(PSTR("using fixed IP address 10.0.0.2"));
		//LEDS = 0x02;

		/* show the address on the LC display */
		clrLCD();
		putsLCD("10.0.0.2/8");
	}
	else
	{
		uip_sethostaddr(s->ipaddr);
	  	uip_setnetmask(s->netmask);
  		uip_setdraddr(s->default_router);

		/* show the address on the LC display */
		clrLCD();
		printUIntLCD((s->ipaddr[0])&0xff);
		putLCD('.');
		printUIntLCD((s->ipaddr[0])>>8);
		putLCD('.');
		printUIntLCD((s->ipaddr[1])&0xff);
		putLCD('.');
		printUIntLCD((s->ipaddr[1])>>8);
		
		printf_P(PSTR("got IP address %d.%d.%d.%d"), (s->ipaddr[0])&0xff, (s->ipaddr[0])>>8, (s->ipaddr[1])&0xff, (s->ipaddr[1])>>8);
		
		//LEDS = 0x01;
	}

	// init the high level application (this is where our protocol is decoded)
	appInit();
}


/* return the current system tick counter */
clock_time_t clock_time(void)
{
	uint32_t ret;
	START_CRIT_SEC();	// note: copying a 32 bit value is not an atomic operation on AVR -> block interrupts
	ret = ticCnt;
	END_CRIT_SEC();
	return ret;
}


/* system timer interrupt, generated periodically */
ISR (INT0_vect)
{
	ticCnt++;
}

