/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* This is file implements functions to control the uart. It buffers the output data and allows the usage of 
--* libc output functions like printf and printf_P (for strings in program space).
--*
--***************************************************************************************************************/


/***************************************************************************************************************
*   I N C L U D E S
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "uart.h"
#include "io.h"



/***************************************************************************************************************
*   H A R D W A R E   C O N F I G U R A T I O N
*/

#define UART_BASE	0x30
#define UART_RXB	_SFR_IO8(UART_BASE+0)
#define UART_TXB	_SFR_IO8(UART_BASE+0)
#define UART_IER	_SFR_IO8(UART_BASE+1)
#define UART_IIR	_SFR_IO8(UART_BASE+2)
#define UART_FCR	_SFR_IO8(UART_BASE+2)
#define UART_LCR	_SFR_IO8(UART_BASE+3)
#define UART_MCR	_SFR_IO8(UART_BASE+4)
#define UART_LSR	_SFR_IO8(UART_BASE+5)
#define UART_MSR	_SFR_IO8(UART_BASE+6)

/*
 * These are the definitions for the FIFO Control Register
 * (16650 only)
 */
#define UART_FCR_ENABLE_FIFO	0x01 /* Enable the FIFO */
#define UART_FCR_CLEAR_RCVR		0x02 /* Clear the RCVR FIFO */
#define UART_FCR_CLEAR_XMIT		0x04 /* Clear the XMIT FIFO */
#define UART_FCR_DMA_SELECT		0x08 /* For DMA applications */
#define UART_FCR_TRIGGER_MASK	0xC0 /* Mask for the FIFO trigger range */
#define UART_FCR_TRIGGER_1		0x00 /* Mask for trigger set at 1 */
#define UART_FCR_TRIGGER_4		0x40 /* Mask for trigger set at 4 */
#define UART_FCR_TRIGGER_8		0x80 /* Mask for trigger set at 8 */
#define UART_FCR_TRIGGER_14		0xC0 /* Mask for trigger set at 14 */

/*
 * These are the definitions for the Line Control Register
 * 
 * Note: if the word length is 5 bits (UART_LCR_WLEN5), then setting 
 * UART_LCR_STOP will select 1.5 stop bits, not 2 stop bits.
 */
#define UART_LCR_DLAB	0x80	/* Divisor latch access bit */
#define UART_LCR_SBC	0x40	/* Set break control */
#define UART_LCR_SPAR	0x20	/* Stick parity (?) */
#define UART_LCR_EPAR	0x10	/* Even parity select */
#define UART_LCR_PARITY	0x08	/* Parity Enable */
#define UART_LCR_STOP	0x04	/* Stop bits: 0=1 stop bit, 1= 2 stop bits */
#define UART_LCR_WLEN5  0x00	/* Wordlength: 5 bits */
#define UART_LCR_WLEN6  0x01	/* Wordlength: 6 bits */
#define UART_LCR_WLEN7  0x02	/* Wordlength: 7 bits */
#define UART_LCR_WLEN8  0x03	/* Wordlength: 8 bits */

/*
 * These are the definitions for the Line Status Register
 */
#define UART_LSR_TEMT	0x40	/* Transmitter empty */
#define UART_LSR_THRE	0x20	/* Transmit-hold-register empty */
#define UART_LSR_BI	0x10	/* Break interrupt indicator */
#define UART_LSR_FE	0x08	/* Frame error indicator */
#define UART_LSR_PE	0x04	/* Parity error indicator */
#define UART_LSR_OE	0x02	/* Overrun error indicator */
#define UART_LSR_DR	0x01	/* Receiver data ready */

/*
 * These are the definitions for the Interrupt Identification Register
 */
#define UART_IIR_NO_INT	0x01	/* No interrupts pending */
#define UART_IIR_ID	0x06	/* Mask for the interrupt ID */

#define UART_IIR_MSI	0x00	/* Modem status interrupt */
#define UART_IIR_THRI	0x02	/* Transmitter holding register empty */
#define UART_IIR_TOI	0x0c	/* Receive time out interrupt */
#define UART_IIR_RDI	0x04	/* Receiver data interrupt */
#define UART_IIR_RLSI	0x06	/* Receiver line status interrupt */

/*
 * These are the definitions for the Interrupt Enable Register
 */
#define UART_IER_MSI	0x08	/* Enable Modem status interrupt */
#define UART_IER_RLSI	0x04	/* Enable receiver line status interrupt */
#define UART_IER_THRI	0x02	/* Enable Transmitter holding register int. */
#define UART_IER_RDI	0x01	/* Enable receiver data interrupt */

/*
 * These are the definitions for the Modem Control Register
 */
#define UART_MCR_LOOP	0x10	/* Enable loopback test mode */
#define UART_MCR_OUT2	0x08	/* Out2 complement */
#define UART_MCR_OUT1	0x04	/* Out1 complement */
#define UART_MCR_RTS	0x02	/* RTS complement */
#define UART_MCR_DTR	0x01	/* DTR complement */

/*
 * These are the definitions for the Modem Status Register
 */
#define UART_MSR_DCD	0x80	/* Data Carrier Detect */
#define UART_MSR_RI		0x40	/* Ring Indicator */
#define UART_MSR_DSR	0x20	/* Data Set Ready */
#define UART_MSR_CTS	0x10	/* Clear to Send */
#define UART_MSR_DDCD	0x08	/* Delta DCD */
#define UART_MSR_TERI	0x04	/* Trailing edge ring indicator */
#define UART_MSR_DDSR	0x02	/* Delta DSR */
#define UART_MSR_DCTS	0x01	/* Delta CTS */
#define UART_MSR_ANY_DELTA 0x0F	/* Any of the delta bits! */



/***************************************************************************************************************
*   D E F I N E S  
*/

// lenght of the tx buffer in bytes, must not be larger than 255 (8bit)
#define TX_BUF_LEN	32

// control flags for the ringbuffer
#define TX_BUF_FULL	0x01
#define TX_BUF_EMPTY 0x02



/***************************************************************************************************************
*   G L O B A L S  
*/

// transmit buffer to store bytes for uart transmission, implemented as ring buffer
volatile char txBuf[TX_BUF_LEN];
volatile uint8_t txBufWr = 0;
volatile uint8_t txBufRd = 0;
volatile uint8_t txBufFlags = TX_BUF_EMPTY;



/***************************************************************************************************************
*   P R O T O T Y P E S
*/

void strPrintHex (char* str, uint8_t i);



/***************************************************************************************************************
*   I M P L E M E N T A T I O N  
*/

void initUart(void)
{
	UART_LCR = 0x83;	// select baud rate divisor latch
	UART_TXB = F_CPU / (16 * 115200);	// lsb
	UART_IER = 0;	// msb
	UART_LCR = 0x03;	// select 8 bit chars, no parity, 1 stop bit, deselect baud rate divisor latch
	UART_FCR = 0xff;	// clear the fifos, set highest trigger level
	UART_FCR = 0xc0;
	UART_MCR = 0x00;
	txBufWr = 0;
	txBufRd = 0;
	txBufFlags = TX_BUF_EMPTY;
}


int uartPutcStream(char c, FILE * stream)
{
	// wait until the buffer is not full anymore
	while(txBufFlags&TX_BUF_FULL)
		asm("nop");

	// insert the byte into the ring buffer
	START_CRIT_SEC();
	uint8_t ind = txBufWr;	// use a local var. to speed things up, txBufWr is volatile
	txBuf[ind++] = c;
	if (ind >= TX_BUF_LEN)
		ind = 0;
	// we inserted one char into the buffer. if the two indexes match this means the buffer is full
	if (ind == txBufRd)
		txBufFlags |= TX_BUF_FULL;
	// the buffer can not be empty now
	txBufFlags &= (~TX_BUF_EMPTY);
	// write back modified index
	txBufWr = ind;
	
	// enable the transmitter ready interrupt
	UART_IER |= UART_IER_THRI;

	END_CRIT_SEC();
	
	return c;
}

void uartPutc(char c)
{
	// MS windows compatibilty
	//if(c == '\n') 
	//	uartPutc('\r');

	uartPutcStream(c, NULL);
}

void uartFlush()
{
	uint8_t f;
	// wait until the ring buffers empty flag is set
	do
	{
		cli();
		f = txBufFlags;
		sei();
	} while (!(f&TX_BUF_EMPTY));
}

/*
void uartPrintUInt8 (uint8_t i)
{
	uint8_t flag = 0;
	if (i>100)
	{
		uartPutc((i/100)+'0');
		i = i % 100;
		flag = 1;
	}
	if ((i>10) || flag)
	{
		uartPutc((i/10)+'0');
		i = i % 10;
		flag = 1;
	}
	uartPutc(i+'0');
}*/
/*
void uartPrintUInt16 (uint16_t i)
{
	uint8_t flag = 0;
	if ((i>10000) || flag)
	{
		uartPutc((i/10000)+'0');
		i = i % 10000;
		flag = 1;
	}
	if ((i>1000) || flag)
	{
		uartPutc((i/1000)+'0');
		i = i % 1000;
		flag = 1;
	}
	if ((i>100)|| flag)
	{
		uartPutc((i/100)+'0');
		i = i % 100;
		flag = 1;
	}
	if ((i>10) || flag)
	{
		uartPutc((i/10)+'0');
		i = i % 10;
		flag = 1;
	}
	uartPutc(i+'0');
}

void uartPrintStr(const char *s)
{
	while (*s != 0)
	{
		uartPutcStream(*s, NULL);
		s++;
	}
}*/

// print a string stored in program space
void uartPrintStr_P(const char *s)
{
	char c;
	while(1)
	{
		c = pgm_read_byte(s++);
		if (c==0)
			return;
		uartPutcStream(c,NULL);
	}
}

/*
void printHex (uint8_t i)
{
	char c;
	c = ((i>>4)>9) ? (i>>4)-10+'a' : (i>>4)+'0';
	uartPutc(c);
	i &= 0x0F;
	c = i>9 ? i-10+'a' : i+'0';
	uartPutc(c);
}

void printHex32 (uint32_t i)
{
	char str[9] = {0};
	strPrintHex(str,(i>>24)&0xff);
	strPrintHex(str+2,(i>>16)&0xff);
	strPrintHex(str+4,(i>>8)&0xff);
	strPrintHex(str+6,i&0xff);
	uartPrintStr(str);
}

void strPrintHex (char* str, uint8_t i)
{
	char c;
	c = ((i>>4)>9) ? (i>>4)-10+'a' : (i>>4)+'0';
	*str = c;
	i &= 0x0F;
	c = i>9 ? i-10+'a' : i+'0';
	*(str+1) = c;
}

void printMac(uint8_t *b)
{
	char str[18]={0};
	strPrintHex(str,b[0]);
	str[2]=':';
	strPrintHex(str+3,b[1]);
	str[5]=':';
	strPrintHex(str+6,b[2]);
	str[8]=':';
	strPrintHex(str+9,b[3]);
	str[11]=':';
	strPrintHex(str+12,b[4]);
	str[14]=':';
	strPrintHex(str+15,b[5]);
	uartPrintStr(str);
}*/

char uartGetc()
{
	// block until we have data
	while ((UART_LSR & (1<<0)) == 0)	
		asm("nop");

    return UART_RXB;
}



// uart interrupt
ISR(INT1_vect)
{
	uint8_t flags = txBufFlags;	// use a local var. to speed up things

	if (UART_IIR & 0x01)
	{
		// this means no interupt pending
		LEDS ^= 0x04;
	}

	if ((UART_LSR&UART_LSR_THRE) && (!(flags&TX_BUF_EMPTY)))
	{
		// there is at least one byte in the tx buffer and the uart can accept data -> send
		uint8_t ind = txBufRd;
		UART_TXB = txBuf[ind++];
		if (ind >= TX_BUF_LEN)
			ind = 0;
		if (ind == txBufWr)
		{
			// we read one byte so the buffer can not be full now -> it is empty
			flags |= TX_BUF_EMPTY;
			// disable the interrupt
			UART_IER &= ~UART_IER_THRI;
		}
		flags &= ~TX_BUF_FULL;
		txBufRd = ind;
		txBufFlags = flags;
	}					
}

	


