/* 
 Test file for ATmega103 core (this is NOT a whole uC!) on Spartan6 FPGA running at 40MHz
 */


#include <avr/io.h>
#include <avr/interrupt.h>
#include "eth.h"


// uart defines
#define UART_BASE	0x30
#define UART_RXB	_SFR_IO8(UART_BASE+0)
#define UART_TXB	_SFR_IO8(UART_BASE+0)
#define UART_IER	_SFR_IO8(UART_BASE+1)
#define UART_IIR	_SFR_IO8(UART_BASE+2)
#define UART_FCR	_SFR_IO8(UART_BASE+2)
#define UART_LCR	_SFR_IO8(UART_BASE+3)
#define UART_MCR	_SFR_IO8(UART_BASE+4)
#define UART_LSR	_SFR_IO8(UART_BASE+5)
#define UART_MSR	_SFR_IO8(UART_BASE+6)

#define GPIO_BASE	0x00
#define LEDS		_SFR_IO8(GPIO_BASE+0)



// DRAM DMA interface
// NOTE: due to the wb-wb-bridge inthe hardware all accesses to these register have to be atomic
#define DRAM_BASE				((uint16_t)0xa000)
#define DRAM_CFG_STAT			(*((volatile uint32_t*)(DRAM_BASE+0)))
#define DRAM_SRC_ADDR			(*((volatile uint32_t*)(DRAM_BASE+4)))
#define DRAM_DEST_ADDR			(*((volatile uint32_t*)(DRAM_BASE+8)))
#define DRAM_TX_LEN				(*((volatile uint32_t*)(DRAM_BASE+12)))
#define DRAM_SAMPLING_PTR		(*((volatile uint32_t*)(DRAM_BASE+16)))
#define DRAM_BUSY				0x01
#define DRAM_IE					0x02
#define DRAM_DONE				0x04
#define DRAM_ERR				0x08
#define DRAM_DONT_INC_DEST		0x10

// Data Compressor Hardware
// define macros for register access
#define CMPR_BASE				((uint16_t)0xc000)
#define CMPR_DATA_IN			(*((volatile uint32_t*)(DRAM_BASE+0)))
#define CMPR_CFG_STAT			(*((volatile uint32_t*)(DRAM_BASE+4)))
#define CMPR_IN_FIFO_THR		(*((volatile uint32_t*)(DRAM_BASE+8)))
#define CMPR_OUT_FIFO_THR		(*((volatile uint32_t*)(DRAM_BASE+12)))
#define CMPR_DMA_DEST			(*((volatile uint32_t*)(DRAM_BASE+16)))
#define CMPR_DMA_LEN			(*((volatile uint32_t*)(DRAM_BASE+20)))
// define bits of config register
#define CMPR_RESET				((uint32_t)0x00000001)
#define CMPR_FLUSH				((uint32_t)0x00000002)
#define CMPR_DMA_INC_DEST		((uint32_t)0x00000100)
#define CMPR_IE_IN_EMPTY		((uint32_t)0x00010000)
#define CMPR_IE_IN_FULL			((uint32_t)0x00020000)
#define CMPR_IE_OUT_EMPTY		((uint32_t)0x00040000)
#define CMPR_IE_OUT_FULL		((uint32_t)0x00080000)
#define CMPR_IE_ERR				((uint32_t)0x00100000)
#define CMPR_IE_DONE			((uint32_t)0x00200000)
// define bits of stauts register
#define CMPR_DMA_BUSY			((uint32_t)0x00000004)
#define CMPR_IN_FIFO_EMPTY		((uint32_t)0x01000000)
#define CMPR_IN_FIFO_FULL		((uint32_t)0x02000000)
#define CMPR_OUT_FIFO_EMPTY		((uint32_t)0x04000000)
#define CMPR_OUT_FIFO_FULL		((uint32_t)0x08000000)
#define CMPR_CORE_DONE			((uint32_t)0x10000000)
    

void initUart(void);
void testETH();
void testCompressor();


int main (void)
{
	//testETH();
	
	testCompressor();
	
	LEDS = 0xFF;
	while(1)
		asm("nop");
}

void testCompressor()
{
	int16_t len = 10;	// number of byte to be transferd
	uint32_t i = 0;
	LEDS = 1;
	
	// set the destination address (use some random sram value)
	uint16_t dmaDest = (uint16_t)4096;	// default: no compression, write straight into datagram
	
	{ 	// enable data compression
		
		// config compressor core
		cli();	// note: due to the WB-WB-bridge these accesses have to be atomic -> disable interrupts
		CMPR_CFG_STAT = CMPR_RESET; // reset the compressor core
		CMPR_CFG_STAT = CMPR_DMA_INC_DEST; // tell core to increment the destination address with each transfer
		CMPR_OUT_FIFO_THR = (uint32_t)(0xffff0001); // set a full threshold of 0xffff (unused) and an empty threshold of 1 (empty when there are no bytes in fifo)
		CMPR_DMA_DEST = (uint32_t)dmaDest; // tell the compressors dma unit where to put the data
		CMPR_DMA_LEN = 1440; // max payload size in bytes
		// tell the DRAM DMA unit to write to a fixed memory location insted of incrementing with every transfer
		DRAM_CFG_STAT = (uint32_t)DRAM_DONT_INC_DEST;
		sei();
		dmaDest = (uint16_t)(&(CMPR_DATA_IN)); // tell dram dma where to put the data
	}
	cli(); // note: due to the WB-WB-bridge these accesses have to be atomic -> disable interrupts
	DRAM_DEST_ADDR = (uint32_t)(dmaDest);
	DRAM_TX_LEN = (uint32_t)len;
	sei();
	
	LEDS = 2;
	
	// wait until the DRAM DMA transfer is done
	do
	{
		cli();
		i = DRAM_CFG_STAT;
		sei();	
	
		// add some nops here so interrupts won't be dissabled all the time
		asm("nop");
		asm("nop");
		asm("nop");
	}
	while (!((i) & (DRAM_DONE | DRAM_ERR)));
			
	LEDS = 3;		
			
	// clear err and done flag
	cli();
	DRAM_CFG_STAT	= (uint32_t)(DRAM_DONE | DRAM_ERR);	
	sei();

	if (i & DRAM_ERR)
		return;
	
	LEDS = 4;
	
	{ 	
		// first step: flush the core
		cli();
		CMPR_CFG_STAT |= CMPR_FLUSH;
		sei();
		
		// second step: wait until the core is done
		do
		{
			cli();
			i = CMPR_CFG_STAT;
			sei();
			asm("nop");
			asm("nop");
		}
		while (!( i&CMPR_CORE_DONE ));
	
		LEDS = 5;
	
		// third step: wait until the compressors output fifo is empty
		do
		{
			cli();
			i = CMPR_CFG_STAT;
			sei();
			asm("nop");
			asm("nop");
		}
		while (!( i&CMPR_OUT_FIFO_EMPTY ));
		
		LEDS = 6;
		
		cli();
		CMPR_CFG_STAT = CMPR_RESET; // reset the compressor core
		sei();
	}
}


void testETH()
{
uint32_t i;
	uint8_t led = 0x01;
	LEDS = led;
	initUart();
	eth_init();
	sei();	
	led = 0x02;
	LEDS = led;

	tx_data[0] = 0xff;
	tx_data[1] = 0xff;
	tx_data[2] = 0xff;
	tx_data[3] = 0xff;
	tx_data[4] = 0xff;
	tx_data[5] = 0xff;

	tx_data[6] = (uint8_t)(OWN_MAC_ADDRESS >> 40);
	tx_data[7] = (uint8_t)(OWN_MAC_ADDRESS >> 32);
	tx_data[8] = (uint8_t)(OWN_MAC_ADDRESS >> 24);
	tx_data[9] = (uint8_t)(OWN_MAC_ADDRESS >> 16);
	tx_data[10] = (uint8_t)(OWN_MAC_ADDRESS >> 8);
	tx_data[11] = (uint8_t)(OWN_MAC_ADDRESS);

	tx_data[14] = 0xFF;
	tx_data[15] = 0x2B;
	tx_data[16] = 0x40;
	tx_data[17] = 0x50;

	eth_send(4);

	i=0;
    while(1)
    {
        if (rx_done)
        {
            eth_receive();
        }
		i++;
		if (i==100000L)
		{
			i=0;
			led++;
			LEDS = led;
			/* send a frame of minimal length */	
			eth_send(72);
		//	while (!tx_done)
		//		asm("nop");
		}

    }

}

void initUart(void)
{
	UART_LCR = 0x83;	// select baud rate divisor latch
	UART_TXB = F_CPU / (16 * 115200);	// lsb
	UART_IER = 0;	// msb
	UART_LCR = 0x03;	// select 8 bit chars, no parity, 1 stop bit, deselect baud rate divisor latch
	UART_FCR = 0xff;	// clear the fifos, set highest trigger level
	UART_MCR = 0x00;
}

void uart_putc(char c)
{
   // wait until transmit fifo is empty
    while ((UART_LSR & (1<<5)) != (1<<5))
		asm("nop");

    UART_TXB = c;
	// MS windows compatibilty
    if(c == '\n') 
		uart_putc('\r');
}

/* system timer, generated periodically */
ISR (INT0_vect)
{
	asm("nop");
}


/*
// uart interrupt
ISR(INT1_vect)
{
	
}
*/
