--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--* 
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This is a 16kByte RAM moduled used by the SOC. It can be accessed
--* simulatenoulsy by an 8 bit and one 32 bit Wishbone slave interface. 
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_arith.all;


library UNISIM;
use UNISIM.VComponents.all;

entity DataMem is
  port (ClkxCI      : in  std_logic;
        ClkxCIB     : in  std_logic;
        RstxRI      : in  std_logic;
        Wb8CycxSI   : in  std_logic;
        Wb8StbxSI   : in  std_logic;
        Wb8WexSI    : in  std_logic;
        Wb8AdrxDI   : in  std_logic_vector (13 downto 0);
        Wb8DataxDI  : in  std_logic_vector (7 downto 0);
        Wb8DataxDO  : out std_logic_vector (7 downto 0);
        Wb8AckxSO   : out std_logic;
        Wb32CycxSI  : in  std_logic;
        Wb32StbxSI  : in  std_logic;
        Wb32WexSI   : in  std_logic;
        Wb32SelxSI  : in  std_logic_vector (3 downto 0);
        Wb32AdrxDI  : in  std_logic_vector (13 downto 2);
        Wb32DataxDI : in  std_logic_vector (31 downto 0);
        Wb32DataxDO : out std_logic_vector (31 downto 0);
        Wb32AckxSO  : out std_logic);
end DataMem;

architecture Behavioral of DataMem is

  signal Wb8EnBramxS          : std_logic_vector (3 downto 0);
  signal Wb32EnBramxS         : std_logic_vector (3 downto 0);
  signal Wb8AckxSN, Wb8AckxSP : std_logic;
  type BramDataOutType is array (0 to 3) of std_logic_vector(7 downto 0);
  signal BramDataOutxD        : BramDataOutType;
  signal Wb8ClkxC             : std_logic;
  
begin
  -- Note: the xilinx memory needs one clock cycle for reading because it is
  -- a synchronous type of memory. This would require one extra cycle on the
  -- wishbone bus (2 cycles for one transfer). We overcome this disadvantage
  -- by clocking the memory with a 180 degree phaseshifted clock. This shortens
  -- the path to and from the memory to half the period time of the clock
  -- but removes the need for an extra cycle. 

  -- Note 2: This is only used on the 32 bit bus, the 8 bit bus has routing 
  -- problems with this.

  Wb8AckxSO <= Wb8AckxSP;
  --Wb8AckxSO <= Wb8CycxSI and Wb8StbxSI;
  Wb8ClkxC  <= ClkxCI;
  --Wb8ClkxC  <= ClkxCIB;


  Wb8AckxSN  <= Wb8CycxSI and Wb8StbxSI and not Wb8AckxSP;
  Wb32AckxSO <= Wb32CycxSI and Wb32StbxSI;
  -- generate select signal (important for write access)
  process (Wb8AdrxDI, Wb8CycxSI, Wb8StbxSI, Wb32CycxSI, Wb32StbxSI, Wb32SelxSI)
  begin
    if Wb8CycxSI = '1' and Wb8StbxSI = '1' then
      case Wb8AdrxDI(1 downto 0) is
        when "00"   => Wb8EnBramxS <= "0001";
        when "01"   => Wb8EnBramxS <= "0010";
        when "10"   => Wb8EnBramxS <= "0100";
        when "11"   => Wb8EnBramxS <= "1000";
        when others => Wb8EnBramxS <= "0000";
      end case;
    else
      Wb8EnBramxS <= "0000";
    end if;
    if Wb32CycxSI = '1' and Wb32StbxSI = '1' then
      Wb32EnBramxS <= Wb32SelxSI;
    else
      Wb32EnBramxS <= "0000";
    end if;
  end process;

  -- output multiplexer for 8 bit wishbone bus
  Wb8DataxDO <= BramDataOutxD(conv_integer(unsigned(Wb8AdrxDI(1 downto 0))));

  process (ClkxCI)
  begin
    if ClkxCI'event and ClkxCI = '1' then
      Wb8AckxSP <= Wb8AckxSN;
    end if;
  end process;

  BRamInst0_0 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(0)(3 downto 0),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(3 downto 0),
      ENA   => Wb8EnBramxS(0),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(3 downto 0),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(3 downto 0),
      ENB   => Wb32EnBramxS(0),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst0_1 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(0)(7 downto 4),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(7 downto 4),
      ENA   => Wb8EnBramxS(0),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(7 downto 4),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(7 downto 4),
      ENB   => Wb32EnBramxS(0),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst1_0 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(1)(3 downto 0),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(3 downto 0),
      ENA   => Wb8EnBramxS(1),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(11 downto 8),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(11 downto 8),
      ENB   => Wb32EnBramxS(1),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst1_1 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(1)(7 downto 4),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(7 downto 4),
      ENA   => Wb8EnBramxS(1),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(15 downto 12),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(15 downto 12),
      ENB   => Wb32EnBramxS(1),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst2_0 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(2)(3 downto 0),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(3 downto 0),
      ENA   => Wb8EnBramxS(2),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(19 downto 16),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(19 downto 16),
      ENB   => Wb32EnBramxS(2),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst2_1 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(2)(7 downto 4),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(7 downto 4),
      ENA   => Wb8EnBramxS(2),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(23 downto 20),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(23 downto 20),
      ENB   => Wb32EnBramxS(2),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst3_0 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(3)(3 downto 0),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(3 downto 0),
      ENA   => Wb8EnBramxS(3),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(27 downto 24),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(27 downto 24),
      ENB   => Wb32EnBramxS(3),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

  BRamInst3_1 : RAMB16_S4_S4
    generic map (
      INIT_A       => "0000",  --  Value of output RAM registers at startup
      INIT_B       => "0000",  --  Value of output RAM registers at startup
      SRVAL_A      => "0000",           --  Ouput value upon SSR assertion
      SRVAL_B      => "0000",           --  Ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST",  --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST"   --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      )
    port map (
      -- to 8 bit wishbone bus
      DOA   => BramDataOutxD(3)(7 downto 4),
      ADDRA => Wb8AdrxDI(13 downto 2),
      CLKA  => Wb8ClkxC,
      DIA   => Wb8DataxDI(7 downto 4),
      ENA   => Wb8EnBramxS(3),
      SSRA  => RstxRI,
      WEA   => Wb8WexSI,
      --DOPA    => open,
      --DIPA    => "0",
      -- to 32 bit wishbone bus
      DOB   => Wb32DataxDO(31 downto 28),
      ADDRB => Wb32AdrxDI(13 downto 2),
      CLKB  => ClkxCIB,
      DIB   => Wb32DataxDI(31 downto 28),
      ENB   => Wb32EnBramxS(3),
      SSRB  => RstxRI,
      WEB   => Wb32WexSI
      --DOPB    => open,
      --DIPB    => "0"          
      );

end Behavioral;

