--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This is BitHounds top level fpga source file. It instanciates the components and
--* hooks them up.
--*
--***************************************************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.all;

library UNISIM;
use UNISIM.VComponents.all;

entity LATop is
  generic (
    -- Set this to "TRUE" in a testbench, has to be "FALSE" for implementation
    -- this is needed for the memory controller block (MCB) of xilinx, see dramCtrl.vhd
    IS_SIMULATION : string := "FALSE";

    -- The system timer generates a periodic interrupt, set this constant
    -- to the value needed by the software. Default is 100Hz when running
    -- the cpu core at 50MHz.
    -- To test in the simulator it should be set to a lower value 
    SYS_TIMER_TOP : integer := (50000000 / 100)-1);
  port (
    xtalClock     : in  std_logic;
    resetSwitch   : in  std_logic;
    -- logic analyzer core
    exClockxCI    : in  std_logic;
    inputChxDI    : in  std_logic_vector(31 downto 0);
    testOutputxSO : out std_logic;
    -- uart interface
    UartTxxDO     : out std_logic;
    UartRxxDI     : in  std_logic;
    --UartRtsxSO              : out std_logic;
    --UartCtsxSI              : in  std_logic;

    -- ethernet phy interface (this is GMII because it is a tri-speed phy, however we use only a 100MBit MII mac)
    --PHY_COL                 : in  std_logic;
    --PHY_CRS                 : in  std_logic;
    PHY_INT        : in    std_logic;
    PHY_MDC        : out   std_logic;
    PHY_MDIO       : inout std_logic;
    PHY_RESET      : out   std_logic;
    PHY_RXCLK      : in    std_logic;
    PHY_RXCTL_RXDV : in    std_logic;
    PHY_RXD        : in    std_logic_vector(7 downto 0);
    PHY_RXER       : in    std_logic;
    PHY_TXCLK      : in    std_logic;
    PHY_TXCTL_TXEN : out   std_logic;
    PHY_TXC_GTXCLK : in    std_logic;
    PHY_TXD        : out   std_logic_vector(7 downto 0);
    PHY_TXER       : out   std_logic;
    -- physical memory interface to DDR2 SDRAM (sample memory)
    MemDQxDIO      : inout std_logic_vector(15 downto 0);
    MemAdrxDO      : out   std_logic_vector(12 downto 0);
    MemBaxDO       : out   std_logic_vector(2 downto 0);
    MemRasxEO      : out   std_logic;
    MemCasxEO      : out   std_logic;
    MemWexEO       : out   std_logic;
    MemOdtxO       : out   std_logic;
    MemCkexEO      : out   std_logic;
    MemClkxCO      : out   std_logic;
    MemClkxCOB     : out   std_logic;
    MemLDqsxEO     : inout std_logic;
    MemLDqsxEOB    : inout std_logic;
    MemUDqsxEO     : inout std_logic;
    MemUDqsxEOB    : inout std_logic;
    MemLdmxEO      : out   std_logic;
    MemUdmxEO      : out   std_logic;
    -- calibration (not actually logical signals)
    RzqxIO         : inout std_logic;
    ZioxIO         : inout std_logic;
    -- spi interface
    SpiSckxSO      : out   std_logic;
    SpiMosixDO     : out   std_logic;
    SpiMisoxDI     : in    std_logic;
    SpiSSxSO       : out   std_logic_vector(2 downto 0);
    -- gpio
    SwitchxDI      : in    std_logic_vector(3 downto 0);
    ButtonxDI      : in    std_logic_vector(3 downto 0);
    LedxDO         : out   std_logic_vector (7 downto 0));
end LATop;

architecture Behavioral of LATop is

  component ClkMgr
    port
      (                                 -- Clock in ports
        CLK_IN1  : in  std_logic;
        -- Clock out ports
        CLK_OUT1 : out std_logic;
        CLK_OUT2 : out std_logic;
        CLK_OUT3 : out std_logic;
        CLK_OUT4 : out std_logic;
        -- Status and control signals
        RESET    : in  std_logic;
        LOCKED   : out std_logic
        );
  end component;

  component ProgMem is
    port (ClkxC   : in  std_logic;
          RstxR   : in  std_logic;
          AdrxDI  : in  std_logic_vector (13 downto 0);
          DoutxDO : out std_logic_vector (15 downto 0));
  end component;

  component AVR_Core is
    port(
      --Clock and reset
      cp2         : in  std_logic;
      cp2en       : in  std_logic;
      ireset      : in  std_logic;
      -- JTAG OCD support
      valid_instr : out std_logic;
      insert_nop  : in  std_logic;
      block_irq   : in  std_logic;
      change_flow : out std_logic;
      -- Program Memory
      pc          : out std_logic_vector(15 downto 0);
      inst        : in  std_logic_vector(15 downto 0);
      -- I/O control
      adr         : out std_logic_vector(5 downto 0);
      iore        : out std_logic;
      iowe        : out std_logic;
      -- Data memory control
      ramadr      : out std_logic_vector(15 downto 0);
      ramre       : out std_logic;
      ramwe       : out std_logic;
      cpuwait     : in  std_logic;
      -- Data paths
      dbusin      : in  std_logic_vector(7 downto 0);
      dbusout     : out std_logic_vector(7 downto 0);
      -- Interrupt
      irqlines    : in  std_logic_vector(22 downto 0);
      irqack      : out std_logic;
      irqackad    : out std_logic_vector(4 downto 0);
      --Sleep Control
      sleepi      : out std_logic;
      irqok       : out std_logic;
      globint     : out std_logic;
      --Watchdog
      wdri        : out std_logic
      );
  end component;

  component AVR_WB_Translator is
    port(
      CLK           : in  std_logic;
      RST           : in  std_logic;
      --AVR Side:
      --IO Bus in:
      AVR_IO_ADR_I  : in  std_logic_vector(5 downto 0);
      AVR_iore_I    : in  std_logic;
      AVR_iowe_I    : in  std_logic;
      --Ram Bus in:
      AVR_RAMADR_I  : in  std_logic_vector(15 downto 0);
      AVR_ramre_I   : in  std_logic;
      AVR_ramwe_I   : in  std_logic;
      --Datapath:
      AVR_DAT_O     : out std_logic_vector(7 downto 0);
      AVR_DAT_I     : in  std_logic_vector(7 downto 0);
      --CPU wait:
      AVR_cpuwait_O : out std_logic;

      --IO Wishbone:
      Wb_DAT_IO_O : out std_logic_vector(7 downto 0);
      Wb_DAT_IO_I : in  std_logic_vector(7 downto 0);
      Wb_ADR_IO_O : out std_logic_vector(5 downto 0);
      Wb_ACK_IO_I : in  std_logic;
      Wb_CYC_IO_O : out std_logic;
      Wb_STB_IO_O : out std_logic;
      Wb_WE_IO_O  : out std_logic;

      --RAM Wishbone:
      Wb_DAT_RAM_O : out std_logic_vector(7 downto 0);
      Wb_DAT_RAM_I : in  std_logic_vector(7 downto 0);
      Wb_ADR_RAM_O : out std_logic_vector(15 downto 0);
      Wb_ACK_RAM_I : in  std_logic;
      Wb_CYC_RAM_O : out std_logic;
      Wb_STB_RAM_O : out std_logic;
      Wb_WE_RAM_O  : out std_logic
      );
  end component;

  component ioBus is
    port (
      ClkxCI     : in  std_logic;
      RstxRI     : in  std_logic;
      -- master interface from cpu
      CpuCycxSI  : in  std_logic;
      CpuStbxSI  : in  std_logic;
      CpuWexSI   : in  std_logic;
      CpuAdrxDI  : in  std_logic_vector(5 downto 0);
      CpuDatxDI  : in  std_logic_vector(7 downto 0);
      CpuDatxDO  : out std_logic_vector(7 downto 0);
      CpuAckxSO  : out std_logic;
      CpuErrxSO  : out std_logic;
      CpuRtyxSO  : out std_logic;
      -- interrupts
      UartIntxSO : out std_logic;
      SpiIntxSO  : out std_logic;
      -- uart io pins
      UartTxxDO  : out std_logic;
      UartRxxDI  : in  std_logic;
      UartRtsxSO : out std_logic;
      UartCtsxSI : in  std_logic;
      -- spi pins
      SpiSckxSO  : out std_logic;
      SpiMisoxDI : in  std_logic;
      SpiMosixDO : out std_logic;
      SpiSSxSO   : out std_logic_vector(2 downto 0);
      -- gpio pins                      
      SwitchxDI  : in  std_logic_vector(3 downto 0);
      ButtonxDI  : in  std_logic_vector(3 downto 0);
      LedxDO     : out std_logic_vector(7 downto 0)
      );
  end component;

  component dataBus is
    generic (
      IS_SIMULATION : string := "FALSE";
      SIMULATE_DRAM : boolean;
      SIMULATE_MAC  : boolean
      );
    port (
      -- system singals for the dram controller (it does clock & reset handling)
      RstButtonxR   : in  std_logic;
      BoardClkxCI   : in  std_logic;
      --BoardClkxCIB    : in  std_logic;
      LaCoreClkxCI  : in  std_logic;
      LaCoreClkxCBI : in  std_logic;
      LaCoreRstxRI  : in  std_logic;
      RefClkxCO     : out std_logic;
      McbReadyxSO   : out std_logic;

      ClkxCI  : in std_logic;
      ClkxCIB : in std_logic;
      RstxRI  : in std_logic;

      -- master interface from cpu
      CpuCycxSI : in  std_logic;
      CpuStbxSI : in  std_logic;
      CpuWexSI  : in  std_logic;
      CpuAdrxDI : in  std_logic_vector(15 downto 0);
      CpuDatxDI : in  std_logic_vector(7 downto 0);
      CpuDatxDO : out std_logic_vector(7 downto 0);
      CpuAckxSO : out std_logic;
      CpuErrxSO : out std_logic;
      CpuRtyxSO : out std_logic;

      -- MII : phy <-> mac
      PhyTxClkxCI  : in  std_logic;
      PhyTxdxDO    : out std_logic_vector(3 downto 0);
      PhyTxEnxSO   : out std_logic;
      PhyTxErrxSO  : out std_logic;
      -- MII RX
      PhyRxClkxCI  : in  std_logic;
      PhyRxdxDI    : in  std_logic_vector(3 downto 0);
      PhyRxDvxSI   : in  std_logic;
      PhyRxErrxSI  : in  std_logic;
      PhyColxSI    : in  std_logic;
      PhyCrsxSI    : in  std_logic;
      -- MIIM
      PhyMdcxSO    : out std_logic;
      PhyMdioxDI   : in  std_logic;
      PhyMdioxDO   : out std_logic;
      PhyMdioEnxSO : out std_logic;

      -- physical memory interface to DDR2 SDRAM (sample memory)
      MemDQxDIO   : inout std_logic_vector(15 downto 0);
      MemAdrxDO   : out   std_logic_vector(12 downto 0);
      MemBaxDO    : out   std_logic_vector(2 downto 0);
      MemRasxEO   : out   std_logic;
      MemCasxEO   : out   std_logic;
      MemWexEO    : out   std_logic;
      MemOdtxO    : out   std_logic;
      MemCkexEO   : out   std_logic;
      MemClkxCO   : out   std_logic;
      MemClkxCOB  : out   std_logic;
      MemLDqsxEO  : inout std_logic;
      MemLDqsxEOB : inout std_logic;
      MemUDqsxEO  : inout std_logic;
      MemUDqsxEOB : inout std_logic;
      MemLdmxEO   : out   std_logic;
      MemUdmxEO   : out   std_logic;
      -- calibration (not actually logical signals)
      RzqxIO      : inout std_logic;
      ZioxIO      : inout std_logic;

      -- la core interface (to outside of fpga)
      exClockxCI    : in  std_logic;
      inputChxDI    : in  std_logic_vector(31 downto 0);
      testOutputxSO : out std_logic;

      -- interrupts
      LaCoreIRQxSO    : out std_logic;
      LaCoreIntAckxSI : in  std_logic;
      EthIntxSO       : out std_logic;
      DDmaIntxSO      : out std_logic;
      CmpIntxSO       : out std_logic
      );
  end component;

  -- define how long (in clock cycles of the startup clock) the reset impuls is
  --constant RST_CNT_MAX        : integer :=  10000;
  -- shift register are used to generate the reset pulses in the cpu- and the sampling-clock-domain
  -- the length of this register configured here defines how long the puls lasts
  -- This was necessary to resolve meta-stability problems which led to improper device resets causing
  -- strange behaviour
  constant RstSregSocLen  : integer := 8;
  constant RstSregCoreLen : integer := 8;

  -- cpu signals
  signal ProgMemAdrxD  : std_logic_vector(15 downto 0);
  signal ProgMemDataxD : std_logic_vector(15 downto 0);
  signal IOAdrxD       : std_logic_vector(5 downto 0);
  signal IORexS        : std_logic;
  signal IOWexS        : std_logic;
  signal RamAdrxD      : std_logic_vector(15 downto 0);
  signal RamRexS       : std_logic;
  signal RamWexS       : std_logic;
  signal CpuDataInxD   : std_logic_vector(7 downto 0)  := x"00";
  signal CpuDataOutxD  : std_logic_vector(7 downto 0);
  signal IrqReqxS      : std_logic_vector(22 downto 0) := "00000000000000000000000";
  signal IrqAckxS      : std_logic;
  signal IrqAckAdrxD   : std_logic_vector(4 downto 0);
  signal CpuWaitxS     : std_logic;

  -- system control
  signal ClkxC, ClkxCB                  : std_logic;
  signal LaCoreClkxC, LaCoreClkxCB      : std_logic;
  signal PllRefClkxC                    : std_logic;
  signal McbReadyxS                     : std_logic;
  signal McbReadyxSB                    : std_logic;
  signal RstxR                          : std_logic                                   := '1';
  signal RstxRB                         : std_logic                                   := '0';
  --signal RstxRN, RstxRP       : std_logic := '1';
  signal DcmLockedxS                    : std_logic;
  --signal RstCntxDN, RstCntxDP : integer range 0 to RST_CNT_MAX := 0;
  signal RstSregSocxDN, RstSregSocxDP   : std_logic_vector(RstSregSocLen-1 downto 0)  := (others => '1');
  signal RstSregCorexDN, RstSregCorexDP : std_logic_vector(RstSregCoreLen-1 downto 0) := (others => '1');
  signal RstCorexR                      : std_logic;
  signal resetSwitchxSB                 : std_logic;
  signal PhyIntxSN, PhyIntxSP           : std_logic                                   := '0';

  -- special cpu control
  signal CpuClkEnxS : std_logic;

  -- data memory wishbone bus
  signal WbRamCycxS     : std_logic;
  signal WbRamStbxS     : std_logic;
  signal WbRamWexS      : std_logic;
  signal WbRamAdrxD     : std_logic_vector(15 downto 0);
  signal WbRamDatMISOxD : std_logic_vector(7 downto 0);
  signal WbRamDatMOSIxD : std_logic_vector(7 downto 0);
  signal WbRamAckxS     : std_logic;

  -- io wishbone bus
  signal WbIoCycxS     : std_logic;
  signal WbIoStbxS     : std_logic;
  signal WbIoWexS      : std_logic;
  signal WbIoAdrxD     : std_logic_vector(5 downto 0);
  signal WbIoDatMISOxD : std_logic_vector(7 downto 0);
  signal WbIoDatMOSIxD : std_logic_vector(7 downto 0);
  signal WbIoAckxS     : std_logic;

  -- interrupts
  signal UartIntxS : std_logic;
  signal MacIntxS  : std_logic;
  signal SpiIntxS  : std_logic;
  signal DDmaIntxS : std_logic;
  signal CmpIntxS  : std_logic;

  -- gpio
  signal LedxD : std_logic_vector(7 downto 0);

  -- system timer
  signal SysTimerxDN    : integer range 0 to SYS_TIMER_TOP;
  signal SysTimerxDP    : integer range 0 to SYS_TIMER_TOP;
  signal SysTimerIntxDN : std_logic;
  signal SysTimerIntxDP : std_logic;
  -- phy mac interface signals
  signal PhyMdioxDI     : std_logic;
  signal PhyMdioxDO     : std_logic;
  signal PhyMdioEnxS    : std_logic;

  signal LaCoreIRQxS    : std_logic;
  signal LaCoreIntAckxS : std_logic;

  signal FreezexSN, FreezexSP : std_logic := '0';  -- used to freeze the cpu (debugging)
  
begin

  IrqReqxS(0)           <= SysTimerIntxDP;
  IrqReqxS(1)           <= UartIntxS;
  IrqReqxS(2)           <= MacIntxS;
  IrqReqxS(3)           <= LaCoreIRQxS;
  IrqReqxS(4)           <= SpiIntxS;
  IrqReqxS(5)           <= PhyIntxSP;
  IrqReqxS(6)           <= CmpIntxS;
  IrqReqxS(7)           <= DDmaIntxS;
  LaCoreIntAckxS        <= '1' when IrqAckxS = '1' and (IrqAckAdrxD = "00100") else '0';
  IrqReqxS(22 downto 8) <= (others => '0');
  -- we have to stall the cpu by using the clock enable instead of the cpuwait port
  -- because a cpuwait='1' does not prevent the core from generating two concurrent 
  -- bus access on IO and RAM bus. This, however, can't be handeld by the avr-to-wb
  -- translator. Setting cp2en low during bus transfer pervent this condition
  --CpuClkEnxS            <= not CpuWaitxS and not FreezexSP;  -- use switch to freeze cpu (debugging)
  CpuClkEnxS            <= not CpuWaitxS;
  FreezexSN             <= FreezexSP or SwitchxDI(2);

  --CpuClkEnxS <=  '1';
  CpuInst : AVR_Core
    port map(
      --Clock and reset
      cp2         => ClkxC,
      cp2en       => CpuClkEnxS,
      ireset      => RstxRB,
      -- JTAG OCD support
      valid_instr => open,
      insert_nop  => '0',
      block_irq   => '0',
      change_flow => open,
      -- Program Memory
      pc          => ProgMemAdrxD,
      inst        => ProgMemDataxD,
      -- I/O control
      adr         => IOAdrxD,
      iore        => IORexS,
      iowe        => IOWexS,
      -- Data memory control 
      ramadr      => RamAdrxD,
      ramre       => RamRexS,
      ramwe       => RamWexS,
      cpuwait     => CpuWaitxS,
--              cpuwait     => '0',
      -- Data paths
      dbusin      => CpuDataInxD,
      dbusout     => CpuDataOutxD,
      -- Interrupt
      irqlines    => IrqReqxS,
      irqack      => IrqAckxS,
      irqackad    => IrqAckAdrxD,
      --Sleep Control
      sleepi      => open,
      irqok       => open,
      globint     => open,
      --Watchdog
      wdri        => open
      );

  AvrWbTranslInst : AVR_WB_Translator
    port map (
      CLK           => ClkxC,
      RST           => RstxR,
      --AVR Side:
      --IO Bus in:
      AVR_IO_ADR_I  => IOAdrxD,
      AVR_iore_I    => IORexS,
      AVR_iowe_I    => IOWexS,
      --Ram Bus in:
      AVR_RAMADR_I  => RamAdrxD,
      AVR_ramre_I   => RamRexS,
      AVR_ramwe_I   => RamWexS,
      --Datapath:
      AVR_DAT_O     => CpuDataInxD,
      AVR_DAT_I     => CpuDataOutxD,
      AVR_cpuwait_O => CpuWaitxS,
      --IO Wishbone:
      Wb_DAT_IO_O   => WbIoDatMOSIxD,
      Wb_DAT_IO_I   => WbIoDatMISOxD,
      Wb_ADR_IO_O   => WbIoAdrxD,
      Wb_ACK_IO_I   => WbIoAckxS,
      Wb_CYC_IO_O   => WbIoCycxS,
      Wb_STB_IO_O   => WbIoStbxS,
      Wb_WE_IO_O    => WbIoWexS,

      --RAM Wishbone:
      Wb_DAT_RAM_O => WbRamDatMOSIxD,
      Wb_DAT_RAM_I => WbRamDatMISOxD,
      Wb_ADR_RAM_O => WbRamAdrxD,
      Wb_ACK_RAM_I => WbRamAckxS,
      Wb_CYC_RAM_O => WbRamCycxS,
      Wb_STB_RAM_O => WbRamStbxS,
      Wb_WE_RAM_O  => WbRamWexS
      );


  ProgMemInst : ProgMem
    port map(
      ClkxC   => ClkxCB,
      RstxR   => RstxR,
      AdrxDI  => ProgMemAdrxD(13 downto 0),
      DoutxDO => ProgMemDataxD
      );


  RstxRB         <= not RstxR;
  resetSwitchxSB <= not resetSwitch;
  McbReadyxSB    <= not McbReadyxS;

  RstSregSocxDN <= '0' & RstSregSocxDP(RstSregSocLen-1 downto 1);
  RstxR         <= RstSregSocxDP(0);

  RstSregCorexDN <= '0' & RstSregCorexDP(RstSregCoreLen-1 downto 1);
  RstCorexR      <= RstSregCorexDP(0);

  ClkMgrInst : ClkMgr
    port map (
      -- Clock in ports
      CLK_IN1  => PllRefClkxC,
      -- Clock out ports
      CLK_OUT1 => LaCoreClkxC,
      CLK_OUT2 => LaCoreClkxCB,
      CLK_OUT3 => ClkxC,
      CLK_OUT4 => ClkxCB,
      -- Status and control signals
      RESET    => McbReadyxSB,
      LOCKED   => DcmLockedxS
      );

  SocRstRegs : process (ClkxC, DcmLockedxS)
  begin
    if DcmLockedxS = '0' then
      RstSregSocxDP <= (others => '1');
    elsif ClkxC'event and ClkxC = '1' then
      RstSregSocxDP <= RstSregSocxDN;
    end if;
  end process;

  CoreRstRegs : process (LaCoreClkxC, DcmLockedxS)
  begin
    if DcmLockedxS = '0' then
      RstSregCorexDP <= (others => '1');
    elsif LaCoreClkxC'event and LaCoreClkxC = '1' then
      RstSregCorexDP <= RstSregCorexDN;
    end if;
    
  end process;

--      process (StartUpClkxCI, resetSwitch)
--      begin
--              if resetSwitch='1' then
--                      RstCntxDP       <= 0;
--                      RstxRP          <= '1';
--              elsif StartUpClkxCI'event and StartUpClkxCI='1' then
--                      RstCntxDP       <= RstCntxDN;
--                      RstxRP          <= RstxRN;
--              end if;
--      end process;


  -- system timer, generates a periodic interrupt as time base for the software
  SysTimer : process(SysTimerIntxDP, SysTimerxDP, IrqAckxS, IrqAckAdrxD)
  begin
    SysTimerIntxDN <= SysTimerIntxDP;  -- default: don't change interrupt state
    if SysTimerxDP < SYS_TIMER_TOP then
      -- counter has not reached to vaule -> increment
      SysTimerxDN <= SysTimerxDP+1;
    else
      -- timer has reached final value -> reset it to 0 and assert interrupt
      SysTimerxDN    <= 0;
      SysTimerIntxDN <= '1';
    end if;
    -- deassert interrupt if cpu acknowledes the interrupt
    if IrqAckxS = '1' and IrqAckAdrxD = "00001" then  -- Cpu sends ack adr 1 for interrupt 0
      SysTimerIntxDN <= '0';
    end if;
  end process;

  SysTimerRegs : process (ClkxC, RstxR)
  begin
    if RstxR = '1' then
      SysTimerIntxDP <= '0';
      SysTimerxDP    <= 0;
      FreezexSP      <= '0';
    elsif ClkxC'event and ClkxC = '1' then
      SysTimerxDP    <= SysTimerxDN;
      SysTimerIntxDP <= SysTimerIntxDN;
      FreezexSP      <= FreezexSN;
    end if;
  end process;

  -- bus with all devices connected to the io port of the cpu
  LedxDO <= LedxD(7 downto 0);
  --LedxDO <= ProgMemAdrxD(15 downto 8) when SwitchxDI(3) = '0' else ProgMemAdrxD(7 downto 0);
  IoDevicesInst : ioBus
    port map (
      ClkxCI     => ClkxC,
      RstxRI     => RstxR,
      -- master interface from cpu
      CpuCycxSI  => WbIoCycxS,
      CpuStbxSI  => WbIoStbxS,
      CpuWexSI   => WbIoWexS,
      CpuAdrxDI  => WbIoAdrxD,
      CpuDatxDI  => WbIoDatMOSIxD,
      CpuDatxDO  => WbIoDatMISOxD,
      CpuAckxSO  => WbIoAckxS,
      CpuErrxSO  => open,
      CpuRtyxSO  => open,
      -- interrupts
      UartIntxSO => UartIntxS,
      SpiIntxSO  => SpiIntxS,
      -- uart io pins
      UartTxxDO  => UartTxxDO,
      UartRxxDI  => UartRxxDI,
--              UartRtsxSO      => UartRtsxSO,
--              UartCtsxSI      => UartCtsxSI,
      UartRtsxSO => open,
      UartCtsxSI => '1',
      -- spi pins
      SpiSckxSO  => SpiSckxSO,
      SpiMisoxDI => SpiMisoxDI,
      SpiMosixDO => SpiMosixDO,
      SpiSSxSO   => SpiSSxSO,
      -- gpio pins            
      SwitchxDI  => SwitchxDI,
      ButtonxDI  => ButtonxDI,
      LedxDO     => LedxD
      );

  dataBusInst : dataBus
    generic map (
      --IS_SIMULATION   => IS_SIMULATION
      IS_SIMULATION => "FALSE",
      SIMULATE_DRAM => false,
      SIMULATE_MAC  => false
      )
    port map (
      -- system singals for the dram controller (it does clock & reset handling)
      RstButtonxR     => resetSwitchxSB,
      BoardClkxCI     => xtalClock,
      --BoardClkxCIB    => xtalClockN,
      LaCoreClkxCI    => LaCoreClkxC,
      LaCoreClkxCBI   => LaCoreClkxCB,
      LaCoreRstxRI    => RstCorexR,
      RefClkxCO       => PllRefClkxC,
      McbReadyxSO     => McbReadyxS,
      -- wishbone clock and reset signals
      ClkxCI          => ClkxC,
      ClkxCIB         => ClkxCB,
      RstxRI          => RstxR,
      -- master interface from cpu
      CpuCycxSI       => WbRamCycxS,
      CpuStbxSI       => WbRamStbxS,
      CpuWexSI        => WbRamWexS,
      CpuAdrxDI       => WbRamAdrxD,
      CpuDatxDI       => WbRamDatMOSIxD,
      CpuDatxDO       => WbRamDatMISOxD,
      CpuAckxSO       => WbRamAckxS,
      CpuErrxSO       => open,
      CpuRtyxSO       => open,
      -- MII : phy <-> mac
      PhyTxClkxCI     => PHY_TXCLK,
      PhyTxdxDO       => PHY_TXD(3 downto 0),
      PhyTxEnxSO      => PHY_TXCTL_TXEN,
      PhyTxErrxSO     => PHY_TXER,
      -- MII RX
      PhyRxClkxCI     => PHY_RXCLK,
      PhyRxdxDI       => PHY_RXD(3 downto 0),
      PhyRxDvxSI      => PHY_RXCTL_RXDV,
      PhyRxErrxSI     => PHY_RXER,
--              PhyColxSI               => PHY_COL,
--              PhyCrsxSI               => PHY_CRS,
      PhyColxSI       => '0',
      PhyCrsxSI       => '0',
      -- MIIM
      PhyMdcxSO       => PHY_MDC,
      PhyMdioxDI      => PhyMdioxDI,
      PhyMdioxDO      => PhyMdioxDO,
      PhyMdioEnxSO    => PhyMdioEnxS,
      -- physical memory interface
      MemDQxDIO       => MemDQxDIO,
      MemAdrxDO       => MemAdrxDO,
      MemBaxDO        => MemBaxDO,
      MemRasxEO       => MemRasxEO,
      MemCasxEO       => MemCasxEO,
      MemWexEO        => MemWexEO,
      MemOdtxO        => MemOdtxO,
      MemCkexEO       => MemCkexEO,
      MemClkxCO       => MemClkxCO,
      MemClkxCOB      => MemClkxCOB,
      MemLDqsxEO      => MemLDqsxEO,
      MemLDqsxEOB     => MemLDqsxEOB,
      MemUDqsxEO      => MemUDqsxEO,
      MemUDqsxEOB     => MemUDqsxEOB,
      MemLdmxEO       => MemLdmxEO,
      MemUdmxEO       => MemUdmxEO,
      -- calibration (not actually logical signals)
      RzqxIO          => RzqxIO,
      ZioxIO          => ZioxIO,
      -- la core interface (to outside of fpga)
      exClockxCI      => exClockxCI,
      inputChxDI      => inputChxDI,
      testOutputxSO   => testOutputxSO,
      -- interrupts
      LaCoreIRQxSO    => LaCoreIRQxS,
      LaCoreIntAckxSI => LaCoreIntAckxS,
      EthIntxSO       => MacIntxS,
      DDmaIntxSO      => DDmaIntxS,
      CmpIntxSO       => CmpIntxS
      );

  -- tristate buffer for MIIM
  PhyMdioxDI          <= PHY_MDIO;
  PHY_MDIO            <= '0' when PhyMdioEnxS = '1' and PhyMdioxDO = '0' else 'Z';
  PHY_TXD(7 downto 4) <= "0000";
  PHY_RESET           <= RstxRB;

  -- use a flipflop to synchronize the phy interrupt request pin
  PhyIntxSN <= not PHY_INT;              -- Note: phy_int is active low
  process (ClkxC, RstxR)
  begin  -- process
    if ClkxC'event and ClkxC = '1' then  -- rising clock edge
      if RstxR = '1' then
        PhyIntxSP <= '0';
      else
        PhyIntxSP <= PhyIntxSN;
      end if;
    end if;
  end process;
  
end Behavioral;

