--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This is simple Wishbone bridge which translates an 8 bit slave to a 32 bit master device.
--* 4 consecutive accesses from
--* the 8 bit bus are transferred to one 32 bit access. The access on the 4 bit interface
--* have to be in little endian order and have to word aligned, i.e.
--*
--* 1st access:  rd/wr least  siginificant byte, Wb8AdrxDI(1 downto 0) = 00
--* 2nd access:  rd/wr lower  siginificant byte, Wb8AdrxDI(1 downto 0) = 01
--* 3rd access:  rd/wr higher siginificant byte, Wb8AdrxDI(1 downto 0) = 10
--* 4th access:  rd/wr most   siginificant byte, Wb8AdrxDI(1 downto 0) = 11
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_arith.all;


entity Wb8Wb32Bridge is
  port (
    -- common signals for both ports
    ClkxCI     : in  std_logic;
    RstxRI     : in  std_logic;
    -- 8 bit wide wishbone slave interface
    Wb8CycxSI  : in  std_logic;
    Wb8StbxSI  : in  std_logic;
    Wb8WexSI   : in  std_logic;
    Wb8AdrxDI  : in  std_logic_vector(15 downto 0);
    Wb8DatxDI  : in  std_logic_vector(7 downto 0);
    Wb8DatxDO  : out std_logic_vector(7 downto 0);
    Wb8AckxSO  : out std_logic;
    Wb8ErrxSO  : out std_logic;
    -- 32 bite wide wishbone master interface
    Wb32CycxSO : out std_logic;
    Wb32StbxSO : out std_logic;
    Wb32WexSO  : out std_logic;
    Wb32SelxSO : out std_logic_vector(3 downto 0);
    Wb32AdrxDO : out std_logic_vector(15 downto 0);
    Wb32DatxDI : in  std_logic_vector(31 downto 0);
    Wb32DatxDO : out std_logic_vector(31 downto 0);
    Wb32AckxSI : in  std_logic;
    Wb32ErrxSI : in  std_logic
    );
end Wb8Wb32Bridge;

architecture Behavioral of Wb8Wb32Bridge is

  signal WrDatRegxDN, WrDatRegxDP : std_logic_vector(31 downto 0);
  signal RdDatRegxDN, RdDatRegxDP : std_logic_vector(31 downto 0);
  signal RdDOutxDN, RdDOutxDP     : std_logic_vector(7 downto 0);
  signal AckxSN, AckxSP           : std_logic;
  signal ErrxSN, ErrxSP           : std_logic;
  signal CycStbxSN, CycStbxSP     : std_logic;
begin

  -- we only do 32 bit transfer on the wb32 interface
  Wb32SelxSO <= "1111";

  -- the address on the 32 bit bus is the same as on the 8 bit bus
  Wb32AdrxDO <= Wb8AdrxDI(15 downto 2) & "00";

  -- write or reads stay the same on the 32 bit bus as on the 8 bit
  Wb32WexSO <= Wb8WexSI;

  Wb32CycxSO <= CycStbxSP;
  Wb32StbxSO <= CycStbxSP;

  Wb8AckxSO <= AckxSP;
  Wb8ErrxSO <= ErrxSP;

  Wb32DatxDO <= WrDatRegxDP;
  Wb8DatxDO  <= RdDOutxDP;

  -- data input register for wb8 -> wb32 (write)
  process (Wb8CycxSI, Wb8StbxSI, Wb8WexSI, Wb8DatxDI, Wb8AdrxDI, WrDatRegxDP)
    variable ByteAdr : integer range 0 to 4;
  begin
    -- default assignment = don't change reg
    WrDatRegxDN <= WrDatRegxDP;
    -- check for write access on wb 8 
    if Wb8CycxSI = '1' and Wb8StbxSI = '1' and Wb8WexSI = '1' then
      ByteAdr                                       := conv_integer(unsigned(Wb8AdrxDI(1 downto 0)))+1;  -- get the offset of the byte within the word
      WrDatRegxDN(ByteAdr*8-1 downto (ByteAdr-1)*8) <= Wb8DatxDI;
    end if;
  end process;

  -- data input register for wb32 -> wb8 (read)
  process (Wb32AckxSI, RdDatRegxDP, Wb32DatxDI)
  begin
    -- latch data when 32 bit slave asserts ack
    if Wb32AckxSI = '1' then
      RdDatRegxDN <= Wb32DatxDI;
    else
      RdDatRegxDN <= RdDatRegxDP;
    end if;
  end process;

  -- data output mux for 8 bit read
  process (Wb8AdrxDI, RdDatRegxDP, Wb32AckxSI, Wb8WexSI, Wb32DatxDI, RdDoutxDP)
    variable ByteAdr : integer range 0 to 4;
  begin
    ByteAdr   := conv_integer(unsigned(Wb8AdrxDI(1 downto 0)))+1;  -- get the offset of the byte within the word
    -- foreward input data coming from wb32 diretly to the output for read access
    RdDoutxDN <= RdDoutxDP;
    if Wb32AckxSI = '1' and Wb8WexSI = '0' then
      RdDoutxDN <= Wb32DatxDI(ByteAdr*8-1 downto (ByteAdr-1)*8);
    else
      RdDoutxDN <= RdDatRegxDP(ByteAdr*8-1 downto (ByteAdr-1)*8);
    end if;
  end process;

  -- ack signal generation (8 bit)
  -- cyc & stb generation (32 bit)
  process (Wb8CycxSI, Wb8StbxSI, Wb8WexSI, Wb8AdrxDI, Wb32AckxSI, Wb32ErrxSI, AckxSP)
    variable ByteAdr : integer range 0 to 4;
  begin
    ByteAdr   := conv_integer(unsigned(Wb8AdrxDI(1 downto 0)));  -- get the offset of the byte within the word
    AckxSN    <= '0';                   -- default: no ack
    ErrxSN    <= '0';
    CycStbxSN <= '0';
    --Wb32StbxSO <= '0';
    -- check for valid bus access
    if Wb8CycxSI = '1' and Wb8StbxSI = '1' then
      if Wb8WexSI = '1' then
        -- we have a write, answer with an immediate ack for bytes 0 to 2, byte 3 triggers the write on 32 bit bus
        if ByteAdr < 3 then
          AckxSN <= not AckxSP;         -- assert ack only for one cycle 
        else
          AckxSN    <= Wb32AckxSI;
          ErrxSN    <= Wb32ErrxSI;
          CycStbxSN <= not (Wb32AckxSI or Wb32ErrxSI) and (not AckxSP);  -- deassert cyc & stb when ack (or err) is asserted
                                        --Wb32StbxSO <= not Wb32AckxSI;
        end if;
      else
        -- read: access to first byte (0) triggers read on 32 bit bus
        if ByteAdr > 0 then
          AckxSN <= not AckxSP;         -- assert ack only for one cycle 
        else
          AckxSN    <= Wb32AckxSI;
          CycStbxSN <= not Wb32AckxSI and (not AckxSP);
                                        --Wb32StbxSO <= not Wb32AckxSI;
        end if;
      end if;
    end if;
  end process;

  -- registers
  process (ClkxCI)
  begin
    if ClkxCI'event and ClkxCI = '1' then
      if RstxRI = '1' then
        WrDatRegxDP <= (others => '0');
        RdDatRegxDP <= (others => '0');
        RdDOutxDP   <= (others => '0');
        AckxSP      <= '0';
        ErrxSP      <= '0';
        CycStbxSP   <= '0';
      else
        WrDatRegxDP <= WrDatRegxDN;
        RdDatRegxDP <= RdDatRegxDN;
        RdDOutxDP   <= RdDOutxDN;
        AckxSP      <= AckxSN;
        ErrxSP      <= ErrxSN;
        CycStbxSP   <= CycStbxSN;
      end if;
    end if;
  end process;

end Behavioral;

