--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This Module fits the sump-logic-analyzer with a wishbone-compatible interface in order to be able
--* to communicate with the CPU and to control the LA with the AVR-CPU.
--*
--* the data on the 8 bit bus are transferred to one 32 bit access. 
--*
--* 1st access:  rd/wr least  siginificant byte, Wb8AdrxDI(1 downto 0) = 00
--* 2nd access:  rd/wr lower  siginificant byte, Wb8AdrxDI(1 downto 0) = 01
--* 3st access:  rd/wr higher siginificant byte, Wb8AdrxDI(1 downto 0) = 10
--* 4st access:  rd/wr most   siginificant byte, Wb8AdrxDI(1 downto 0) = 11
--* 5th access: access to the opcode register. Address: d"4" After this access has been performed, the execute-signal will be asserted for 1 clock cycle.
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_arith.all;


entity Wb_sump_interface is
  port(
    --These are the connections for the Wishbone-Bus:(slave, 8 bit)
    ClkxCI      : in  std_logic;        -- wishbone clock from the AVR
    RstxRI      : in  std_logic;        --global reset
    Wb_cycxSI   : in  std_logic;
    Wb_stbxSI   : in  std_logic;
    Wb_wexSI    : in  std_logic;
    Wb_ackxSO   : out std_logic;
    Wb_errxSO   : out std_logic;
    Wb_adrxDI   : in  std_logic_vector(3 downto 0);   -- 4bit address
    Wb_datxDI   : in  std_logic_vector(7 downto 0);   -- 8bit data
    Wb_datxDO   : out std_logic_vector(7 downto 0);
    --This is the interface to the logic analyzer core:
    Clk_sumpxCI : in  std_logic;        -- sampling clock
    Rst_sumpxRI : in  std_logic;
    channelsxDI : in  std_logic_vector(31 downto 0);  -- current state of sampling channels
    cmd_dataxDO : out std_logic_vector(31 downto 0);  -- 32bit data to transfer data to the sump
    opcodexDO   : out std_logic_vector(7 downto 0);   --opcode - for control
    executexSO  : out std_logic  --after the transaction of the opcode-byte, this signal will go high for 1 cycle to indicate a valid opcode to the core.
    );
end Wb_sump_interface;

architecture Behavioral of Wb_sump_interface is

  --Used to create the execute-pulse:
  signal ExecutexSN, ExecutexSP         : std_logic;  --40MHz side
  signal ExecuteSyncxSN, ExecuteSyncxSP : std_logic;  --100MHz side - clock domain crossing-ff

  signal ExecutepulsegenxSN, ExecutepulsegenxSP : std_logic;  --100MHz side - needed to generate the execute-pulse (1 clock cycle long)
  signal ExecutepulsexSN, ExecutepulsexSP       : std_logic;  --This is the 100MHz-Pulse.

  --Data signals:
  signal SumpDataxDN, SumpDataxDP             : std_logic_vector(31 downto 0);
  signal SumpOpcodexDN, SumpOpcodexDP         : std_logic_vector(7 downto 0);
  --Data synchronization signals: (clock domain crossing)
  signal SumpDataSyncxDN, SumpDataSyncxDP     : std_logic_vector(31 downto 0);  -- for clock domain crossing
  signal SumpOpcodeSyncxDN, SumpOpcodeSyncxDP : std_logic_vector(7 downto 0);

  signal ChannelsWbxDN, ChannelsWbxDP : std_logic_vector (31 downto 0) := (others => '0');

begin

--Handle the data-writes to the sump:
  Data_collect : process(Wb_cycxSI, Wb_stbxSI, Wb_wexSI, Wb_datxDI, Wb_adrxDI, SumpDataxDP, SumpOpcodexDP, ExecutexSP, ChannelsWbxDP) is
    variable ByteAdr : integer range 0 to 4;
    variable DataAdr : integer range 0 to 15;
  begin
    --DefAss:
    SumpDataxDN   <= SumpDataxDP;
    SumpOpcodexDN <= SumpOpcodexDP;
    ExecutexSN    <= '0';

    ByteAdr := conv_integer(unsigned(Wb_adrxDI(1 downto 0)))+1;  --get the offset of the byte within the word
    DataAdr := conv_integer(unsigned(Wb_adrxDI(3 downto 0)));  --get the address itself into the integer

    --if there is a write:
    if Wb_cycxSI = '1' and Wb_stbxSI = '1' and Wb_wexSI = '1' then
      if DataAdr = 4 then  --there is an access to the opcode-register
        SumpOpcodexDN <= Wb_datxDI;  --transfer the data to the opcode-register
      elsif DataAdr >= 0 and DataAdr <= 3 then  --there is an access to one byte of the 32bit-sump-data-register
        SumpDataxDN(ByteAdr*8-1 downto (ByteAdr-1)*8) <= Wb_datxDI;  --transfer the data to the appropriate location in the 32bit-Register
      end if;

      if Wb_cycxSI = '1' and DataAdr = 4 and ExecutexSP = '0' then  --generate the execute-pulse:(a few 40MHz-cycles)
        ExecutexSN <= '1';
      end if;
    end if;

    -- select the output data based on the address
    case DataAdr is
      when 8      => Wb_datxDO <= ChannelsWbxDP(7 downto 0);
      when 9      => Wb_datxDO <= ChannelsWbxDP(15 downto 8);
      when 10     => Wb_datxDO <= ChannelsWbxDP(23 downto 16);
      when 11     => Wb_datxDO <= ChannelsWbxDP(31 downto 24);
      when others => Wb_datxDO <= x"00";
    end case;
    
  end process Data_collect;

--Handle Wb-Accesses:
  Wb_Handler : process(Wb_cycxSI, Wb_stbxSI) is
  begin
    --generate the ack:
    Wb_ackxSO <= Wb_cycxSI and Wb_stbxSI;
    --no Errors:
    Wb_errxSO <= '0';
  end process Wb_Handler;

-- Transfer sampling channels from sampling clock to wishbone clock domain
  ChannelsWbxDN <= channelsxDI;
  process (ClkxCI)
  begin
    if ClkxCI'event and ClkxCI = '1' then
      ChannelsWbxDP <= ChannelsWbxDN;
    end if;
  end process;


--Forward the data to the core : ==> Clock domain crossing
  Sump_Data_Update : process(SumpDataxDP, SumpDataSyncxDP, SumpOpcodeSyncxDP, SumpOpcodexDP, ExecutexSP, ExecutepulsexSP) is
  begin
    --use sync-register for clock domain crossing:
    SumpDataSyncxDN   <= SumpDataxDP;
    SumpOpcodeSyncxDN <= SumpOpcodexDP;
    ExecutesyncxSN    <= ExecutexSP;

    cmd_dataxDO <= SumpDataSyncxDP;     --the core always gets the data
    opcodexDO   <= SumpOpcodeSyncxDP;   --the core always gets the opcode

    executexSO <= ExecutepulsexSP;
  end process Sump_Data_Update;


--generate the execute-pulse on the 200MHz-Side:
  Execute_Gen : process(ExecutesyncxSP, ExecutepulsexSP, ExecutepulsegenxSP) is
  begin
    ExecutepulsegenxSN <= ExecutesyncxSP;
    ExecutepulsexSN    <= ExecutepulsexSP;

    if ExecutepulsegenxSP = '0' and ExecutesyncxSP = '1' then
      ExecutepulsexSN <= '1';
    else
      ExecutepulsexSN <= '0';
    end if;
    
  end process Execute_Gen;


--Update the FFs:  (40MHz)
  Update_Registers : process(ClkxCI) is
  begin
    if rising_edge(ClkxCI) then
      if RstxRI = '1' then
        SumpDataxDP   <= (others => '0');
        SumpOpcodexDP <= (others => '0');
        ExecutexSP    <= '0';
      else
        SumpDataxDP   <= SumpDataxDN;
        SumpOpcodexDP <= SumpOpcodexDN;
        ExecutexSP    <= ExecutexSN;
      end if;
    end if;
  end process Update_Registers;

--Update Clock-Domain-crossing-FFs:
  Update_Sump_Sync_Regs : process(Clk_sumpxCI) is
  begin
    if rising_edge(Clk_sumpxCI) then
      if Rst_sumpxRI = '1' then
        SumpDataSyncxDP    <= (others => '0');
        SumpOpcodeSyncxDP  <= (others => '0');
        ExecuteSyncxSP     <= '0';
        ExecutepulsegenxSP <= '0';
        ExecutepulsexSP    <= '0';
      else
        SumpDataSyncxDP    <= SumpDataSyncxDN;
        SumpOpcodeSyncxDP  <= SumpOpcodeSyncxDN;
        ExecuteSyncxSP     <= ExecuteSyncxSN;
        ExecutepulsegenxSP <= ExecutepulsegenxSN;
        ExecutepulsexSP    <= ExecutepulsexSN;
      end if;
    end if;
  end process Update_Sump_Sync_Regs;
  
  
end Behavioral;

