--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--* Version 1.1 - 2012/11/04 - LS
--*   added LZRW compressor unit
--*   added genercis to disable certain part in simulation (speed up due to reduced code size)
--*   added DRAM controller interrupt line
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* Implements the AVRs data bus. The logical bus is implemented as two sperate
--* physical busses, one is 8 bits wide, the other is 32 bits. The two busses are
--* coupled using a wishbone-wishbone-bridge.
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;


entity dataBus is
  generic (
    IS_SIMULATION : string  := "FALSE";
    SIMULATE_DRAM : boolean := true;
    SIMULATE_MAC  : boolean := true);
  port (  -- system singals for the dram controller (it does clock & reset handling)
    RstButtonxR   : in  std_logic;
    BoardClkxCI   : in  std_logic;
    --BoardClkxCIB      : in  std_logic;
    LaCoreClkxCI  : in  std_logic;
    LaCoreClkxCBI : in  std_logic;
    LaCoreRstxRI  : in  std_logic;
    RefClkxCO     : out std_logic;
    McbReadyxSO   : out std_logic;

    -- wishbone clock and reset signals
    ClkxCI  : in std_logic;
    ClkxCIB : in std_logic;
    RstxRI  : in std_logic;

    -- master interface from cpu
    CpuCycxSI : in  std_logic;
    CpuStbxSI : in  std_logic;
    CpuWexSI  : in  std_logic;
    CpuAdrxDI : in  std_logic_vector(15 downto 0);
    CpuDatxDI : in  std_logic_vector(7 downto 0);
    CpuDatxDO : out std_logic_vector(7 downto 0);
    CpuAckxSO : out std_logic;
    CpuErrxSO : out std_logic;
    CpuRtyxSO : out std_logic;

    -- MII : phy <-> mac
    PhyTxClkxCI  : in  std_logic;
    PhyTxdxDO    : out std_logic_vector(3 downto 0);
    PhyTxEnxSO   : out std_logic;
    PhyTxErrxSO  : out std_logic;
    -- MII RX
    PhyRxClkxCI  : in  std_logic;
    PhyRxdxDI    : in  std_logic_vector(3 downto 0);
    PhyRxDvxSI   : in  std_logic;
    PhyRxErrxSI  : in  std_logic;
    PhyColxSI    : in  std_logic;
    PhyCrsxSI    : in  std_logic;
    -- MIIM
    PhyMdcxSO    : out std_logic;
    PhyMdioxDI   : in  std_logic;
    PhyMdioxDO   : out std_logic;
    PhyMdioEnxSO : out std_logic;

    -- physical memory interface to DDR2 SDRAM (sample memory)
    MemDQxDIO   : inout std_logic_vector(15 downto 0);
    MemAdrxDO   : out   std_logic_vector(12 downto 0);
    MemBaxDO    : out   std_logic_vector(2 downto 0);
    MemRasxEO   : out   std_logic;
    MemCasxEO   : out   std_logic;
    MemWexEO    : out   std_logic;
    MemOdtxO    : out   std_logic;
    MemCkexEO   : out   std_logic;
    MemClkxCO   : out   std_logic;
    MemClkxCOB  : out   std_logic;
    MemLDqsxEO  : inout std_logic;
    MemLDqsxEOB : inout std_logic;
    MemUDqsxEO  : inout std_logic;
    MemUDqsxEOB : inout std_logic;
    MemLdmxEO   : out   std_logic;
    MemUdmxEO   : out   std_logic;
    -- calibration (not actually logical signals)
    RzqxIO      : inout std_logic;
    ZioxIO      : inout std_logic;

    -- la core interface (to outside of fpga)
    exClockxCI    : in  std_logic;
    inputChxDI    : in  std_logic_vector(31 downto 0);
    testOutputxSO : out std_logic;

    -- interrupts
    LaCoreIRQxSO    : out std_logic;
    LaCoreIntAckxSI : in  std_logic;
    EthIntxSO       : out std_logic;
    DDmaIntxSO      : out std_logic;
    CmpIntxSO       : out std_logic
    );
end dataBus;

architecture Behavioral of dataBus is

  component DataMem is
    port(ClkxCI      : in  std_logic;
         ClkxCIB     : in  std_logic;
         RstxRI      : in  std_logic;
         Wb8CycxSI   : in  std_logic;
         Wb8StbxSI   : in  std_logic;
         Wb8WexSI    : in  std_logic;
         Wb8AdrxDI   : in  std_logic_vector (13 downto 0);
         Wb8DataxDI  : in  std_logic_vector (7 downto 0);
         Wb8DataxDO  : out std_logic_vector (7 downto 0);
         Wb8AckxSO   : out std_logic;
         Wb32CycxSI  : in  std_logic;
         Wb32StbxSI  : in  std_logic;
         Wb32WexSI   : in  std_logic;
         Wb32SelxSI  : in  std_logic_vector (3 downto 0);
         Wb32AdrxDI  : in  std_logic_vector (13 downto 2);
         Wb32DataxDI : in  std_logic_vector (31 downto 0);
         Wb32DataxDO : out std_logic_vector (31 downto 0);
         Wb32AckxSO  : out std_logic);
  end component;

  component Wb8Wb32Bridge
    port(
      ClkxCI     : in  std_logic;
      RstxRI     : in  std_logic;
      Wb8CycxSI  : in  std_logic;
      Wb8StbxSI  : in  std_logic;
      Wb8WexSI   : in  std_logic;
      Wb8AdrxDI  : in  std_logic_vector(15 downto 0);
      Wb8DatxDI  : in  std_logic_vector(7 downto 0);
      Wb8DatxDO  : out std_logic_vector(7 downto 0);
      Wb8AckxSO  : out std_logic;
      Wb8ErrxSO  : out std_logic;
      Wb32CycxSO : out std_logic;
      Wb32StbxSO : out std_logic;
      Wb32WexSO  : out std_logic;
      Wb32SelxSO : out std_logic_vector(3 downto 0);
      Wb32AdrxDO : out std_logic_vector(15 downto 0);
      Wb32DatxDI : in  std_logic_vector(31 downto 0);
      Wb32DatxDO : out std_logic_vector(31 downto 0);
      Wb32AckxSI : in  std_logic;
      Wb32ErrxSI : in  std_logic
      );
  end component;

  component eth_top is
    port (
      wb_clk_i      : in  std_logic;
      wb_rst_i      : in  std_logic;
      -- wishbone slave interface
      wb_dat_i      : in  std_logic_vector(31 downto 0);
      wb_dat_o      : out std_logic_vector(31 downto 0);
      wb_adr_i      : in  std_logic_vector(11 downto 2);
      wb_sel_i      : in  std_logic_vector(3 downto 0);
      wb_we_i       : in  std_logic;
      wb_cyc_i      : in  std_logic;
      wb_stb_i      : in  std_logic;
      wb_ack_o      : out std_logic;
      wb_err_o      : out std_logic;
      -- WISHBONE master
      m_wb_adr_o    : out std_logic_vector(31 downto 0);
      m_wb_sel_o    : out std_logic_vector(3 downto 0);
      m_wb_we_o     : out std_logic;
      m_wb_dat_o    : out std_logic_vector(31 downto 0);
      m_wb_dat_i    : in  std_logic_vector(31 downto 0);
      m_wb_cyc_o    : out std_logic;
      m_wb_stb_o    : out std_logic;
      m_wb_ack_i    : in  std_logic;
      m_wb_err_i    : in  std_logic;
      -- MII TX
      mtx_clk_pad_i : in  std_logic;
      mtxd_pad_o    : out std_logic_vector(3 downto 0);
      mtxen_pad_o   : out std_logic;
      mtxerr_pad_o  : out std_logic;
      -- MII RX
      mrx_clk_pad_i : in  std_logic;
      mrxd_pad_i    : in  std_logic_vector(3 downto 0);
      mrxdv_pad_i   : in  std_logic;
      mrxerr_pad_i  : in  std_logic;
      mcoll_pad_i   : in  std_logic;
      mcrs_pad_i    : in  std_logic;
      -- MIIM
      mdc_pad_o     : out std_logic;
      md_pad_i      : in  std_logic;
      md_pad_o      : out std_logic;
      md_padoe_o    : out std_logic;
      -- general
      int_o         : out std_logic
      );
  end component;

  component dramCtrl
    generic(
      IS_SIMULATION : string;
      WB_BURST_LEN  : integer);
    port(
      -- system interface
      ResetSwitchxR : in    std_logic;  -- reset switch input (this comes from a push button)
      RstSocxRI     : in    std_logic;  -- logic reset for wishbone bus
      RstCorexRI    : in    std_logic;  -- logic reset for analyzer core (sampling clock domain)
      --RstxRO                  : out STD_LOGIC;        -- system reset output (held high during pll locking)
      CalibDonexSO  : out   std_logic;  -- get high when the dram has been initialized and is ready to receive data
      -- clocks
      BoardClkxCI   : in    std_logic;  -- positive of differential on board clock (200MHz)
      --BoardClkxCIB    : in  STD_LOGIC;     -- negative of differential on board clock (200MHz)
      RefClkxCO     : out   std_logic;  -- PLL generated reference clock for the system (25MHz)
      -- interface to sampling core
      CoreClkxC     : in    std_logic;  -- samplin system clock 100MHz
      DataxDI       : in    std_logic_vector(31 downto 0);
      WritexSI      : in    std_logic;
      -- wishbone dma master interface
      WbClkxCI      : in    std_logic;  -- wishbone clock (master & slave)
      MaCycxSO      : out   std_logic;
      MaStbxSO      : out   std_logic;
      MaWexSO       : out   std_logic;
      MaSelxDO      : out   std_logic_vector(3 downto 0);
      MaAdrxDO      : out   std_logic_vector(31 downto 0);
      MaDatxDO      : out   std_logic_vector(31 downto 0);
      MaDatxDI      : in    std_logic_vector(31 downto 0);
      MaAckxSI      : in    std_logic;
      MaErrxSI      : in    std_logic;
      -- wishbone configuration slave interface
      SlCycxSI      : in    std_logic;
      SlStbxSI      : in    std_logic;
      SlWexSI       : in    std_logic;
      SlAdrxDI      : in    std_logic_vector(4 downto 2);
      SlDatxDI      : in    std_logic_vector(31 downto 0);
      SlDatxDO      : out   std_logic_vector(31 downto 0);
      SlAckxSO      : out   std_logic;
      IntxSO        : out   std_logic;
      -- memory interface
      MemDQxDIO     : inout std_logic_vector(15 downto 0);
      MemAdrxDO     : out   std_logic_vector(12 downto 0);
      MemBaxDO      : out   std_logic_vector(2 downto 0);
      MemRasxEO     : out   std_logic;
      MemCasxEO     : out   std_logic;
      MemWexEO      : out   std_logic;
      MemOdtxO      : out   std_logic;
      MemCkexEO     : out   std_logic;
      MemClkxCO     : out   std_logic;
      MemClkxCOB    : out   std_logic;
      MemLDqsxEO    : inout std_logic;
      MemLDqsxEOB   : inout std_logic;
      MemUDqsxEO    : inout std_logic;
      MemUDqsxEOB   : inout std_logic;
      MemLdmxEO     : out   std_logic;
      MemUdmxEO     : out   std_logic;
      RzqxIO        : inout std_logic;
      ZioxIO        : inout std_logic
      );
  end component;

  -- the logic analyzer hardware
  component la
    port(
      reset                : in  std_logic;
      clock                : in  std_logic;  -- LA base clock, 200MHz
      clockB               : in  std_logic;
      AVR_Clock            : in  std_logic;
      AVR_Reset            : in  std_logic;
      exClock              : in  std_logic;
      input                : in  std_logic_vector(31 downto 0);
      testOutput           : out std_logic;
      memoryClkxCO         : out std_logic;
      memoryOut            : out std_logic_vector(31 downto 0);
      memoryWrite          : out std_logic;
      -- Wishbone Interface for configuration
      Wb_cycxSI            : in  std_logic;
      Wb_stbxSI            : in  std_logic;
      Wb_wexSI             : in  std_logic;
      Wb_ackxSO            : out std_logic;
      Wb_errxSO            : out std_logic;
      Wb_adrxDI            : in  std_logic_vector(3 downto 0);
      Wb_datxDI            : in  std_logic_vector(7 downto 0);
      Wb_datxDO            : out std_logic_vector(7 downto 0);
      --AVR-Interrupt:
      sample_finish_IRQxSO : out std_logic;  --interrupt-output-flag for the AVR. (finished sampling)
      sample_clrflagxSI    : in  std_logic  --input to clear the interrupt flag
      );
  end component;

  component CompressorTop
    port (
      ClkxCI   : in  std_logic;
      RstxRI   : in  std_logic;
      SlCycxSI : in  std_logic;
      SlStbxSI : in  std_logic;
      SlWexSI  : in  std_logic;
      SlSelxDI : in  std_logic_vector(3 downto 0);
      SlAdrxDI : in  std_logic_vector(4 downto 2);
      SlDatxDI : in  std_logic_vector(31 downto 0);
      SlDatxDO : out std_logic_vector(31 downto 0);
      SlAckxSO : out std_logic;
      SlErrxSO : out std_logic;
      IntxSO   : out std_logic;
      MaCycxSO : out std_logic;
      MaStbxSO : out std_logic;
      MaWexSO  : out std_logic;
      MaSelxDO : out std_logic_vector(3 downto 0);
      MaAdrxDO : out std_logic_vector(31 downto 0);
      MaDatxDO : out std_logic_vector(31 downto 0);
      MaDatxDI : in  std_logic_vector(31 downto 0);
      MaAckxSI : in  std_logic;
      MaErrxSI : in  std_logic);
  end component;

  component wb_conbus_top is
    generic (
      dw         : integer;
      aw         : integer;
      s0_addr_w  : integer;
      s0_addr    : integer;
      s1_addr_w  : integer;
      s1_addr    : integer;
      s27_addr_w : integer;
      s2_addr    : integer;
      s3_addr    : integer;
      s4_addr    : integer;
      s5_addr    : integer;
      s6_addr    : integer;
      s7_addr    : integer);
    port (
      clk_i    : in  std_logic;
      rst_i    : in  std_logic;
      -- Master 0 Interface
      m0_dat_i : in  std_logic_vector(dw-1 downto 0);
      m0_dat_o : out std_logic_vector(dw-1 downto 0);
      m0_adr_i : in  std_logic_vector(aw-1 downto 0);
      m0_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m0_we_i  : in  std_logic;
      m0_cyc_i : in  std_logic;
      m0_stb_i : in  std_logic;
      m0_ack_o : out std_logic;
      m0_err_o : out std_logic;
      m0_rty_o : out std_logic;
      m0_cab_i : in  std_logic;

      -- Master 1 Interface
      m1_dat_i : in  std_logic_vector(dw-1 downto 0);
      m1_dat_o : out std_logic_vector(dw-1 downto 0);
      m1_adr_i : in  std_logic_vector(aw-1 downto 0);
      m1_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m1_we_i  : in  std_logic;
      m1_cyc_i : in  std_logic;
      m1_stb_i : in  std_logic;
      m1_ack_o : out std_logic;
      m1_err_o : out std_logic;
      m1_rty_o : out std_logic;
      m1_cab_i : in  std_logic;

      -- Master 2 Interface
      m2_dat_i : in  std_logic_vector(dw-1 downto 0);
      m2_dat_o : out std_logic_vector(dw-1 downto 0);
      m2_adr_i : in  std_logic_vector(aw-1 downto 0);
      m2_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m2_we_i  : in  std_logic;
      m2_cyc_i : in  std_logic;
      m2_stb_i : in  std_logic;
      m2_ack_o : out std_logic;
      m2_err_o : out std_logic;
      m2_rty_o : out std_logic;
      m2_cab_i : in  std_logic;

      -- Master 3 Interface
      m3_dat_i : in  std_logic_vector(dw-1 downto 0);
      m3_dat_o : out std_logic_vector(dw-1 downto 0);
      m3_adr_i : in  std_logic_vector(aw-1 downto 0);
      m3_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m3_we_i  : in  std_logic;
      m3_cyc_i : in  std_logic;
      m3_stb_i : in  std_logic;
      m3_ack_o : out std_logic;
      m3_err_o : out std_logic;
      m3_rty_o : out std_logic;
      m3_cab_i : in  std_logic;

      -- Master 4 Interface
      m4_dat_i : in  std_logic_vector(dw-1 downto 0);
      m4_dat_o : out std_logic_vector(dw-1 downto 0);
      m4_adr_i : in  std_logic_vector(aw-1 downto 0);
      m4_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m4_we_i  : in  std_logic;
      m4_cyc_i : in  std_logic;
      m4_stb_i : in  std_logic;
      m4_ack_o : out std_logic;
      m4_err_o : out std_logic;
      m4_rty_o : out std_logic;
      m4_cab_i : in  std_logic;

      -- Master 5 Interface
      m5_dat_i : in  std_logic_vector(dw-1 downto 0);
      m5_dat_o : out std_logic_vector(dw-1 downto 0);
      m5_adr_i : in  std_logic_vector(aw-1 downto 0);
      m5_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m5_we_i  : in  std_logic;
      m5_cyc_i : in  std_logic;
      m5_stb_i : in  std_logic;
      m5_ack_o : out std_logic;
      m5_err_o : out std_logic;
      m5_rty_o : out std_logic;
      m5_cab_i : in  std_logic;

      -- Master 6 Interface
      m6_dat_i : in  std_logic_vector(dw-1 downto 0);
      m6_dat_o : out std_logic_vector(dw-1 downto 0);
      m6_adr_i : in  std_logic_vector(aw-1 downto 0);
      m6_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m6_we_i  : in  std_logic;
      m6_cyc_i : in  std_logic;
      m6_stb_i : in  std_logic;
      m6_ack_o : out std_logic;
      m6_err_o : out std_logic;
      m6_rty_o : out std_logic;
      m6_cab_i : in  std_logic;

      -- Master 7 Interface
      m7_dat_i : in  std_logic_vector(dw-1 downto 0);
      m7_dat_o : out std_logic_vector(dw-1 downto 0);
      m7_adr_i : in  std_logic_vector(aw-1 downto 0);
      m7_sel_i : in  std_logic_vector((dw/8)-1 downto 0);
      m7_we_i  : in  std_logic;
      m7_cyc_i : in  std_logic;
      m7_stb_i : in  std_logic;
      m7_ack_o : out std_logic;
      m7_err_o : out std_logic;
      m7_rty_o : out std_logic;
      m7_cab_i : in  std_logic;

      --        // Slave 0 Interface
      s0_dat_i : in  std_logic_vector(dw-1 downto 0);
      s0_dat_o : out std_logic_vector(dw-1 downto 0);
      s0_adr_o : out std_logic_vector(aw-1 downto 0);
      s0_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s0_we_o  : out std_logic;
      s0_cyc_o : out std_logic;
      s0_stb_o : out std_logic;
      s0_ack_i : in  std_logic;
      s0_err_i : in  std_logic;
      s0_rty_i : in  std_logic;
      s0_cab_o : out std_logic;

      --        // Slave 1 Interface
      s1_dat_i : in  std_logic_vector(dw-1 downto 0);
      s1_dat_o : out std_logic_vector(dw-1 downto 0);
      s1_adr_o : out std_logic_vector(aw-1 downto 0);
      s1_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s1_we_o  : out std_logic;
      s1_cyc_o : out std_logic;
      s1_stb_o : out std_logic;
      s1_ack_i : in  std_logic;
      s1_err_i : in  std_logic;
      s1_rty_i : in  std_logic;
      s1_cab_o : out std_logic;

      --        // Slave 2 Interface
      s2_dat_i : in  std_logic_vector(dw-1 downto 0);
      s2_dat_o : out std_logic_vector(dw-1 downto 0);
      s2_adr_o : out std_logic_vector(aw-1 downto 0);
      s2_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s2_we_o  : out std_logic;
      s2_cyc_o : out std_logic;
      s2_stb_o : out std_logic;
      s2_ack_i : in  std_logic;
      s2_err_i : in  std_logic;
      s2_rty_i : in  std_logic;
      s2_cab_o : out std_logic;

      --        // Slave 3 Interface
      s3_dat_i : in  std_logic_vector(dw-1 downto 0);
      s3_dat_o : out std_logic_vector(dw-1 downto 0);
      s3_adr_o : out std_logic_vector(aw-1 downto 0);
      s3_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s3_we_o  : out std_logic;
      s3_cyc_o : out std_logic;
      s3_stb_o : out std_logic;
      s3_ack_i : in  std_logic;
      s3_err_i : in  std_logic;
      s3_rty_i : in  std_logic;
      s3_cab_o : out std_logic;

      --        // Slave 4 Interface
      s4_dat_i : in  std_logic_vector(dw-1 downto 0);
      s4_dat_o : out std_logic_vector(dw-1 downto 0);
      s4_adr_o : out std_logic_vector(aw-1 downto 0);
      s4_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s4_we_o  : out std_logic;
      s4_cyc_o : out std_logic;
      s4_stb_o : out std_logic;
      s4_ack_i : in  std_logic;
      s4_err_i : in  std_logic;
      s4_rty_i : in  std_logic;
      s4_cab_o : out std_logic;

      --        // Slave 5 Interface
      s5_dat_i : in  std_logic_vector(dw-1 downto 0);
      s5_dat_o : out std_logic_vector(dw-1 downto 0);
      s5_adr_o : out std_logic_vector(aw-1 downto 0);
      s5_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s5_we_o  : out std_logic;
      s5_cyc_o : out std_logic;
      s5_stb_o : out std_logic;
      s5_ack_i : in  std_logic;
      s5_err_i : in  std_logic;
      s5_rty_i : in  std_logic;
      s5_cab_o : out std_logic;

      --        // Slave 6 Interface
      s6_dat_i : in  std_logic_vector(dw-1 downto 0);
      s6_dat_o : out std_logic_vector(dw-1 downto 0);
      s6_adr_o : out std_logic_vector(aw-1 downto 0);
      s6_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s6_we_o  : out std_logic;
      s6_cyc_o : out std_logic;
      s6_stb_o : out std_logic;
      s6_ack_i : in  std_logic;
      s6_err_i : in  std_logic;
      s6_rty_i : in  std_logic;
      s6_cab_o : out std_logic;

      --        // Slave 7 Interface
      s7_dat_i : in  std_logic_vector(dw-1 downto 0);
      s7_dat_o : out std_logic_vector(dw-1 downto 0);
      s7_adr_o : out std_logic_vector(aw-1 downto 0);
      s7_sel_o : out std_logic_vector((dw/8)-1 downto 0);
      s7_we_o  : out std_logic;
      s7_cyc_o : out std_logic;
      s7_stb_o : out std_logic;
      s7_ack_i : in  std_logic;
      s7_err_i : in  std_logic;
      s7_rty_i : in  std_logic;
      s7_cab_o : out std_logic
      );
  end component;

  -- 8 bit la core config interface
  signal LaCoreCycxS     : std_logic;
  signal LaCoreStbxS     : std_logic;
  signal LaCoreWexS      : std_logic;
  signal LaCoreAckxS     : std_logic;
  signal LaCoreErrxS     : std_logic;
  signal LaCoreAdrxD     : std_logic_vector(15 downto 0);
  signal LaCoreDatMOSIxD : std_logic_vector(7 downto 0);
  signal LaCoreDatMISOxD : std_logic_vector(7 downto 0);
  signal LaCoreIRQxS     : std_logic;
  signal LaCoreIntClrxS  : std_logic;

  -- 8 bit interface of wb-wb bridge
  signal Brdg8CycxS     : std_logic;
  signal Brdg8StbxS     : std_logic;
  signal Brdg8WexS      : std_logic;
  signal Brdg8AdrxD     : std_logic_vector(15 downto 0);
  signal Brdg8DatMISOxD : std_logic_vector(7 downto 0);
  signal Brdg8DatMOSIxD : std_logic_vector(7 downto 0);
  signal Brdg8AckxS     : std_logic;
  signal Brdg8ErrxS     : std_logic;

  -- 32 bit interface of wb-wb bridge
  signal Brdg32CycxS     : std_logic;
  signal Brdg32StbxS     : std_logic;
  signal Brdg32WexS      : std_logic;
  signal Brdg32SelxS     : std_logic_vector(3 downto 0);
  signal Brdg32AdrxD     : std_logic_vector(15 downto 0);
  signal Brdg32DatMISOxD : std_logic_vector(31 downto 0);
  signal Brdg32DatMOSIxD : std_logic_vector(31 downto 0);
  signal Brdg32AckxS     : std_logic;
  signal Brdg32ErrxS     : std_logic;

  -- 8 bit ram interface (sram)
  signal Ram8CycxS     : std_logic;
  signal Ram8StbxS     : std_logic;
  signal Ram8WexS      : std_logic;
  signal Ram8AdrxD     : std_logic_vector(15 downto 0);
  signal Ram8DatMISOxD : std_logic_vector(7 downto 0);
  signal Ram8DatMOSIxD : std_logic_vector(7 downto 0);
  signal Ram8AckxS     : std_logic;

  -- 32 bit ram interface (sram)
  signal Ram32CycxS     : std_logic;
  signal Ram32StbxS     : std_logic;
  signal Ram32WexS      : std_logic;
  signal Ram32SelxS     : std_logic_vector(3 downto 0);
  signal Ram32AdrxD     : std_logic_vector(15 downto 0);
  signal Ram32DatMISOxD : std_logic_vector(31 downto 0);
  signal Ram32DatMOSIxD : std_logic_vector(31 downto 0);
  signal Ram32AckxS     : std_logic;

  -- 32 bit mac master interface
  signal EthMaCycxS            : std_logic;
  signal EthMaStbxS            : std_logic;
  signal EthMaWexS             : std_logic;
  signal EthMaSelxS            : std_logic_vector(3 downto 0);
  signal EthMaAdrxD            : std_logic_vector(31 downto 0);
  signal EthMaDatMISOxD        : std_logic_vector(31 downto 0);
  signal EthMaDatMOSIxD        : std_logic_vector(31 downto 0);
  signal EthMaDatSwappedMISOxD : std_logic_vector(31 downto 0);
  signal EthMaDatSwappedMOSIxD : std_logic_vector(31 downto 0);
  signal EthMaAckxS            : std_logic;
  signal EthMaErrxS            : std_logic;

  -- 32 bit mac slave interface
  signal EthSlCycxS     : std_logic;
  signal EthSlStbxS     : std_logic;
  signal EthSlWexS      : std_logic;
  signal EthSlSelxS     : std_logic_vector(3 downto 0);
  signal EthSlAdrxD     : std_logic_vector(15 downto 0);
  signal EthSlDatMISOxD : std_logic_vector(31 downto 0);
  signal EthSlDatMOSIxD : std_logic_vector(31 downto 0);
  signal EthSlAckxS     : std_logic := '0';
  signal EthSlErrxS     : std_logic := '0';

  -- 32 bit dram dma master interface
  signal DDmaMaCycxS     : std_logic;
  signal DDmaMaStbxS     : std_logic;
  signal DDmaMaWexS      : std_logic;
  signal DDmaMaSelxS     : std_logic_vector(3 downto 0);
  signal DDmaMaAdrxD     : std_logic_vector(31 downto 0);
  signal DDmaMaDatMISOxD : std_logic_vector(31 downto 0);
  signal DDmaMaDatMOSIxD : std_logic_vector(31 downto 0);
  signal DDmaMaAckxS     : std_logic;
  signal DDmaMaErrxS     : std_logic;

  -- 32 bit dram dma slave interface
  signal DDmaSlCycxS     : std_logic;
  signal DDmaSlStbxS     : std_logic;
  signal DDmaSlWexS      : std_logic;
  signal DDmaSlSelxS     : std_logic_vector(3 downto 0);
  signal DDmaSlAdrxD     : std_logic_vector(15 downto 0);
  signal DDmaSlDatMISOxD : std_logic_vector(31 downto 0);
  signal DDmaSlDatMOSIxD : std_logic_vector(31 downto 0);
  signal DDmaSlAckxS     : std_logic := '0';
  signal DDmaSlErrxS     : std_logic := '0';

  signal LaCoreSampleDataxD : std_logic_vector(31 downto 0);
  signal LaCoreSampleWexS   : std_logic;
  signal LaCoreSampleClkxC  : std_logic;

  -- 32 bit compressor slave interface
  signal CmpSlCycxS  : std_logic;
  signal CmpSlStbxS  : std_logic;
  signal CmpSlWexS   : std_logic;
  signal CmpSlSelxD  : std_logic_vector(3 downto 0);
  signal CmpSlAdrxD  : std_logic_vector(15 downto 0);
  signal CmpSlMOSIxD : std_logic_vector(31 downto 0);
  signal CmpSlMISOxD : std_logic_vector(31 downto 0);
  signal CmpSlAckxS  : std_logic;
  signal CmpSlErrxS  : std_logic;

  -- 32 bit compressor master interface (for dma)
  signal CmpMaCycxS  : std_logic;
  signal CmpMaStbxS  : std_logic;
  signal CmpMaWexS   : std_logic;
  signal CmpMaSelxD  : std_logic_vector(3 downto 0);
  signal CmpMaAdrxD  : std_logic_vector(31 downto 0);
  signal CmpMaMOSIxD : std_logic_vector(31 downto 0);
  signal CmpMaMISOxD : std_logic_vector(31 downto 0);
  signal CmpMaAckxS  : std_logic;
  signal CmpMaErrxS  : std_logic;

  
begin

  -- data memory    
  DataMemInst : DataMem
    port map (
      ClkxCI      => ClkxCI,
      ClkxCIB     => ClkxCIB,
      RstxRI      => RstxRI,
      Wb8CycxSI   => Ram8CycxS,
      Wb8StbxSI   => Ram8StbxS,
      Wb8WexSI    => Ram8WexS,
      Wb8AdrxDI   => Ram8AdrxD(13 downto 0),
      Wb8DataxDI  => Ram8DatMOSIxD,
      Wb8DataxDO  => Ram8DatMISOxD,
      Wb8AckxSO   => Ram8AckxS,
      Wb32CycxSI  => Ram32CycxS,
      Wb32StbxSI  => Ram32StbxS,
      Wb32WexSI   => Ram32WexS,
      Wb32SelxSI  => Ram32SelxS,
      Wb32AdrxDI  => Ram32AdrxD(13 downto 2),
      Wb32DataxDI => Ram32DatMOSIxD,
      Wb32DataxDO => Ram32DatMISOxD,
      Wb32AckxSO  => Ram32AckxS
      );   

  -- wb8 -> wb32 bridge
  wbBridgeInst : Wb8Wb32Bridge
    port map (
      ClkxCI     => ClkxCI,
      RstxRI     => RstxRI,
      Wb8CycxSI  => Brdg8CycxS,
      Wb8StbxSI  => Brdg8StbxS,
      Wb8WexSI   => Brdg8WexS,
      Wb8AdrxDI  => Brdg8AdrxD,
      Wb8DatxDI  => Brdg8DatMOSIxD,
      Wb8DatxDO  => Brdg8DatMISOxD,
      Wb8AckxSO  => Brdg8AckxS,
      Wb8ErrxSO  => Brdg8ErrxS,
      Wb32CycxSO => Brdg32CycxS,
      Wb32StbxSO => Brdg32StbxS,
      Wb32WexSO  => Brdg32WexS,
      Wb32SelxSO => Brdg32SelxS,
      Wb32AdrxDO => Brdg32AdrxD,
      Wb32DatxDI => Brdg32DatMISOxD,
      Wb32DatxDO => Brdg32DatMOSIxD,
      Wb32AckxSI => Brdg32AckxS,
      Wb32ErrxSI => Brdg32ErrxS
      );

  -- ethernet MAC
-- genMAC : if SIMULATE_MAC = true or  IS_SIMULATION="FALSE"  generate
  -- note: the mac sends data in big endian mode, this doesn't fit to avr
  -- so we do an endianness swap on the data lines
  EthMaDatSwappedMISOxD <= EthMaDatMISOxD(7 downto 0) & EthMaDatMISOxD(15 downto 8) & EthMaDatMISOxD(23 downto 16) & EthMaDatMISOxD(31 downto 24);
  EthMaDatMOSIxD        <= EthMaDatSwappedMOSIxD(7 downto 0) & EthMaDatSwappedMOSIxD(15 downto 8) & EthMaDatSwappedMOSIxD(23 downto 16) & EthMaDatSwappedMOSIxD(31 downto 24);

  ethMacInst : eth_top
    port map (
      wb_clk_i      => ClkxCI,
      wb_rst_i      => RstxRI,
      -- wishbone slave interface
      wb_dat_i      => EthSlDatMOSIxD,
      wb_dat_o      => EthSlDatMISOxD,
      wb_adr_i      => EthSlAdrxD(11 downto 2),
      wb_sel_i      => EthSlSelxS,
      wb_we_i       => EthSlWexS,
      wb_cyc_i      => EthSlCycxS,
      wb_stb_i      => EthSlStbxS,
      wb_ack_o      => EthSlAckxS,
      wb_err_o      => EthSlErrxS,
      -- WISHBONE master
      m_wb_adr_o    => EthMaAdrxD,
      m_wb_sel_o    => EthMaSelxS,
      m_wb_we_o     => EthMaWexS,
      m_wb_dat_o    => EthMaDatSwappedMOSIxD,
      m_wb_dat_i    => EthMaDatSwappedMISOxD,
      m_wb_cyc_o    => EthMaCycxS,
      m_wb_stb_o    => EthMaStbxS,
      m_wb_ack_i    => EthMaAckxS,
      m_wb_err_i    => EthMaErrxS,
      -- MII TX
      mtx_clk_pad_i => PhyTxClkxCI,
      mtxd_pad_o    => PhyTxdxDO,
      mtxen_pad_o   => PhyTxEnxSO,
      mtxerr_pad_o  => PhyTxErrxSO,
      -- MII RX
      mrx_clk_pad_i => PhyRxClkxCI,
      mrxd_pad_i    => PhyRxdxDI,
      mrxdv_pad_i   => PhyRxDvxSI,
      mrxerr_pad_i  => PhyRxErrxSI,
      mcoll_pad_i   => PhyColxSI,
      mcrs_pad_i    => PhyCrsxSI,
      -- MIIM
      mdc_pad_o     => PhyMdcxSO,
      md_pad_i      => PhyMdioxDI,
      md_pad_o      => PhyMdioxDO,
      md_padoe_o    => PhyMdioEnxSO,
      -- general
      int_o         => EthIntxSO
      );        
 -- end generate genMAC;

--  genDRAMCtrl : if SIMULATE_DRAM = true or IS_SIMULATION="FALSE" generate
  dramCtrl_inst : dramCtrl
    generic map(
      -- defines wheter we simulate or implement the design. This is needed by the MCB rtl
      -- allowed values "TRUE" and "FALSE"
      IS_SIMULATION => IS_SIMULATION,

      -- defines how long one burst (= block transfer) on the wishbone master interface is
      -- data blocks contain WB_BURST_LEN words at maximum. Between two blocks is some time 
      -- (approx. 9 cycles of WbClkxCI) in which data is loaded from the dram. Lower values 
      -- give a very bad bus utilization.
      WB_BURST_LEN => 4                 -- range 2 .. 64 
      )
    port map (
      -- system interface
      ResetSwitchxR => RstButtonxR,
      RstSocxRI     => RstxRI,
      RstCorexRI    => LaCoreRstxRI,
      --RstxRO                  : out STD_LOGIC;        -- system reset output (held high during pll locking)
      CalibDonexSO  => McbReadyxSO,
      -- clocks 
      BoardClkxCI   => BoardClkxCI,
      --BoardClkxCIB    => BoardClkxCIB,
      RefClkxCO     => RefClkxCO,
      -- interface to sampling core
      CoreClkxC     => LaCoreSampleClkxC,
      DataxDI       => LaCoreSampleDataxD,
      WritexSI      => LaCoreSampleWexS,
      -- wishbone dma master interface
      WbClkxCI      => ClkxCI,          -- wishbone clock (master & slave)
      MaCycxSO      => DDmaMaCycxS,
      MaStbxSO      => DDmaMaStbxS,
      MaWexSO       => DDmaMaWexS,
      MaSelxDO      => DDmaMaSelxS,
      MaAdrxDO      => DDmaMaAdrxD,
      MaDatxDO      => DDmaMaDatMOSIxD,
      MaDatxDI      => DDmaMaDatMISOxD,
      MaAckxSI      => DDmaMaAckxS,
      MaErrxSI      => DDmaMaErrxS,
      -- wishbone configuration slave interface
      SlCycxSI      => DDmaSlCycxS,
      SlStbxSI      => DDmaSlStbxS,
      SlWexSI       => DDmaSlWexS,
      SlAdrxDI      => DDmaSlAdrxD(4 downto 2),
      SlDatxDI      => DDmaSlDatMOSIxD,
      SlDatxDO      => DDmaSlDatMISOxD,
      SlAckxSO      => DDmaSlAckxS,
      IntxSO        => DDmaIntxSO,
      -- physical memory interface
      MemDQxDIO     => MemDQxDIO,
      MemAdrxDO     => MemAdrxDO,
      MemBaxDO      => MemBaxDO,
      MemRasxEO     => MemRasxEO,
      MemCasxEO     => MemCasxEO,
      MemWexEO      => MemWexEO,
      MemOdtxO      => MemOdtxO,
      MemCkexEO     => MemCkexEO,
      MemClkxCO     => MemClkxCO,
      MemClkxCOB    => MemClkxCOB,
      MemLDqsxEO    => MemLDqsxEO,
      MemLDqsxEOB   => MemLDqsxEOB,
      MemUDqsxEO    => MemUDqsxEO,
      MemUDqsxEOB   => MemUDqsxEOB,
      MemLdmxEO     => MemLdmxEO,
      MemUdmxEO     => MemUdmxEO,
      -- calibration (not actually logical signals)
      RzqxIO        => RzqxIO,
      ZioxIO        => ZioxIO
      );
  -- end generate genDRAMCtrl;

  laInst : la
    port map(
      reset                => LaCoreRstxRI,
      clock                => LaCoreClkxCI,  -- LA base clock, 200MHz
      clockB               => LaCoreClkxCBI,
      AVR_Clock            => ClkxCI,
      AVR_Reset            => RstxRI,
      exClock              => exClockxCI,
      input                => inputChxDI,
      testOutput           => testOutputxSO,
      -- interface to sample memory
      memoryClkxCO         => LaCoreSampleClkxC,
      memoryOut            => LaCoreSampleDataxD,
      memoryWrite          => LaCoreSampleWexS,
      -- Wishbone Interface for configuration
      Wb_cycxSI            => LaCoreCycxS,
      Wb_stbxSI            => LaCoreStbxS,
      Wb_wexSI             => LaCoreWexS,
      Wb_ackxSO            => LaCoreAckxS,
      Wb_errxSO            => LaCoreErrxS,
      Wb_adrxDI            => LaCoreAdrxD(3 downto 0),
      Wb_datxDI            => LaCoreDatMOSIxD,
      Wb_datxDO            => LaCoreDatMISOxD,
      --AVR-Interrupt:
      sample_finish_IRQxSO => LaCoreIRQxSO,  --interrupt-output-flag for the cpu (finished sampling)
      sample_clrflagxSI    => LaCoreIntAckxSI  --input to clear the interrupt flag
      );


  -- data compression unit
  CompressorInst : CompressorTop
    port map (
      ClkxCI   => ClkxCI,
      RstxRI   => RstxRI,
      SlCycxSI => CmpSlCycxS,
      SlStbxSI => CmpSlStbxS,
      SlWexSI  => CmpSlWexS,
      SlSelxDI => CmpSlSelxD,
      SlAdrxDI => CmpSlAdrxD(4 downto 2),
      SlDatxDI => CmpSlMOSIxD,
      SlDatxDO => CmpSlMISOxD,
      SlAckxSO => CmpSlAckxS,
      SlErrxSO => CmpSlErrxS,
      IntxSO   => CmpIntxSO,
      MaCycxSO => CmpMaCycxS,
      MaStbxSO => CmpMaStbxS,
      MaWexSO  => CmpMaWexS,
      MaSelxDO => CmpMaSelxD,
      MaAdrxDO => CmpMaAdrxD,
      MaDatxDO => CmpMaMOSIxD,
      MaDatxDI => CmpMaMISOxD,
      MaAckxSI => CmpMaAckxS,
      MaErrxSI => CmpMaErrxS);


  -- 8 bit wishbone bus
  Wb8DataBus : wb_conbus_top
    generic map (
      dw         => 8,
      aw         => 16,
      s0_addr_w  => 2,  -- how many bits are decoded to address the slave 0 (sram)
      s0_addr    => 0,  -- higher bits of sram address (sram is in the lower 32kB)        
      s1_addr_w  => 1,                  -- slave 1: wb8 to wb32 bridge 
      s1_addr    => 1,  -- 32 bit bus maps to higher 32kB address space                         
      s27_addr_w => 2,
      s2_addr    => 1,                  -- la hardware cfg interface
      s3_addr    => 2,                  -- others: unused 
      s4_addr    => 3,
      s5_addr    => 3,
      s6_addr    => 3,
      s7_addr    => 3)
    port map (
      clk_i    => ClkxCI,
      rst_i    => RstxRI,
      -- Master 0 Interface
      m0_dat_i => CpuDatxDI,
      m0_dat_o => CpuDatxDO,
      m0_adr_i => CpuAdrxDI,
      m0_sel_i => "1",                  -- dummy signal, no use in 8 bit bus
      m0_we_i  => CpuWexSI,
      m0_cyc_i => CpuCycxSI,
      m0_stb_i => CpuStbxSI,
      m0_ack_o => CpuAckxSO,
      m0_err_o => CpuErrxSO,
      m0_rty_o => CpuRtyxSO,
      m0_cab_i => '0',
      -- Master 1 Interface
      m1_dat_i => x"00",
      m1_dat_o => open,
      m1_adr_i => x"0000",
      m1_sel_i => "0",
      m1_we_i  => '0',
      m1_cyc_i => '0',
      m1_stb_i => '0',
      m1_ack_o => open,
      m1_err_o => open,
      m1_rty_o => open,
      m1_cab_i => '0',
      -- Master 2 Interface
      m2_dat_i => x"00",
      m2_dat_o => open,
      m2_adr_i => x"0000",
      m2_sel_i => "0",
      m2_we_i  => '0',
      m2_cyc_i => '0',
      m2_stb_i => '0',
      m2_ack_o => open,
      m2_err_o => open,
      m2_rty_o => open,
      m2_cab_i => '0',
      -- Master 3 Interface
      m3_dat_i => x"00",
      m3_dat_o => open,
      m3_adr_i => x"0000",
      m3_sel_i => "0",
      m3_we_i  => '0',
      m3_cyc_i => '0',
      m3_stb_i => '0',
      m3_ack_o => open,
      m3_err_o => open,
      m3_rty_o => open,
      m3_cab_i => '0',
      -- Master 4 Interface
      m4_dat_i => x"00",
      m4_dat_o => open,
      m4_adr_i => x"0000",
      m4_sel_i => "0",
      m4_we_i  => '0',
      m4_cyc_i => '0',
      m4_stb_i => '0',
      m4_ack_o => open,
      m4_err_o => open,
      m4_rty_o => open,
      m4_cab_i => '0',
      -- Master 5 Interface
      m5_dat_i => x"00",
      m5_dat_o => open,
      m5_adr_i => x"0000",
      m5_sel_i => "0",
      m5_we_i  => '0',
      m5_cyc_i => '0',
      m5_stb_i => '0',
      m5_ack_o => open,
      m5_err_o => open,
      m5_rty_o => open,
      m5_cab_i => '0',
      -- Master 6 Interface
      m6_dat_i => x"00",
      m6_dat_o => open,
      m6_adr_i => x"0000",
      m6_sel_i => "0",
      m6_we_i  => '0',
      m6_cyc_i => '0',
      m6_stb_i => '0',
      m6_ack_o => open,
      m6_err_o => open,
      m6_rty_o => open,
      m6_cab_i => '0',
      -- Master 7 Interface
      m7_dat_i => x"00",
      m7_dat_o => open,
      m7_adr_i => x"0000",
      m7_sel_i => "0",
      m7_we_i  => '0',
      m7_cyc_i => '0',
      m7_stb_i => '0',
      m7_ack_o => open,
      m7_err_o => open,
      m7_rty_o => open,
      m7_cab_i => '0',
      --        Slave 0 Interface - SRAM       
      s0_dat_i => Ram8DatMISOxD,
      s0_dat_o => Ram8DatMOSIxD,
      s0_adr_o => Ram8AdrxD,
      s0_sel_o => open,
      s0_we_o  => Ram8WexS,
      s0_cyc_o => Ram8CycxS,
      s0_stb_o => Ram8StbxS,
      s0_ack_i => Ram8AckxS,
      s0_err_i => '0',
      s0_rty_i => '0',
      s0_cab_o => open,
      --        Slave 1 Interface - Wb8-Wb32 bridge 
      s1_dat_i => Brdg8DatMISOxD,
      s1_dat_o => Brdg8DatMOSIxD,
      s1_adr_o => Brdg8AdrxD,
      s1_sel_o => open,
      s1_we_o  => Brdg8WexS,
      s1_cyc_o => Brdg8CycxS,
      s1_stb_o => Brdg8StbxS,
      s1_ack_i => Brdg8AckxS,
      s1_err_i => Brdg8ErrxS,
      s1_rty_i => '0',
      s1_cab_o => open,
      --        Slave 2 Interface 
      s2_dat_i => LaCoreDatMISOxD,
      s2_dat_o => LaCoreDatMOSIxD,
      s2_adr_o => LaCoreAdrxD,
      s2_sel_o => open,
      s2_we_o  => LaCoreWexS,
      s2_cyc_o => LaCoreCycxS,
      s2_stb_o => LaCoreStbxS,
      s2_ack_i => LaCoreAckxS,
      s2_err_i => LaCoreErrxS,
      s2_rty_i => '0',
      s2_cab_o => open,
      --        Slave 3 Interface - unused
      s3_dat_i => x"00",
      s3_dat_o => open,
      s3_adr_o => open,
      s3_sel_o => open,
      s3_we_o  => open,
      s3_cyc_o => open,
      s3_stb_o => open,
      s3_ack_i => '0',
      s3_err_i => '0',
      s3_rty_i => '0',
      s3_cab_o => open,
      --        Slave 1 Interface - unused
      s4_dat_i => x"00",
      s4_dat_o => open,
      s4_adr_o => open,
      s4_sel_o => open,
      s4_we_o  => open,
      s4_cyc_o => open,
      s4_stb_o => open,
      s4_ack_i => '0',
      s4_err_i => '0',
      s4_rty_i => '0',
      s4_cab_o => open,
      --        Slave 1 Interface - unused
      s5_dat_i => x"00",
      s5_dat_o => open,
      s5_adr_o => open,
      s5_sel_o => open,
      s5_we_o  => open,
      s5_cyc_o => open,
      s5_stb_o => open,
      s5_ack_i => '0',
      s5_err_i => '0',
      s5_rty_i => '0',
      s5_cab_o => open,
      --        Slave 1 Interface - unused
      s6_dat_i => x"00",
      s6_dat_o => open,
      s6_adr_o => open,
      s6_sel_o => open,
      s6_we_o  => open,
      s6_cyc_o => open,
      s6_stb_o => open,
      s6_ack_i => '0',
      s6_err_i => '0',
      s6_rty_i => '0',
      s6_cab_o => open,
      --        Slave 1 Interface - unused
      s7_dat_i => x"00",
      s7_dat_o => open,
      s7_adr_o => open,
      s7_sel_o => open,
      s7_we_o  => open,
      s7_cyc_o => open,
      s7_stb_o => open,
      s7_ack_i => '0',
      s7_err_i => '0',
      s7_rty_i => '0',
      s7_cab_o => open
      );           

  -- 32 bit wishbone bus
  Wb32DataBus : wb_conbus_top
    generic map (
      dw         => 32,
      aw         => 16,
      s0_addr_w  => 2,  -- how many bits are decoded to address the slave 0 (sram)
      s0_addr    => 0,  -- higher bits of sram address (sram is in the lower 32kB)        
      s1_addr_w  => 3,                  -- unused
      s1_addr    => 7,
      s27_addr_w => 3,
      s2_addr    => 4,                  -- eth mac slave port (configuration)
      s3_addr    => 5,                  -- dram dma interface (configuraiton)  
      s4_addr    => 6,                  -- data compressor (configuration)
      s5_addr    => 7,
      s6_addr    => 7,
      s7_addr    => 7)
    port map (
      clk_i    => ClkxCI,
      rst_i    => RstxRI,
      -- Master 0 Interface - cpu (over wb8-wb32 bridge)
      m0_dat_i => Brdg32DatMOSIxD,
      m0_dat_o => Brdg32DatMISOxD,
      m0_adr_i => Brdg32AdrxD,
      m0_sel_i => Brdg32SelxS,
      m0_we_i  => Brdg32WexS,
      m0_cyc_i => Brdg32CycxS,
      m0_stb_i => Brdg32StbxS,
      m0_ack_o => Brdg32AckxS,
      m0_err_o => Brdg32ErrxS,
      m0_rty_o => open,
      m0_cab_i => '0',
      -- Master 1 Interface - eth mac
      m1_dat_i => EthMaDatMOSIxD,
      m1_dat_o => EthMaDatMISOxD,
      m1_adr_i => EthMaAdrxD(15 downto 0),
      m1_sel_i => EthMaSelxS,
      m1_we_i  => EthMaWexS,
      m1_cyc_i => EthMaCycxS,
      m1_stb_i => EthMaStbxS,
      m1_ack_o => EthMaAckxS,
      m1_err_o => EthMaErrxS,
      m1_rty_o => open,
      m1_cab_i => '0',
      -- Master 2 Interface - dram dma 
      m2_dat_i => DDmaMaDatMOSIxD,
      m2_dat_o => DDmaMaDatMISOxD,
      m2_adr_i => DDmaMaAdrxD(15 downto 0),
      m2_sel_i => DDmaMaSelxS,
      m2_we_i  => DDmaMaWexS,
      m2_cyc_i => DDmaMaCycxS,
      m2_stb_i => DDmaMaStbxS,
      m2_ack_o => DDmaMaAckxS,
      m2_err_o => DDmaMaErrxS,
      m2_rty_o => open,
      m2_cab_i => '0',
      -- Master 3 Interface - data compressor dma
      m3_dat_i => CmpMaMOSIxD,
      m3_dat_o => CmpMaMISOxD,
      m3_adr_i => CmpMaAdrxD(15 downto 0),
      m3_sel_i => CmpMaSelxD,
      m3_we_i  => CmpMaWexS,
      m3_cyc_i => CmpMaCycxS,
      m3_stb_i => CmpMaStbxS,
      m3_ack_o => CmpMaAckxS,
      m3_err_o => CmpMaErrxS,
      m3_rty_o => open,
      m3_cab_i => '0',
      -- Master 4 Interface
      m4_dat_i => x"00000000",
      m4_dat_o => open,
      m4_adr_i => x"0000",
      m4_sel_i => "0000",
      m4_we_i  => '0',
      m4_cyc_i => '0',
      m4_stb_i => '0',
      m4_ack_o => open,
      m4_err_o => open,
      m4_rty_o => open,
      m4_cab_i => '0',
      -- Master 5 Interface
      m5_dat_i => x"00000000",
      m5_dat_o => open,
      m5_adr_i => x"0000",
      m5_sel_i => "0000",
      m5_we_i  => '0',
      m5_cyc_i => '0',
      m5_stb_i => '0',
      m5_ack_o => open,
      m5_err_o => open,
      m5_rty_o => open,
      m5_cab_i => '0',
      -- Master 6 Interface
      m6_dat_i => x"00000000",
      m6_dat_o => open,
      m6_adr_i => x"0000",
      m6_sel_i => "0000",
      m6_we_i  => '0',
      m6_cyc_i => '0',
      m6_stb_i => '0',
      m6_ack_o => open,
      m6_err_o => open,
      m6_rty_o => open,
      m6_cab_i => '0',
      -- Master 7 Interface
      m7_dat_i => x"00000000",
      m7_dat_o => open,
      m7_adr_i => x"0000",
      m7_sel_i => "0000",
      m7_we_i  => '0',
      m7_cyc_i => '0',
      m7_stb_i => '0',
      m7_ack_o => open,
      m7_err_o => open,
      m7_rty_o => open,
      m7_cab_i => '0',
      --        Slave 0 Interface - SRAM       
      s0_dat_i => Ram32DatMISOxD,
      s0_dat_o => Ram32DatMOSIxD,
      s0_adr_o => Ram32AdrxD,
      s0_sel_o => Ram32SelxS,
      s0_we_o  => Ram32WexS,
      s0_cyc_o => Ram32CycxS,
      s0_stb_o => Ram32StbxS,
      s0_ack_i => Ram32AckxS,
      s0_err_i => '0',
      s0_rty_i => '0',
      s0_cab_o => open,
      --        Slave 1 Interface  
      s1_dat_i => x"00000000",
      s1_dat_o => open,
      s1_adr_o => open,
      s1_sel_o => open,
      s1_we_o  => open,
      s1_cyc_o => open,
      s1_stb_o => open,
      s1_ack_i => '0',
      s1_err_i => '0',
      s1_rty_i => '0',
      s1_cab_o => open,
      --        Slave 2 Interface 
      s2_dat_i => EthSlDatMISOxD,
      s2_dat_o => EthSlDatMOSIxD,
      s2_adr_o => EthSlAdrxD,
      s2_sel_o => EthSlSelxS,
      s2_we_o  => EthSlWexS,
      s2_cyc_o => EthSlCycxS,
      s2_stb_o => EthSlStbxS,
      s2_ack_i => EthSlAckxS,
      s2_err_i => EthSlErrxS,
      s2_rty_i => '0',
      s2_cab_o => open,
      --        Slave 3 Interface
      s3_dat_i => DDmaSlDatMISOxD,
      s3_dat_o => DDmaSlDatMOSIxD,
      s3_adr_o => DDmaSlAdrxD,
      s3_sel_o => DDmaSlSelxS,
      s3_we_o  => DDmaSlWexS,
      s3_cyc_o => DDmaSlCycxS,
      s3_stb_o => DDmaSlStbxS,
      s3_ack_i => DDmaSlAckxS,
      s3_err_i => '0',
      s3_rty_i => '0',
      s3_cab_o => open,
      --        Slave 4 Interface 
      s4_dat_i => CmpSlMISOxD,
      s4_dat_o => CmpSlMOSIxD,
      s4_adr_o => CmpSlAdrxD,
      s4_sel_o => CmpSlSelxD,
      s4_we_o  => CmpSlWexS,
      s4_cyc_o => CmpSlCycxS,
      s4_stb_o => CmpSlStbxS,
      s4_ack_i => CmpSlAckxS,
      s4_err_i => CmpSlErrxS,
      s4_rty_i => '0',
      s4_cab_o => open,
      --        Slave 5 Interface - unused
      s5_dat_i => x"00000000",
      s5_dat_o => open,
      s5_adr_o => open,
      s5_sel_o => open,
      s5_we_o  => open,
      s5_cyc_o => open,
      s5_stb_o => open,
      s5_ack_i => '0',
      s5_err_i => '0',
      s5_rty_i => '0',
      s5_cab_o => open,
      --        Slave 6 Interface - unused
      s6_dat_i => x"00000000",
      s6_dat_o => open,
      s6_adr_o => open,
      s6_sel_o => open,
      s6_we_o  => open,
      s6_cyc_o => open,
      s6_stb_o => open,
      s6_ack_i => '0',
      s6_err_i => '0',
      s6_rty_i => '0',
      s6_cab_o => open,
      --        Slave 7 Interface - unused
      s7_dat_i => x"00000000",
      s7_dat_o => open,
      s7_adr_o => open,
      s7_sel_o => open,
      s7_we_o  => open,
      s7_cyc_o => open,
      s7_stb_o => open,
      s7_ack_i => '0',
      s7_err_i => '0',
      s7_rty_i => '0',
      s7_cab_o => open
      );           
end Behavioral;

