--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* Generates the clock signals needed by the analyzer. It selectes internal or
--* external clock (according to a config flag) and provides the deskewed clock
--* as well as a 180 degree phase shifted version of the clock to the core.
--*
--***************************************************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.vcomponents.all;



entity coreClkMgr is
  
  port (
    IntClkxCI     : in  std_logic;      -- internal sampling clock (200MHz)
    IntClkxCBI    : in  std_logic;
    ExtClkxCI     : in  std_logic;      -- external sampling clock (fpga pin)
    DcmLockedxSO  : out std_logic;
    ResetxRI      : in  std_logic;      -- external hardware reset
    FlagExtClkxSI : in  std_logic;  -- select between external (1) and internal sampling clk
    SampleClkxCO  : out std_logic;      -- sample clock for the core
    SampleClkxCBO : out std_logic);     -- inverted version of the sample clock

end coreClkMgr;

architecture Behavioral of coreClkMgr is

  -- constant RST_PULS_LEN : integer := 8;  -- defines the length of the reset pulse on reset output (in clock cycles)

  signal dcmClkxC, dcmClkxCB       : std_logic;
  signal dcmClk0xC                 : std_logic;
  signal ibufgOutxC                : std_logic;
  --signal dcmClkDvxC                   : std_logic;
  signal lockedxS                  : std_logic;
  signal clkSelxS                  : std_logic;
  signal statusxD                  : std_logic_vector(7 downto 0);
  signal dcmRefClkxC               : std_logic;
  signal dcmFeedBackxC             : std_logic_vector(1 downto 0);
  signal dcmRstxR                  : std_logic;
  signal SampleClkxC, SampleClkxCB : std_logic;
  --signal rstShiftRegxDN, rstShiftRegxDP : std_logic_vector(RST_PULS_LEN-1 downto 0) := (others => '1');

  
begin

  -- switching to external clock is possible only if the dcm has locked
  clkSelxS <= FlagExtClkxSI and lockedxS;

  -- select between internal and external sampling clock
  selClkSrc : BUFGMUX
    port map (
      O  => SampleClkxC,                -- Clock MUX output
      I0 => IntClkxCI,                  -- Clock0 input
      I1 => dcmClkxC,                   -- Clock1 input
      S  => clkSelxS                    -- Clock select input
      );
  selClkSrcB : BUFGMUX
    port map (
      O  => SampleClkxCB,               -- Clock MUX output
      I0 => IntClkxCBI,                 -- Clock0 input
      I1 => dcmClkxCB,                  -- Clock1 input
      S  => clkSelxS                    -- Clock select input
      );


  -- input clock buffer for external clock
  -- this is the implementation recomended by xilinx in ug382, page 34
  extClk_IBUFG_inst : IBUFG
    generic map (
      IBUF_LOW_PWR => false,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD   => "DEFAULT")
    port map (
      O => ibufgOutxC,                  -- Clock buffer output
      I => ExtClkxCI  -- Clock buffer input (connect directly to top-level port)
      );

  extClk_BUFIO2_inst : BUFIO2
    generic map (
      DIVIDE        => 1,               -- DIVCLK divider (1-8)
      DIVIDE_BYPASS => true,  -- Bypass the divider circuitry (TRUE/FALSE)
      I_INVERT      => false,           -- Invert clock (TRUE/FALSE)
      USE_DOUBLER   => false            -- Use doubler circuitry (TRUE/FALSE)
      )
    port map (
      DIVCLK       => dcmRefClkxC,      -- 1-bit Divided clock output
      IOCLK        => open,             -- 1-bit I/O output clock
      SERDESSTROBE => open,  -- 1-bit Output SERDES strobe (connect to ISERDES2/OSERDES2)
      I            => ibufgOutxC        -- 1-bit Clock input (connect to IBUFG)
      );

  -- reset the DCM when requested by reset input or when it looses lock
  -- this is the circuit proposed in ug382 by xilinx
  dcmRstxR <= ResetxRI or ((not lockedxS) and statusxD(2));


  -- A DCM is used to deskew, dejitter and dutycycle correct the external
  -- sample clock
  DCM_SP_inst : DCM_SP
    generic map (
      CLKDV_DIVIDE          => 2.0,     -- CLKDV divide value
      -- (1.5,2,2.5,3,3.5,4,4.5,5,5.5,6,6.5,7,7.5,8,9,10,11,12,13,14,15,16).
      CLKFX_DIVIDE          => 2,  -- Divide value on CLKFX outputs - D - (1-32)
      CLKFX_MULTIPLY        => 2,  -- Multiply value on CLKFX outputs - M - (2-32)
      CLKIN_DIVIDE_BY_2     => false,   -- CLKIN divide by two (TRUE/FALSE)
      CLKIN_PERIOD          => 5.0,     -- Input clock period specified in nS
      CLKOUT_PHASE_SHIFT    => "NONE",  -- Output phase shift (NONE, FIXED, VARIABLE)
      CLK_FEEDBACK          => "1X",    -- Feedback source (NONE, 1X, 2X)
      DESKEW_ADJUST         => "SYSTEM_SYNCHRONOUS",  -- SYSTEM_SYNCHRNOUS or SOURCE_SYNCHRONOUS
      DFS_FREQUENCY_MODE    => "LOW",   -- Unsupported - Do not change value
      DLL_FREQUENCY_MODE    => "LOW",   -- Unsupported - Do not change value
      DSS_MODE              => "NONE",  -- Unsupported - Do not change value
      DUTY_CYCLE_CORRECTION => true,    -- Unsupported - Do not change value
      FACTORY_JF            => X"c080",  -- Unsupported - Do not change value
      PHASE_SHIFT           => 0,  -- Amount of fixed phase shift (-255 to 255)
      STARTUP_WAIT          => false  -- Delay config DONE until DCM_SP LOCKED (TRUE/FALSE)
      )
    port map (
      CLK0     => dcmClk0xC,  -- 1-bit 0 degree clock output, used for feedback
      CLK180   => open,                 -- 1-bit 180 degree clock output
      CLK270   => open,                 -- 1-bit 270 degree clock output
      CLK2X    => open,  -- 1-bit 2X clock frequency clock output
      CLK2X180 => open,  -- 1-bit 2X clock frequency, 180 degree clock output
      CLK90    => open,                 -- 1-bit 90 degree clock output
      CLKDV    => open,                 -- 1-bit Divided clock output
      CLKFX    => dcmClkxC,  -- 1-bit Digital Frequency Synthesizer output (DFS)
      CLKFX180 => dcmClkxCB,            -- 1-bit 180 degree CLKFX output
      LOCKED   => lockedxS,             -- 1-bit DCM_SP Lock Output
      PSDONE   => open,                 -- 1-bit Phase shift done output
      STATUS   => statusxD,             -- 8-bit DCM_SP status output
      CLKFB    => dcmFeedBackxC(1),     -- 1-bit Clock feedback input
      CLKIN    => dcmRefClkxC,          -- 1-bit Clock input
      DSSEN    => '0',                  -- 1-bit Unsupported, specify to GND.
      PSCLK    => '0',                  -- 1-bit Phase shift clock input
      PSEN     => '0',                  -- 1-bit Phase shift enable
      PSINCDEC => '0',   -- 1-bit Phase shift increment/decrement input
      RST      => dcmRstxR              -- 1-bit Active high reset input
      );

  -- generate the reset output signal
--  rstShiftRegxDN <= '0' & rstShiftRegxDP(RST_PULS_LEN-1 downto 1);
--  ResetxRO       <= rstShiftRegxDP(0);

--  rstShiftReg : process (SampleClkxC, lockedxS)
--  begin
--    if lockedxS = '0' then
--      rstShiftRegxDP <= (others => '1');
--    elsif SampleClkxC'event and SampleClkxC = '1' then  -- rising clock edge
--      rstShiftRegxDP <= rstShiftRegxDN;
--    end if;
--  end process rstShiftReg;

  -- two clock buffers are inserted in the feedback loop of the dcm
  -- to compensate the input buffer and the clock multiplexer.
  -- This allows better alignment of core clock and external sampling clock.
  feedClkBuf0 : BUFG
    port map (
      O => dcmFeedBackxC(0),
      I => dcmClk0xC);

  feedClkBuf1 : BUFIO2FB
    generic map (
      DIVIDE_BYPASS => true             -- Bypass divider (TRUE/FALSE)
      )
    port map (
      O => dcmFeedBackxC(1),  -- 1-bit Output feedback clock (connect to feedback input of DCM/PLL)
      I => dcmFeedBackxC(0)  -- 1-bit Feedback clock input (connect to input port)
      );

  SampleClkxCO  <= SampleClkxC;
  SampleClkxCBO <= SampleClkxCB;
  
end Behavioral;
