--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* This module samples the input channels and applies a programmable clock
--* divider. The sampling rate r is given as:
--*
--*     r = f / (1 + d)
--*
--* where f is the input clock frequency and d is the value of the divider register
--*
--* NAMING CONVETION: http://www.dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity sampler is
  generic (
    CLOCK_DIV_WIDTH : integer := 24);  -- length of the clock divider in bits (up to 48)
  port (
    ClkxCI         : in  std_logic;     -- sampling clock
    RstxRI         : in  std_logic;
    RunxSI         : in  std_logic;
    samplexDI      : in  std_logic_vector (31 downto 0);  -- 32 input channels, from the syncer
    cfgDataxDI     : in  std_logic_vector (CLOCK_DIV_WIDTH-1 downto 0);  -- configuration data
    wrDividerxSI   : in  std_logic;  -- write divider register, from decoder, enables dividing
    samplexDO      : out std_logic_vector (31 downto 0);  -- sampled data, goes to controller and trigger.
    sampleValidxSO : out std_logic;  -- new sample ready, to controller and trigger
    sampleRatexSO  : out std_logic);  -- low rate sample signal with 50% duty cycle, leaves fpga
end sampler;

architecture Behavioral of sampler is

  signal matchxS                        : std_logic;
  signal dividerxDN, dividerxDP         : std_logic_vector(CLOCK_DIV_WIDTH-1 downto 0) := (others => '0');
  signal sampleValidxSN, sampleValidxSP : std_logic                                    := '0';
  signal samplexDN, samplexDP           : std_logic_vector(31 downto 0);
  signal sampleRatexSN, sampleRatexSP   : std_logic                                    := '0';

  -- signals for DSP slice counter
  signal counterxD : std_logic_vector (CLOCK_DIV_WIDTH-1 downto 0);
  signal incCntxS  : std_logic;
  signal clrCntxS  : std_logic;
--  signal dspPxD    : std_logic_vector (47 downto 0);

  signal counterxDN, counterxDP : unsigned(CLOCK_DIV_WIDTH-1 downto 0) := (others => '0');
  
begin

  -- write the divider register if the flag is set to 1
  dividerxDN <= cfgDataxDI when wrDividerxSI = '1' else dividerxDP;

  -- compare counter and divider register
  matchxS <= '1' when dividerxDP = counterxD else '0';

  -- reset the counter when a match occures
  clrCntxS <= matchxS or RstxRI;

  -- we always increment the counter (reset has higher priority anyway)
  incCntxS <= RunxSI;

  -- add a register in the sample data path (to improve timing)
  samplexDN      <= samplexDI;
  sampleValidxSN <= matchxS and RunxSI;

  -- if the sample rate is one half of Clk or less we output the sampling rate
  process (counterxD, dividerxDP, matchxS, sampleRatexSP)
  begin
    sampleRatexSN <= sampleRatexSP;
    if matchxS = '1' then
      sampleRatexSN <= '1';
    end if;
    if counterxD(CLOCK_DIV_WIDTH-2 downto 0) = dividerxDP(CLOCK_DIV_WIDTH-1 downto 1) then
      sampleRatexSN <= '0';
    end if;
  end process;

  sampleRatexSO  <= sampleRatexSP;
  samplexDO      <= samplexDP;
  sampleValidxSO <= sampleValidxSP;

  regs : process (ClkxCI)
  begin
    if ClkxCI'event and ClkxCI = '1' then  -- rising clock edge
      if RstxRI = '1' then
        dividerxDP     <= (others => '0');
        sampleValidxSP <= '0';
        samplexDP      <= (others => '0');
        sampleRatexSP  <= '0';
      else
        dividerxDP     <= dividerxDN;
        sampleValidxSP <= sampleValidxSN;
        samplexDP      <= samplexDN;
        sampleRatexSP  <= sampleRatexSN;
      end if;
    end if;
  end process regs;

  -- use a dsp slice as fast counter

  counterxD <= std_logic_vector(counterxDP);

  cnt : process (clrCntxS, counterxDP, incCntxS)
  begin  -- process cnt
    counterxDN <= counterxDP;
   -- if clrCntxS = '1' then
    --  counterxDN <= (others => '0');
    --else
      if incCntxS = '1' then
        counterxDN <= counterxDP + 1;
      end if;
    --end if;
  end process cnt;

  process (ClkxCI)
  begin
    if ClkxCI'event and ClkxCI = '1' then
      if clrCntxS = '1' then
        counterxDP <= (others => '0');
      else
        counterxDP <= counterxDN;
      end if;
    end if;
  end process;

end Behavioral;

